//-----------------------------------------------------------------------------
// Module "CMemory"
//-----------------------------------------------------------------------------
// Purpose : Library to simplify memory management like GlobalLock,
//           GlobalUnlock, ... of the windows API.
//-----------------------------------------------------------------------------

#include <stdafx.h>
#include <libgege\cmemory.h>
//-----------------------------------------------------------------------------

//%%SrCtl:CMemory%%

CMemory::CMemory(DWORD dwDataSize) {

  //---- Allocate data
	hData = GlobalAlloc(GMEM_MOVEABLE, dwDataSize);
	if (hData == NULL) {
	}
	else {
		dwAllocatedSize = dwDataSize;
		dwOccupedSize = dwDataSize;
		Locked = FALSE;
  }
}

CMemory::CMemory() {

	hData = NULL;
	dwAllocatedSize = 0;
	dwOccupedSize = 0;
	Locked = FALSE;
}



//-----------------------------------------------------------------------------

CMemory::~CMemory() {
  //---- Unlock data if locked
	if (Locked)
    ReleaseDataPtr();

  //---- Free memory
  	if	(hData!=NULL) {
		GlobalFree(hData);
	}
}

BOOL	CMemory::Free	()
	{
	if	(Locked)
		ReleaseDataPtr();
	if	(hData!=NULL)
		GlobalFree(hData);
	hData = NULL;
	Locked=FALSE;
	return TRUE;
	}
//-----------------------------------------------------------------------------

void FAR *CMemory::GetDataPtr() {
	Locked = TRUE;
	return GlobalLock(hData);
}

//-----------------------------------------------------------------------------

BOOL CMemory::ReleaseDataPtr() {
	if (!Locked)
		return FALSE;
	Locked = FALSE;
	return !GlobalUnlock(hData);
}

//-----------------------------------------------------------------------------

BOOL CMemory::Realloc(DWORD dwNewSize) {

  //---- Reallocate memory
    if (hData==NULL)
		hData = GlobalAlloc(GMEM_MOVEABLE, dwNewSize);
    else	
		hData = GlobalReAlloc(hData, dwNewSize, GMEM_MOVEABLE);
	if (hData == NULL)
		return FALSE;
	else {
		dwAllocatedSize = dwNewSize;
		dwOccupedSize = dwNewSize;
		return TRUE;
  }
}

//-----------------------------------------------------------------------------

DWORD CMemory::GetDataSize() {
	return dwOccupedSize;
}

//-----------------------------------------------------------------------------

void CMemory::SetDataSize(DWORD dwDataSize) {
	dwOccupedSize = dwDataSize;
}

//-----------------------------------------------------------------------------

HGLOBAL CMemory::GetHandle() {
  return hData;
}

BOOL	CMemory::IsLocked()	{
  return Locked;
}

//%%EndSrCtl:CMemory%%

