/*
 *   FILE: Deadman.c
 *            Copyright (c) 1994, Joseph Hlavaty
 *            This product is CAREWARE.
 *            Please refer to the licensing agreement LICENSE.AGR on
 *            the diskette.
 *
 *   PURPOSE: Tests fault handling...
*/

#include "windows.h"
#include "deadman.h"

#include <string.h>

HANDLE hInst;

HWND hwnd;                    /* handle to main windows  */

/****************************************************************************

    FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)

    PURPOSE: calls initialization function, processes message loop

****************************************************************************/

int PASCAL WinMain(hInstance, hPrevInstance, lpCmdLine, nCmdShow)
HANDLE hInstance;
HANDLE hPrevInstance;
LPSTR lpCmdLine;
int nCmdShow;
{
    MSG msg;

    if (!hPrevInstance)
        if (!InitApplication(hInstance))
            return (FALSE);

    if (!InitInstance(hInstance, nCmdShow))
        return (FALSE);

    while (GetMessage(&msg, NULL, NULL, NULL))
    {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }
    return (msg.wParam);
}



BOOL InitApplication(hInstance)
HANDLE hInstance;
{
    WNDCLASS  wc;

    wc.style = NULL;
    wc.lpfnWndProc = MainWndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName =  "DeadManMenu";
    wc.lpszClassName = "DeadManWClass";

    return (RegisterClass(&wc));
}


BOOL InitInstance(hInstance, nCmdShow)
    HANDLE          hInstance;
    int             nCmdShow;
{
    hInst = hInstance;

    hwnd = CreateWindow(
        "DeadManWClass",
        "DeadMan",
        WS_OVERLAPPEDWINDOW,
        50,
        400,
        50 ,
        200,
        NULL,
        NULL,
        hInstance,
        NULL
    );

    if (!hwnd)
        return (FALSE);

    ShowWindow(hwnd, nCmdShow);
    UpdateWindow(hwnd);
    return (TRUE);

}

long FAR PASCAL MainWndProc(hWnd, message, wParam, lParam)
HWND hWnd;
unsigned message;
WORD wParam;
LONG lParam;
{
    FARPROC lpProcAbout;

    switch (message)
    {
        case WM_COMMAND:
            switch (wParam)
            {
                case IDM_ABOUT:
                    lpProcAbout = MakeProcInstance(About, hInst);
                    DialogBox(hInst, "AboutBox", hWnd, lpProcAbout);
                    FreeProcInstance(lpProcAbout);
                    break;

                /* file menu commands */

                case IDM_EXIT:
                    DestroyWindow(hWnd);
                    break;

                /* fault menu commands */

                case IDM_GP:
                    DieGPFaultProc() ;
                    break;

                case IDM_GPDS:
                    DieGPViaDSFaultProc() ;
                    break;

                case IDM_STACK:
                    DieSPFaultProc() ;
                    break;

                case DEVNOTPRESENT:
                    DieNPXFaultProc() ;
                    break;

                case IDM_INVALIDOP:
                    DieInvalidOpFaultProc() ;
                    break;

                case IDM_TRAPZERO:
                    DieTrapZeroFaultProc() ;
                    break;

                case IDM_OVERWRITESTACK:
                    DieOverwriteStackProc() ;
                    break;

                case IDM_PARAMVALIDATION:
                    DieParamValidationProc() ;
                    break;

                case IDM_TESTODS:
                    TestODSProc() ;
                    break;


            }
            break;

        case WM_DESTROY:
            PostQuitMessage(0);
            break;

        default:
            return (DefWindowProc(hWnd, message, wParam, lParam));
    }
    return (NULL);
}


/*
 *   FUNCTION: About(HWND, unsigned, WORD, LONG)
*/
BOOL FAR PASCAL About(hDlg, message, wParam, lParam)
HWND hDlg;
unsigned message;
WORD wParam;
LONG lParam;
{
    switch (message) {
        case WM_INITDIALOG:
            return (TRUE);

        case WM_COMMAND:
            if (wParam == IDOK
                || wParam == IDCANCEL) {
                EndDialog(hDlg, TRUE);
                return (TRUE);
            }
            break;
    }
    return (FALSE);
}

int DieGPViaDSFaultProc()
{
   /* now cause a trap via use of NULL selector in DS */
   _asm {
     push ds            /* save old DS, AX */
     push ax
     mov  ax, 0         /* put zero in AX  */
     push ax
     pop  ds            /* now zero is in DS -- this is legal! */
     lodsb              /* <--- TRAP -- It is the attempt to reference a NULL selector */
     pop  ax            /*    that traps! */
     pop  ds
     }

     return(0) ;        /* remove compiler warning of NO RETURN VALUE */
}

int DieGPFaultProc()
{
   /* now cause a general protection fault... */
   _asm {
     push es            /* save old ES, AX        */
     push ax
     mov  al, 0
     push ds
     pop  es            /* ES now contains data selector in DS */
     mov  di, 0ffffh    /* offset 0ffffh exceeds the limit of selector in ES */
     stosb              /* <--- TRAP -- attempt to write 0 byte to invalid address */
     pop  ax
     pop  es
     }

     return(0) ;        /* remove compiler warning of NO RETURN VALUE */
}

int DieSPFaultProc()
{
   /* now cause a stack fault... */
   _asm {
     push ss
     push sp
     mov  sp, 0         /* set stack pointer to zero (base of stack segment) */
     push ax            /* <--- TRAP -- push a value onto the stack */
;    mov  ax, 0
;    push ax
;    pop  ss
                        /*              causing SP to wrap! */
     pop  sp
     pop  ss
     }

     return(0) ;        /* remove compiler warning of NO RETURN VALUE */
}

/* Does NOT fault */
int DieNPXFaultProc()
{
   MessageBox(NULL, "Oops!  Don't know any NPX opcodes!", "DeadMan", MB_OK) ;
     _asm  int 3h
     _asm fwait

     return(0) ;        /* remove compiler warning of NO RETURN VALUE */
}

int DieInvalidOpFaultProc()
{
   /* now cause an invalid op exception */
// MessageBox(NULL, "Oops!  Don't know any invalid opcodes!", "DeadMan", MB_OK) ;
 //_asm  int 3h
   _asm lock shr [bx], cl   /* 0f0h 0d2h 2fh */
                            /* Together these bytes make up an invalid opcode */
                            /*    (an illegal processor instruction) */

    return(0) ;        /* remove compiler warning of NO RETURN VALUE */
}

int DieTrapZeroFaultProc()
{
   /* now divide by zero... */
   _asm {
     push ax
     mov  ax, 0         /* put zero in AX register */
     div  ax            /* <--- TRAP -- cause a divide by zero exception */
     pop  ax
     }

     return(0) ;        /* remove compiler warning of NO RETURN VALUE */
}

/* This routine is used for testing TrapMan's stack-dumping code in the
   full version of trapman */
int DieOverwriteStackProc()
{
     char Tiny[2] ;
     char *ptr=Tiny ;      /* ptr is an address on the stack containing
                              space for two bytes of data */

     if (0)
        return(0) ;        /* remove compiler warning of NO RETURN VALUE */

     strcpy(ptr, "Testing!  DANGER!!! STACK OVERWRITE!") ;

}    /* <--- TRAP -- we trap here as we've overwritten BP and the return
                    address to our caller (and a few others!) on the stack */

int DieParamValidationProc()
{
     RegisterClass ( (WNDCLASS FAR *) 0x10000L ) ; /* <--- TRAP -- invalid far pointer */

     return(0) ;        /* remove compiler warning of NO RETURN VALUE */
}

/* This function is for testing trapman's ability to hook the Windows
      OutputDebugString() API.  Strings will be displayed in TrapMan's
      edit control window */
int TestODSProc()
{
     OutputDebugString("Testing, testing!") ;

     return(0) ;        /* remove compiler warning of NO RETURN VALUE */
}

