/*-----------------------------------------------------------------
  DEMO.C
  Slider Tester Program - By Arie Schwatzman - Mar-1995

  Demonstates the Slider Control -

  Slider range is represented in percents (0 to 100)
  The user can set :

  1. The initial percent setting.
  2. Size of increment for each move
  3. Number of ticks on the bottom of the slide.

  This is done by filling the SLIDERINFO structure and calling the
  SM_SET_DATA message.

  wParam - not used
  lParam - pointer to the SLIDERINFO structure.

  return value = not used.

  Getting the position is done by calling SM_GET_POS
  wParam = Not used
  lParam = Not used

  return value = the current percent setting.


  Notification message:

  WM_COMMAND is sent to the parent window with EN_CHANGE in the
  hiword of lParam every time the slide is moved
  loword of lParam represents the percentage after the move.



------------------------------------------------------------------*/

#include <windows.h>
#include <time.h>
#include <string.h>
#include <stdio.h>
#include "demo.h"


static HANDLE  hInst ;
/*------------------------------------------------------------------------------------*/
long FAR PASCAL _export WndProc (HWND, UINT, UINT, LONG) ;
long FAR PASCAL _export TestProc (HWND, UINT, UINT, LONG) ;
BOOL FAR PASCAL  FRegisterSliderControl(HANDLE hInstance);
/*------------------------------------------------------------------------------------*/
#pragma argsused
int PASCAL WinMain (HANDLE hInstance, HANDLE hPrevInstance,
			 LPSTR lpszCmdLine, int nCmdShow)
	  {
	  static char szAppName [] = "Demo" ;
	  MSG         msg;
	  HWND        hwnd ;
	  WNDCLASS    wndclass ;

	  if (!hPrevInstance)
	  {
	  wndclass.style         = CS_HREDRAW | CS_VREDRAW ;
	  wndclass.lpfnWndProc   = WndProc;
	  wndclass.cbClsExtra    = 0 ;
	  wndclass.cbWndExtra    = 0 ;
	  wndclass.hInstance     = hInstance ;
	  wndclass.hIcon         = LoadIcon (hInstance, szAppName) ;
	  wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
	  wndclass.hbrBackground = GetStockObject (WHITE_BRUSH) ;
	  wndclass.lpszMenuName  = "DemoMenu" ;
	  wndclass.lpszClassName = szAppName ;

	  RegisterClass (&wndclass) ;
	  }

	  hwnd = CreateWindow (szAppName, "Slider Demo Program",
			  WS_OVERLAPPEDWINDOW,
			  CW_USEDEFAULT, CW_USEDEFAULT,
			  CW_USEDEFAULT, CW_USEDEFAULT,
			  NULL, NULL, hInstance, NULL) ;
			  
	  //Register the class and load the DLL implicitly
	  FRegisterSliderControl(hInstance);
	  ShowWindow (hwnd, nCmdShow) ;
	  UpdateWindow (hwnd);
     while (GetMessage (&msg, NULL, 0, 0))
		{
		TranslateMessage (&msg) ;
		DispatchMessage (&msg) ;
		}
	  return msg.wParam ;
     }

long FAR PASCAL _export WndProc (HWND hwnd, UINT message, UINT wParam,
							  LONG lParam)
	  {

	  switch (message)
	  {
	  case WM_CREATE:
			 hInst = ((LPCREATESTRUCT) lParam)->hInstance ;
			 return 0 ;

	  case WM_COMMAND:
			 switch (wParam)
			 {
			 case IDM_DEMO:
				{
				FARPROC lpfn;
				lpfn = MakeProcInstance((FARPROC)TestProc,hInst);
				DialogBox(hInst,"DemoDlg",hwnd,lpfn);
				FreeProcInstance(lpfn);
				}
			 return 0 ;
			 }
			 break ;

	  case WM_DESTROY :
			 PostQuitMessage (0) ;
			 return 0 ;
	  }
	  return DefWindowProc (hwnd, message, wParam, lParam) ;
	  }
/***********************************************************************************/
long FAR PASCAL _export TestProc (HWND hDlg, UINT message, UINT wParam, LONG lParam)
	  {
	  SLIDERINFO si;

	  switch (message)
	  {
	  case WM_INITDIALOG:
			 //set initial values to the controls
			 si.nPercent=50;
			 si.nTicks=10;
			 si.nStepSize=5;
			 SendDlgItemMessage(hDlg,IDC_SLIDER,SM_SET_DATA,NULL,(LONG)&si);
			 return TRUE ;

	  case WM_COMMAND:
			 switch (wParam)
			 {
			 case IDC_SLIDER:
			 if (HIWORD(lParam)==EN_CHANGE)
				 SetDlgItemInt(hDlg,110,LOWORD(lParam),FALSE);
			 break;	      
			 case IDOK:
			 case IDCANCEL:
			 EndDialog (hDlg, 0) ;
			 return TRUE ;
			 }
			 break ;
	  }
	  return FALSE ;
	  }

