//****************************************************************************
// File: dlgtab.c
//
// Purpose: Contains main message loop and application entry point
//
// Functions:
//    WinMain() - initializes everything and enters message loop
//
// Development Team:
//    Homi Hanjani-Rassouli
//
// Written by Microsoft Product Support Services, Windows Developer Support
// Copyright (c) 1992 Microsoft Corporation. All rights reserved.
//****************************************************************************

#include <windows.h>
#include "global.h"
#include "init.h"
#include "dlgtab.h"

// Local function prototypes.
int PASCAL WinMain (HANDLE, HANDLE, LPSTR, int);

//***********************************************************************
// Function: WinMain
//
// Purpose: Called by Windows on app startup.  Initializes everything,
//          and enters a message loop.
//
// Parameters:
//    hInstance     == Handle to _this_ instance.
//    hPrevInstance == Handle to last instance of app.
//    lpCmdLine     == Command Line passed into app.
//    nCmdShow      == How app should come up (i.e. minimized/normal)
//
// Returns: Return value from PostQuitMessage.
//
// Comments:
//
// History:  Date       Author        Reason
//           11/09/92   HHR           Created
//****************************************************************************

int PASCAL WinMain (HANDLE hInstance,  // This instance
                    HANDLE hPrevInstance,                  // Last instance
                    LPSTR lpCmdLine,   // Command Line
                    int nCmdShow)      // Minimized or Normal?
{
   MSG msg;

   if (!hPrevInstance)
      if (!InitApplication(hInstance))
         return (FALSE);

   if (!InitInstance(hInstance, nCmdShow))
      return (FALSE);

   while (GetMessage(&msg,             // Put Message Here
                     NULL,             // Handle of window receiving msg
                     NULL,             // lowest message to examine
                     NULL))            // highest message to examine
   {
     if ( ((hChildDlg == 0) || (!IsDialogMessage(hChildDlg,&msg))) &&
          ((hParentDlg == 0) || (!IsDialogMessage(hParentDlg,&msg))) )
       {
       TranslateMessage(&msg);          // Translates virtual key codes
       DispatchMessage(&msg);           // Dispatches message to window
       } 
   }

   return (msg.wParam);                // Returns the value from PostQuitMessage
}
