//****************************************************************************
// File: flshdlg.c
//
// Purpose:     Contains the controlling dialog box for the FlashEdit application
//
// Functions:   FlashDlgProc()
//
//
// Development Team:    Sridhar Mandyam
//
//
// Written by Microsoft Product Support Services, Windows Developer Support
// Copyright (c) 1992 Microsoft Corporation. All rights reserved.
//****************************************************************************

#include <windows.h>
#include "global.h"
#include "flshdlg.h"   


//***********************************************************************
// Function:    FlashDlgProc(HWND, unsigned, WORD, LONG)
//
// Purpose:  When the user tab around the various edit controls, this dialog
//           procedure Flashes the edit control which currently has the focus 
//           with the color chosen by the user.
//
//    hWnd    == Handle to _this_ window.
//    message == Message to process.
//    wParam  == WORD parameter -- depends on message
//    lParam  == LONG parameter -- depends on message
//
// Returns: Depends on message.
//
// Comments:
//
// History:  Date       Author                  Reason
//           3/23/92   Sridhar Mandyam          Created
//
//****************************************************************************
 
BOOL FAR PASCAL FlashDlgProc (hDlg, message, wParam, lParam)
HWND hDlg;
unsigned message;
WORD wParam;
LONG lParam;
{
    static HANDLE hEditCtl;
    static BOOL bInvert=FALSE,  // flag for changing color to obtain flashing effect 
                bTimerSet,      // flag to indicate if timer set or not
                bFlash=TRUE;    // flag to indicate if flashing set or not
    
    switch (message)
    {
        case WM_COMMAND:
            switch (wParam)
            {            
                case IDC_OKBTN:                    
                    EndDialog(hDlg, TRUE);
                    return (TRUE);

                case IDC_CANCELBTN:
                case IDCANCEL:
                    EndDialog(hDlg, FALSE);                    
                    return (FALSE);
                    
               case IDC_FLASHONBTN:
                    // Check the Flash ON button and set the corresponding flag
                    if (HIWORD(lParam) == BN_CLICKED)
                    {
                      CheckRadioButton(hDlg,IDC_FLASHONBTN,IDC_FLASHOFFBTN,IDC_FLASHONBTN);
                      bFlash = TRUE;
                    }
                    break; 
                    
               case IDC_FLASHOFFBTN:
                    // Check the Flash OFF button and set the corresponding flag                  if (HIWORD(lParam) == BN_CLICKED)
                    {
                      CheckRadioButton(hDlg,IDC_FLASHONBTN,IDC_FLASHOFFBTN,IDC_FLASHOFFBTN);
                      bFlash = FALSE;
                    }
                    break;

               case IDC_REDBTN:
               case IDC_GREENBTN:
               case IDC_BLUEBTN:
               case IDC_YELLOWBTN:
               {
                    int CurrentBtn;
                    
                    CurrentBtn = wParam;
                    if (HIWORD(lParam) == BN_CLICKED)
                    {
                      // Check the color button clicked by the user, and set the 
                      // current brush to that color brush
                      CheckRadioButton(hDlg,IDC_REDBTN,IDC_YELLOWBTN,CurrentBtn);
                      ghbrCurrentBrush  = ghbrColorBrush[CurrentBtn-IDC_REDBTN];
                      gcrCurrentColor = gcrColors[CurrentBtn-IDC_REDBTN];
                    }
                    
               }
               break;
                                                                         
               case IDC_EDIT1:
               case IDC_EDIT2:
               case IDC_EDIT3:
               case IDC_EDIT4:               
               {
                    WORD CurrentEdit;
                    static BOOL bChangedText;
                    
                    // save the current edit control id
                    CurrentEdit = wParam;                    
                    
                    switch(HIWORD(lParam))
                    {
                      case EN_SETFOCUS:
                      {
                         char EditText[STR_LEN];                         
                         int nCount;

                        // When Focus received, if text has not changed, set text
                        // to indicate that this edit control has the focus
                        nCount = GetDlgItemText(hDlg, CurrentEdit, EditText, STR_LEN);                      
                        if (lstrcmp(EditText,szNoFocusText) == NULL)                          
                          SetDlgItemText(hDlg, CurrentEdit, szFocusText);
                          
                        hEditCtl = GetDlgItem(hDlg,CurrentEdit);
                        
                        // if Flashing desired set a timer; Note, that only one
                        // one timer is needed as it is killed everytime the edit 
                        // control loses focus. The same timer can be reused for 
                        // all the controls when they get the focus
                        if (bFlash) 
                        {
                          SetTimer(hDlg, ID_TIMER, FLASHINTERVAL, NULL);
                          bTimerSet = TRUE;
                        }             
                      }
                      return(TRUE);
                      
                      case EN_KILLFOCUS:
                      {
                          char EditText[STR_LEN];                         
                          int nCount;
                          
                          bInvert = FALSE;
                          nCount = GetDlgItemText(hDlg, CurrentEdit, EditText, STR_LEN);
                          
                          // if text not changed or text field is empty, change it
                          // to NoFocusText; else leave the changed text as is

                          nCount = GetDlgItemText(hDlg, CurrentEdit, EditText, STR_LEN);                      
                          bChangedText = lstrcmp(EditText, szFocusText);
                          
                          if (!bChangedText || !nCount)
                            SetDlgItemText(hDlg, CurrentEdit, szNoFocusText);
                          else 
                            SetDlgItemText(hDlg, CurrentEdit, EditText);
                                                                                 
                      }   // fall through to the next case 
                          
                      case EN_CHANGE:
                        // Kill the Timer if it has been set, and reset the flag
                        if (bTimerSet && bFlash)
                        {
                          KillTimer(hDlg,ID_TIMER);
                          bTimerSet = !bTimerSet;
                        }
                        break;                                                                   
                   }   // End switch (lParam)
                }  // End case IDC_EDIT1/2...    
                return(TRUE);
            }
            break;

        case WM_INITDIALOG:                        /* message: initialize    */
            
            // Set Flashing to ON and Color to RED initially 
            CheckRadioButton(hDlg,IDC_FLASHONBTN,IDC_FLASHOFFBTN,IDC_FLASHONBTN);        
            CheckRadioButton(hDlg,IDC_REDBTN,IDC_YELLOWBTN,IDC_REDBTN);
            ghbrCurrentBrush = ghbrColorBrush[0];    // Use RED brush at initialize time
            gcrCurrentColor = gcrColors[0];          // Use RED color
            
            // Set text to indicate that the no edit control has focus at initialize time
            SetDlgItemText(hDlg, IDC_EDIT1, szNoFocusText);
            SetDlgItemText(hDlg, IDC_EDIT2, szNoFocusText);
            SetDlgItemText(hDlg, IDC_EDIT3, szNoFocusText);
            SetDlgItemText(hDlg, IDC_EDIT4, szNoFocusText);
                                   
            SendDlgItemMessage(hDlg,IDC_EDIT1, EM_SETSEL, NULL, MAKELONG(0,0x7fff));
            SetFocus(GetDlgItem(hDlg, IDC_EDIT1));           
            return (FALSE); /* Indicates the focus is set to a control */             
        
        // Flip the bInvert flag to indicate changing the brush color      
        case WM_TIMER:        
            bInvert = !bInvert;
            InvalidateRect(hEditCtl,NULL,TRUE);
            break;

        // change the brush color for painting the edit control's background          
        case WM_CTLCOLOR:
            if (HIWORD(lParam) == CTLCOLOR_EDIT)
            { 
              if (bInvert)
              {               
                SetBkColor(wParam, gcrCurrentColor);
                SetTextColor(wParam, RGB(0,0,0));
                return(ghbrCurrentBrush);     // set to the specified COLOR brush
              }
              else return(ghbrColorBrush[4]);      // set to WHITE brush
            }
    }
    return FALSE;
}                        
