//****************************************************************************
// File: FlshEdit.c
//
// Purpose: Contains main message loop and application entry point
//
// Functions:
//    WinMain() - initializes everything and enters message loop
//
// Development Team:    Sridhar Mandyam
//
//
// Written by Microsoft Product Support Services, Windows Developer Support
// Copyright (c) 1992 Microsoft Corporation. All rights reserved.
//****************************************************************************

#include <windows.h>
#include "init.h"
#include "global.h"

// Local function prototypes.
int PASCAL WinMain (HANDLE, HANDLE, LPSTR, int);

//***********************************************************************
// Function: WinMain
//
// Purpose: Called by Windows on app startup.  Initializes everything,
//          and enters a message loop.
//
// Parameters:
//    hInstance     == Handle to _this_ instance.
//    hPrevInstance == Handle to last instance of app.
//    lpCmdLine     == Command Line passed into app.
//    nCmdShow      == How app should come up (i.e. minimized/normal)
//
// Returns: Return value from PostQuitMessage.
//
// Comments:
//
// History:  Date       Author                  Reason
//           3/23/92    Sridhar Mandyam         Created
//****************************************************************************

int PASCAL WinMain (HANDLE hInstance,           // This instance
                    HANDLE hPrevInstance,       // Last instance
                    LPSTR lpCmdLine,            // Command Line
                    int nCmdShow)               // Minimized or Normal?
{
   MSG msg;
   int i;

   if (!hPrevInstance)
      if (!InitApplication(hInstance))
         return (FALSE);

   if (!InitInstance(hInstance, nCmdShow))
      return (FALSE);
      
    // Create all the different colored brushes and the corresponding color
    // references to be used for SetBkColor
       
    ghbrColorBrush[0] = CreateSolidBrush(RED);           
    ghbrColorBrush[1] = CreateSolidBrush(GREEN);         
    ghbrColorBrush[2] = CreateSolidBrush(BLUE);        
    ghbrColorBrush[3] = CreateSolidBrush(YELLOW);       
    ghbrColorBrush[4] = CreateSolidBrush(WHITE);     
    
    gcrColors[0] = RED;    
    gcrColors[1] = GREEN;    
    gcrColors[2] = BLUE;    
    gcrColors[3] = YELLOW;

   while (GetMessage(&msg,             // Put Message Here
                     NULL,             // Handle of window receiving msg
                     NULL,             // lowest message to examine
                     NULL))            // highest message to examine
   {
      TranslateMessage(&msg);          // Translates virtual key codes
      DispatchMessage(&msg);           // Dispatches message to window
   }
   
   // Delete all the Color Brushes
   for (i=0; i<5; DeleteObject(ghbrColorBrush[i++]));

   return (msg.wParam);                // Returns the value from PostQuitMessage
}
