#include "winexp.inc"


/******************************************************************************/
/*                                                                            */
/*  Set the new selection for display                                         */
/*                                                                            */
/******************************************************************************/

void SetChoice (int choice, int offset, int highlight)
{
    if (choice == LatestChoice) return;

    LatestChoice = choice;

    Scroll_Offset  = offset;             /* Default to beginning               */
    LastHighlight  = highlight;          /* Default to no highlight            */
    Scroll_Depth   = Get_Scroll_Depth (LatestChoice);
    NonScrollLines = Get_NonScrollLines (LatestChoice);
    ClearScrn ();                       /* Clear the screen                   */
    SetWindowCaption ();                /* Set the window title               */
    SetScroll (Scroll_Depth, Scroll_Offset); /* Set the Scroll range and position  */
}

/******************************************************************************/
/*                                                                            */
/*  Select an item from the current Choice                                    */
/*  This will prompt us on to the next choice                                 */
/*                                                                            */
/*  Linenum is the linenum from the screen that was double clicked            */
/*  Starting with Line 0 at the top                                           */
/*                                                                            */
/******************************************************************************/
void SelectItem (int linenum)
{
    if (linenum < NonScrollLines) return;
    if (linenum >= Scroll_Depth-Scroll_Offset+NonScrollLines) return;
    linenum -= NonScrollLines;

    switch (LatestChoice) {
        case C_ACTIVE_VMS:
            VMHandle = Get_nth_VM (linenum+Scroll_Offset+1);
            ActiveVM_Offset = Scroll_Offset;
            ActiveVM_Highlight = LastHighlight;
            SetChoice (C_VMHANDLE, 0, -1);
            break;
        }
}

void PreviousItem ()
{
    switch (LatestChoice) {
        case C_VMHANDLE:
            SetChoice (C_ACTIVE_VMS, ActiveVM_Offset, ActiveVM_Highlight);
            break;

        case C_TIMESLICE:
            SetChoice (TS_Choice, TS_Offset, TS_Highlight);
            break;
        }
}

BOOL BlowAway ()
{
    if (Validate_VM_Handle (VMHandle)) return 0;

    SetChoice (C_ACTIVE_VMS, ActiveVM_Offset, ActiveVM_Highlight);
    return 1;
}


void SetWindowCaption ()
{
    switch (LatestChoice) {
        case C_ACTIVE_VMS:
            if (OS_Type == WINDOWS31_OS)
                SetWindowText (hMainWnd, "Active VM List for Windows Retail");
            if (OS_Type == WINDOWS31_DEBUG_OS)
                SetWindowText (hMainWnd, "Active VM List for Debug version of Windows");
            break;

        case C_VMHANDLE:
            SetWindowText (hMainWnd, "Contents of VM Control Block");
            break;

        case C_TIMESLICE:
            SetWindowText (hMainWnd, "Time Slice Information");
            break;

        default:
            SetWindowText (hMainWnd, "Un-named screen!");
        }
}

void UpdateWatchWindow ()
{

    switch (LatestChoice) {
        case C_ACTIVE_VMS:
            Display_Active_VMs();
            break;

        case C_VMHANDLE:
            if (BlowAway ()) return;
            Display_VMHandle ();
            break;

        case C_TIMESLICE:
            Display_TimeSlice();
            break;
        }
    ClearLastLine ();
    UpdateWindow (hMainWnd);
}
