//****************************************************************************
// File: mainwnd.c
//
// Purpose: Main overlapped window's message handler
//
// Functions:
//    MainWndProc() - message handler for main overlapped window
//    DoCommands() - called by MainWndProc to handle all WM_COMMAND messages
//
// Development Team:    Sridhar Mandyam
//
//
// Written by Microsoft Product Support Services, Windows Developer Support
// Copyright (c) 1992 Microsoft Corporation. All rights reserved.
//****************************************************************************

#include <windows.h>
#include "global.h"
#include "mainwnd.h"
#include "flshdlg.h"
#include "aboutdlg.h"

// Local function prototypes.
long DoCommands (HWND hWnd,
                 unsigned message,
                 WORD wParam,
                 LONG lParam);

//****************************************************************************
// Function: MainWndProc
//
// Purpose: Message handler for main overlapped window.
//
// Parameters:
//    hWnd    == Handle to _this_ window.
//    message == Message to process.
//    wParam  == WORD parameter -- depends on message
//    lParam  == LONG parameter -- depends on message
//
// Returns: Depends on message.
//
// Comments:
//
// History:  Date       Author                  Reason
//           3/23/92   Sridhar Mandyam          Created
//****************************************************************************

long FAR PASCAL MainWndProc (HWND hWnd,
                             unsigned message,
                             WORD wParam,
                             LONG lParam)
{
   switch (message)
   {
      // Dispatch WM_COMMAND messages to our command handler, DoCommands().
      case WM_COMMAND:
         return DoCommands(hWnd, message, wParam, lParam);


      // On WM_DESTROY, terminate this app by posting a WM_QUIT message.

      case WM_DESTROY:
         PostQuitMessage(0);
         break;


      // We didn't handle, pass to DefWindowProc.

      default:
         return DefWindowProc(hWnd, message, wParam, lParam);
   }


   return (NULL);
}

//****************************************************************************
// Function: DoCommands
//
// Purpose: Called by MainWndProc() to handle all WM_COMMAND messages.
//
// Parameters:
//    hWnd    == Handle to _this_ window.
//    message == Message to process.
//    wParam  == WORD parameter -- depends on message
//    lParam  == LONG parameter -- depends on message
//
// Returns : Depends on message.
//
// Comments:
//
// History:  Date       Author                  Reason
//           3/23/92  Sridhar Mandyam           Created
//****************************************************************************

long DoCommands (HWND hWnd,
                 unsigned message,
                 WORD wParam,
                 LONG lParam)
{
   switch (wParam)
   {
       // Call FlashDlgProc() Dialog Box 
       case IDM_FLASH:
       {
          FARPROC lpFlashDlgProc;

          lpFlashDlgProc = MakeProcInstance((FARPROC) FlashDlgProc, ghInst);
          DialogBox(ghInst, "FlashEdit", hWnd, lpFlashDlgProc);
          FreeProcInstance(lpFlashDlgProc);
          break;
       }
   
      // Put up the About box.
      case IDM_ABOUT:
      {
         FARPROC lpAboutDlgProc;

         lpAboutDlgProc = MakeProcInstance((FARPROC) AboutDlgProc, ghInst);

         DialogBox(ghInst,             // current instance
                   AboutBoxName,       // resource to use 
                   hWnd,               // parent handle   
                   lpAboutDlgProc);         // About() instance address

         FreeProcInstance(lpAboutDlgProc);
         break;
      }


      // User picked File.Exit, terminate this app.

      case IDM_EXIT:
         DestroyWindow(hWnd);
         break;


      // Must be some system command -- pass it on to the default
      //  window procedure.

      default:
         return DefWindowProc(hWnd, message, wParam, lParam);
   }

   return (NULL);
}
