//****************************************************************************
// File: mainwnd.c
//
// Purpose: Main overlapped window's message handler
//
// Functions:
//    MainWndProc() - message handler for main overlapped window
//    DoCommands() - called by MainWndProc to handle all WM_COMMAND messages
//
// Development Team:
//    Homi Hanjani-Rassouli
//
// Written by Microsoft Product Support Services, Windows Developer Support
// Copyright (c) 1992 Microsoft Corporation. All rights reserved.
//****************************************************************************

#include <windows.h>
#include "global.h"
#include "about.h"
#include "dialog.h"
#include "mainwnd.h"

// Local function prototypes.
long DoCommands (HWND hWnd,
                 unsigned message,
                 WORD wParam,
                 LONG lParam);

//****************************************************************************
// Function: MainWndProc
//
// Purpose: Message handler for main overlapped window.
//
// Parameters:
//    hWnd    == Handle to _this_ window.
//    message == Message to process.
//    wParam  == WORD parameter -- depends on message
//    lParam  == LONG parameter -- depends on message
//
// Returns: Depends on message.
//
// Comments:
//
// History:  Date       Author        Reason
//           11/09/92   HHR           Created
//****************************************************************************

long FAR PASCAL MainWndProc (HWND hWnd,
                             unsigned message,
                             WORD wParam,
                             LONG lParam)
{
   switch (message)
   {
      case WM_CREATE:
         {
         // Associate the procedure with this instance's data.
         lpParentDlgProc=MakeProcInstance(ParentDlgProc, ghInst);
         lpChildDlgProc = MakeProcInstance(ChildDlgProc, ghInst);

         return DefWindowProc(hWnd, message, wParam, lParam);
         }

      // Dispatch WM_COMMAND messages to our command handler, DoCommands().
      case WM_COMMAND:
         return DoCommands(hWnd, message, wParam, lParam);

      // On WM_DESTROY, terminate this app by posting a WM_QUIT message.

      case WM_DESTROY:
         PostQuitMessage(0);
         break;


      case WM_CLOSE:
         {
         FreeProcInstance(lpParentDlgProc);
         FreeProcInstance(lpChildDlgProc);
         return DefWindowProc(hWnd, message, wParam, lParam);
         }

      // We didn't handle, pass to DefWindowProc.

      default:
         return DefWindowProc(hWnd, message, wParam, lParam);
   }


   return (NULL);
}

//****************************************************************************
// Function: DoCommands
//
// Purpose: Called by MainWndProc() to handle all WM_COMMAND messages.
//
// Parameters:
//    hWnd    == Handle to _this_ window.
//    message == Message to process.
//    wParam  == WORD parameter -- depends on message
//    lParam  == LONG parameter -- depends on message
//
// Returns : Depends on message.
//
// Comments:
//
// History:  Date       Author        Reason
//           11/09/92   HHR           Created
//****************************************************************************

long DoCommands (HWND hWnd,
                 unsigned message,
                 WORD wParam,
                 LONG lParam)
{
   switch (wParam)
   {
      // Puts up the Parent dialog with its childern.
      case IDM_DLGTAB:
      {
         iDoDialog(PARENTDLG);
         iDoDialog(CHILDDLG);
         
         break;
      }

      // Put up the About box.
      case IDM_ABOUT:
      {
         FARPROC lpDlgProc;

         lpDlgProc = MakeProcInstance(AboutDlg, ghInst);

         DialogBox(ghInst,             // current instance
                   AboutBoxName,       // resource to use 
                   hWnd,               // parent handle   
                   lpDlgProc);         // About() instance address

         FreeProcInstance(lpDlgProc);
         break;
      }


      // User picked File.Exit so post WM_CLOSE, causing the default
      // window procedure to call DestroyWindow(). We could process
      // WM_CLOSE for confirmation from user that we should exit.

      case IDM_EXIT:
         PostMessage(hWnd, WM_CLOSE, 0, 0L);
         break;


      // Must be some system command -- pass it on to the default
      //  window procedure.

      default:
         return DefWindowProc(hWnd, message, wParam, lParam);
   }

   return (NULL);
}
