/***********************************************************************
   File mlist.c

   Multiple Feature listbox main file

   Copyright (c) Desaware 1992 - All rights reserved

   The following sample code is provided as-is with no warranty
   as to operation, performance or the suitability of this control to be
   used in any application.

   See the readme.txt file provided with this control for more
   information.

************************************************************************/

#define NOCOMM
#include <windows.h>
#include "vbapi.h"
#define CTL_DATA            // To declare static data in mlist.h
#include "mlist.h"
#include <dos.h>

//---------------------------------------------------------------------------
// Standard Error Values
//---------------------------------------------------------------------------
#define ERR_None              0
#define ERR_InvPropVal      380     // Error$(380) = "Invalid property value"

//------------------------------------------------------------------------
// Global Variables
//------------------------------------------------------------------------
// This flag is set true during the VBM_INITIALIZE message and set to false
// during the VBM_CREATED message.  This allows the code to determine during
// property message handling, if initialization is occuring.

BOOL Initializing = FALSE;
HANDLE hmodDLL;
WORD cVbxUsers = 0;

//---------------------------------------------------------------------------
// Register custom control.
//	This routine is called by VB when the custom control DLL is
//	loaded for use.
//---------------------------------------------------------------------------
BOOL FAR PASCAL _export VBINITCC(USHORT usVersion,BOOL   fRuntime)
{
   cVbxUsers++;
    // Avoid warnings on unused (but required) formal parameters
    fRuntime  = fRuntime;
    usVersion = usVersion;

   if(usVersion <VB_VERSION)
       return(FALSE);
       // return VBRegisterModel(hmodDLL, &modelMlist_Vb1);
   else
       return VBRegisterModel(hmodDLL, &modelMlist);
}


VOID FAR PASCAL _export VBTERMCC(VOID)
{
   cVbxUsers--;
   // if cVbxusers is zero, free any resources
}

LPMODELINFO FAR PASCAL _export VBGetModelInfo(USHORT usVersion)
{
   MODELINFO mi;
   LPMODEL lpm[2];
   if(usVersion < VB_VERSION)
      // lpm[0] = &modelMlist_Vb1;
      lpm[0] = NULL;
   else
      lpm[0] = &modelMlist;
   lpm[1] = NULL;
   mi.usVersion = usVersion;
   mi.lplpmodel = lpm;
   return(&mi);
}

//---------------------------------------------------------------------------
// Initialize library.
//	This routine is called from the DLL entry point in LIBINIT.ASM
//	which is called when the first client loads the DLL.
//---------------------------------------------------------------------------
BOOL FAR PASCAL LibMain(HANDLE hmod,HANDLE segDS,USHORT cbHeapSize,
                        LPSTR lpszCmdLine)
{
    // Avoid warnings on unused (but required) formal parameters
    cbHeapSize = cbHeapSize;
    segDS = segDS;

    hmodDLL = hmod;

    // Leave our DS unlocked when we're not running
    // UnlockData( 0 );

    return TRUE;
}


//---------------------------------------------------------------------------
// Handle exit notification from Windows.
//	This routine is called by Windows when the library is freed
//	by its last client.
//---------------------------------------------------------------------------
VOID FAR PASCAL WEP(BOOL fSystemExit)
{
    // Avoid warnings on unused (but required) formal parameters
    fSystemExit = fSystemExit;
}


//--------------------------------------------------------------------
// Local Function Prototypes
//--------------------------------------------------------------------
/*

 Function Name:   VOID SetControlName(HCTL hctl, HWND hwnd)

 Parameters:      hctl = Control handle
                  hwnd = Window handle of control

 Description:     Sets the first entry of the listbox to contain the
                  name of the control while in design mode.

*/

VOID SetControlName(HCTL hctl, HWND hwnd)
{
   char buf[41];
   if(VBGetMode()!=MODE_DESIGN) return;   // Ignore in run mode
   VBGetControlName(hctl, (LPSTR)buf);    // Get the name of the control
   if(SendMessage(hwnd, LB_GETCOUNT, 0, 0L)>0)
      SendMessage(hwnd, LB_DELETESTRING, 0, 0L);   // Delete the prior name
   // Add the current name into the listbox
   SendMessage(hwnd, LB_ADDSTRING, 0, (LONG)((LPSTR)buf));
}

/*

 Function Name:   WORD GetVisibleLines(HWND hwnd)

 Parameters:      hwnd = Window handle of control

 Description:     Returns the number of lines of text that are visible for
                  this listbox.  A line that is only partially visible is
                  NOT counted.
                  Returns 0 on error.

*/

WORD GetVisibleLines(HWND hwnd)
{
   RECT rc, rc2;
   WORD lines;
   // Get a rectangle describing the size of the first entry is the listbox
   // Note: for mlist it can be assumed that all entries are the same size.
   if(SendMessage(hwnd, LB_GETITEMRECT, 0, (LONG)((LPRECT)&rc))==LB_ERR)
      return(0);
   // Get a rectangle describing the client area of the entire listbox
   GetClientRect(hwnd, (LPRECT)&rc2);
   // Simple division calculates the number of lines visible.
   lines = (WORD)((rc2.bottom-rc2.top)/(rc.bottom-rc.top));
   return(lines);
}

/*

 Function Name:   WORD GetTotalLines(HWND hwnd)

 Parameters:      hwnd = Window handle of control

 Description:     Returns the number of lines in the listbox.
                  Returns 0 on error.

*/
WORD GetTotalLines(HWND hwnd)
{
   short lines;
   lines = (SHORT)SendMessage(hwnd, LB_GETCOUNT, 0, 0L);
   if(lines==LB_ERR) return(0);
   return(lines);
}

/*

 Function Name:   WORD SetVirtualWidth(HCTL hctl, HWND hwnd)

 Parameters:      hctl = Control Handle
                  hwnd = Window handle of control

 Description:     This function helps support horizontal scrolling of a
                  listbox.  Horizontal scrolling is enabled when the
                  "virtual width" of the box is greater than the actual
                  physical width.  The virtual width is defined in
                  dialog units ( = 1/4 the size of the average character
                  width for the current font) and can be found in the
                  virtualwidth field of the model structure.

*/
VOID SetVirtualWidth(HCTL hctl, HWND hwnd)
{
   PMLIST phc;
   char tchar = '\0';

   phc = VBDerefControl(hctl);

   // Turn off redraw temporarily if on (prevents multiple refresh)
   if(phc->redrawstate & 0x01) SendMessage(hwnd, WM_SETREDRAW, 0, 0L);
   // Set the horizontal extent in dialog units
   SendMessage(hwnd, LB_SETHORIZONTALEXTENT,
               phc->virtualwidth * LOWORD(GetDialogBaseUnits()), 0L);
   // A line needs to be inserted or deleted before the control will be
   // properly updated (this was discovered empirically and is not documented
   // anywhere.  So we add, then delete an empty line.  There may be other
   // ways to accomplish this task.
   SendMessage(hwnd, LB_INSERTSTRING, 0, (LONG)((LPSTR)&tchar));
   SendMessage(hwnd, LB_DELETESTRING, 0, 0L);
   // And turn the redraw back on if necessary.
   if(phc->redrawstate & 0x01) SendMessage(hwnd, WM_SETREDRAW, 1, 0L);
   // Always redraw the control
   InvalidateRect(hwnd, (LPRECT)NULL, TRUE);
}

/*

 Function Name:   WORD SetAllTabs(HCTL hctl, HWND hwnd)

 Parameters:      hctl = Control Handle
                  hwnd = Window handle of control

 Description:     mlist.vbx can hold up to 32 tabs as currently defined
                  (see mlist.h).  Tabstops are specified in terms of
                  character position (thus actual tab position will be
                  dependent on the average character width in the current
                  font).  This function updates the internal control tabs
                  based on the contents of the HCTL structure.

 Note:   part of this code is obsolete now that Mlist uses owner draw 
         listboxes. If owner draw was not set, this function would set
         the list box tabstops
*/

VOID SetAllTabs(HCTL hctl, HWND hwnd)
{
   PMLIST phc;
   WORD tabcount;
   // WORD t;
   // short usetabs[32];

   phc = VBDerefControl(hctl);

   // This routine makes sure that each tab is greater than the tab before it.
   for(tabcount = 0; tabcount<33 && phc->tablocs[tabcount]; tabcount++) {
      if(phc->tablocs[tabcount+1] &&
         phc->tablocs[tabcount]>= phc->tablocs[tabcount+1]) {
         phc->tablocs[tabcount+1] = phc->tablocs[tabcount]+1;
         }
      }

   // What follows below is obsolete when using owner draw

   //// We build an array of tabstops in dialog units (4 * character width)
   //for(t=0; t<tabcount; t++) usetabs[t] = phc->tablocs[t] * 4;
   //switch(tabcount) {
   //   case 0:  // No tabs - clear the tabstops
   //            SendMessage(hwnd, LB_SETTABSTOPS, 0, NULL);
   //            break;
   //   default: // One or more tabs are set - send tab array to control.
   //            SendMessage(hwnd, LB_SETTABSTOPS,tabcount,
   //                        (LONG)((LPSTR)usetabs) );
   //            break;
   //   }
   
   // Redraw the control based on the new tabs
   InvalidateRect(hwnd, (LPRECT)NULL, TRUE);
}

/*

 Function Name:   WORD DoScrollAdd(HWND hwnd, LPSTR pstring, PMLIST phc)

 Parameters:      hwnd = Window handle of control
                  pstring = string to add to the listbox
                  phc = Pointer to the HCTL structure for this control

 Description:     This function supports mlist.vbx's unique scrolling mode.
                  If the last entry is visible in the listbox, pstring will
                  be added and the listbox will scroll upward by one.  If
                  the last entry is not visible, pstring will still be added,
                  but there will be no change to the listbox.
                  phc->maxscrolled specifies the maximum number of lines that
                  will be in the listbox after pstring is added (extra lines
                  are deleted off the top of the listbox)
*/
short DoScrollAdd(HWND hwnd, LPSTR pstring, PMLIST phc)
{
   BOOL enddisplayed = FALSE; // Set true if last line is displayed
   short totallines, topindex, totvisible;
   short x, numtodelete, newtopindex;

   totallines = GetTotalLines(hwnd);
   topindex = (short)SendMessage(hwnd, LB_GETTOPINDEX, 0,0L);
   totvisible = GetVisibleLines(hwnd);
   // Determine if the last entry is currently displayed
   if((topindex + totvisible) >= totallines) enddisplayed = TRUE;

   // Must we remove entries from the top of the listbox?
   if (totallines >= phc->maxscrolled) {  // Yes! Remove entries
      SendMessage(hwnd, WM_SETREDRAW, 0, 0L);   // Redraw off
      numtodelete = totallines - phc->maxscrolled+1;
      for(x=0; x<numtodelete; x++)
         SendMessage(hwnd, LB_DELETESTRING, 0, 0L);   // Delete the entries
      // When we deleted lines, the index number of the top visible
      // string was changed by the deletions.  Thus we must recalculate its
      // index and set that as the top index of the listbox.
      newtopindex = topindex - numtodelete;
      // newtopindex<0 implies that we deleted a visible string, thus we
      // simply disply the top string of the listbox
      if(!enddisplayed) SendMessage(hwnd, LB_SETTOPINDEX,
                                    (newtopindex<0)?0:newtopindex, 0L);
      // Turn redraw back on.
      SendMessage(hwnd, WM_SETREDRAW, (phc->redrawstate & 0x01)?1:0,0L);
      if(newtopindex< 0) {
         // Since we deleted visible strings, we must redraw the entire box
         newtopindex = 0;
         InvalidateRect(hwnd, (LPRECT)NULL, TRUE);
         }
      totallines = phc->maxscrolled-1; // Number of lines remaining

      }
   // Now add the entry
   if(SendMessage(hwnd, LB_ADDSTRING, 0, (LONG)pstring)<0) return(-1);
   // And scroll if necessary
   totallines++;
   if(enddisplayed)
      // LB_SETTOPINDEX is smart - we don't have to calculate exactly which
      // entry should be the top entry.  This forces the newly entered
      // string (index = totallines-1) to be displayed which in this case
      // forces the listbox to scroll up by one.
      SendMessage(hwnd, LB_SETTOPINDEX, totallines-1, 0L);
   return(0);
}


/*

 Function Name:   InitializeWindowData(hctl, hwnd)

 Parameters:      hctl = Control Handle
                  hwnd = Window handle of control

 Description:     This routine initializes the window data based on
                  the property values.  It must be called after
                  the window is created.

 Notes:        
*/

VOID InitializeWindowData(HCTL hctl, HWND hwnd)
{
   PMLIST phc;

   phc = VBDerefControl(hctl);
   // Set the redraw flag
   if(VBGetMode()!=MODE_DESIGN) {
      // These are runtime attributes
      SendMessage(hwnd, WM_SETREDRAW,
                  (phc->redrawstate & 0x01)?1:0,0L);
      if(phc->redrawstate & 0x01)
         InvalidateRect(hwnd, (LPRECT)NULL,TRUE);
      if(phc->multicolumn) {
         SendMessage(hwnd, LB_SETCOLUMNWIDTH,
                     phc->multicolumn *
                     LOWORD(GetDialogBaseUnits()),
                     0L);
         }
      SetVirtualWidth(hctl, hwnd);
      }
   else {
      // Design time attributes
      SetControlName(hctl, hwnd);
      }
   SetAllTabs(hctl, hwnd);
   return;
}


//----------------------------------------------------------------
// Error function
//----------------------------------------------------------------
// These functions are used to return custom error messages.  A more
// correct implementation would place these strings in the resource file
// (which would also improve internationalization support).

LONG InvalidProp()
{
   // Some properties are only valid under certain conditions
   VBSetErrorMessage(30000, (LPSTR)"Invalid Property for this ListType");
   return(30000);
}

LONG WriteOnlyProp()
{
   VBSetErrorMessage(30001, (LPSTR)"Property is write only");
   return(30001);
}

/*

 Function Name:   LONG DoSetProperty(HCTL hctl, HWND hwnd, USHORT wp, LONG lp)

 Parameters:      hctl = Control handle
                  hwnd = Window handle of control
                  wp, lp = WORD and LONG parameters depending on property

 Description:     This function handles the VBM_SETPROPERTY messages.
*/

LONG DoSetProperty(HCTL hctl, HWND hwnd, USHORT wp, LONG lp)
{
   BOOL dodefault = FALSE; // Set TRUE to perform default processing
   LPDATASTRUCT lpd;
   WORD lines, vislines;
   SHORT cidx, useval;
   LPSTR pstring;
   PMLIST phc;
   BOOL dorecreate;        // Set TRUE if window must be recreated
   LPITEMATTR lpattr;

   switch(wp) {
      case IPROP_MLIST_NAME: // Called whenever ctlname is changed
                              // If we're in design mode, post a message
                              // to inform our control that the name needs
                              // updating
                              if(VBGetMode()==MODE_DESIGN && !Initializing &&
                                 IsWindow(hwnd))
                                 PostMessage(hwnd, CM_SETCTLNAME, 0, 0L);
                              dodefault = TRUE; // Do default processing
                              break;

      case IPROP_MLIST_LIST:  // List array - note how DATASTRUCT is used
                              // to access property arrays.
                              lpd = (LPDATASTRUCT)lp;
                              cidx = (SHORT)lpd->index[0].data;
                              pstring = (LPSTR)lpd->data;
                              lines = GetTotalLines(hwnd);
                              if(cidx< -1 || cidx>=(short)lines) return(381);
                              // An entry is changed by deleting the prior
                              // entry and inserting the new string at that
                              // location.
                              if(cidx>=0)
                                 SendMessage(hwnd, LB_DELETESTRING, cidx,0L);
                              SendMessage(hwnd, LB_INSERTSTRING, cidx,
                                          (LONG)pstring);
                              break;

      case IPROP_MLIST_LISTINDEX:
                              // DoCheckProperty already checked validity
                              if(SendMessage(hwnd, LB_SETCURSEL, (SHORT)lp,
                                             0L) ==LB_ERR && (SHORT)lp>=0)
                                 return(380);
                              break;

      case IPROP_MLIST_LISTTYPE:
                              // If we are changing the type of the box, we
                              // need to change the class style as well.
                              // All we do here is set the new type and
                              // tell the system to recreate the window.
                              // The real work is done in the WM_NCCREATE
                              // message in MlistCtlProc()
                              phc = VBDerefControl(hctl);
                              if ((BYTE)lp == phc->boxmode) return(0);
                              phc->boxmode = (BYTE)lp;
                              // If this is initialization, just go ahead
                              // and set the control type - this is a 
                              // PF_fPreHwnd property, meaning that it is set
                              // before the window is created.
                              if(!Initializing) {
                                 VBRecreateControlHwnd(hctl);
                                 // The window handle has changed
                                 hwnd = VBGetControlHwnd(hctl);
                                 InitializeWindowData(hctl, hwnd);
                                 }
                              return(0);

      case IPROP_MLIST_TOPINDEX: // Set index of top visible line
                              vislines = GetVisibleLines(hwnd);
                              lines = GetTotalLines(hwnd);
                              useval = (SHORT)lp;
                              // We do some bounds checking here.
                              if(vislines>lines) useval = 0;
                              if ( useval > (SHORT)(lines-vislines))
                                 useval = lines - vislines;
                              SendMessage(hwnd, LB_SETTOPINDEX, useval, 0L);
                              break;

      case IPROP_MLIST_TABLOCS:  // Set a tab.  This is also a property array
                              lpd = (LPDATASTRUCT)lp;
                              cidx = (SHORT)lpd->index[0].data;
                              if(cidx>32 || cidx<0) return(380);
                              phc = VBDerefControl(hctl);
                              phc->tablocs[cidx] = (short)lpd->data;
                              SetAllTabs(hctl, hwnd);
                              break;

      case IPROP_MLIST_REDRAW:   // Redraw mode
                              phc = VBDerefControl(hctl);
                              if(VBGetMode()!=MODE_DESIGN && !Initializing)
                                 SendMessage(hwnd, WM_SETREDRAW,
                                           (phc->redrawstate & 0x01)?1:0,0L);
                                 // Note that turning redraw on does NOT
                                 // automatically repaint the window, so
                                 // we do it here.
                                 if(phc->redrawstate & 0x01)
                                    InvalidateRect(hwnd, (LPRECT)NULL,TRUE);
                              break;

      case IPROP_MLIST_SELECTED: // Set selected state for specified entry
                                 // Use ListIndex prop for single select lists
                              lpd = (LPDATASTRUCT)lp;
                              cidx = (SHORT)lpd->index[0].data;
                              useval = cidx;
                              if(SendMessage(hwnd, LB_SETSEL,
                                             LOWORD(lpd->data)? 1:0,
                                             (long)useval) ==LB_ERR)
                                 return(380);
                              break;

      case IPROP_MLIST_SELSTRING:
                              // Selects the first string that has the
                              // prefix specified in lpd->data, starting
                              // the search from cidx (the property array
                              // index).
                              phc = VBDerefControl(hctl);
                              if(phc->boxmode>0) return(InvalidProp());
                              lpd = (LPDATASTRUCT)lp;
                              cidx = (SHORT)lpd->index[0].data;
                              pstring = (LPSTR)lpd->data;
                              if(SendMessage(hwnd, LB_SELECTSTRING, cidx,
                                 lp)== LB_ERR)
                                 return(380);
                              break;


      case IPROP_MLIST_SORTED: // Once again, the real work is done during
                               // the WM_NCCREATE message
                              if(!Initializing) {
                                 VBRecreateControlHwnd(hctl);
                                 hwnd = VBGetControlHwnd(hctl);
                                 InitializeWindowData(hctl, hwnd);
                                 }
                              break;

      case IPROP_MLIST_COLUMNS:
                              // Specify the number of columns.  When
                              // switching between multi-column and single-
                              // column the listbox is recreated.
                              phc = VBDerefControl(hctl);
                              if ((short)lp == phc->multicolumn) return(0);
                              dorecreate = TRUE;
                              if( phc->multicolumn && (short)lp) {
                                 // Change column width on existing multicol
                                 SendMessage(hwnd, LB_SETCOLUMNWIDTH,
                                             (short)lp *
                                             LOWORD(GetDialogBaseUnits()),
                                             0L);
                                 dorecreate = FALSE;
                                 }
                              phc->multicolumn = (short)lp;
                              if(!Initializing && dorecreate) {
                                 VBRecreateControlHwnd(hctl);
                                 hwnd = VBGetControlHwnd(hctl);
                                 InitializeWindowData(hctl, hwnd);
                                 UpdateAllHeights(hctl);
                                 }
                              return(0);

      case IPROP_MLIST_VIRTUALWIDTH:
                              phc = VBDerefControl(hctl);
                              if(VBGetMode()!=MODE_DESIGN && !Initializing)
                                 SetVirtualWidth(hctl, hwnd);
                              break;

      case IPROP_MLIST_ADDSCROLLED:
                              phc = VBDerefControl(hctl);
                              if(Initializing) break;
                              pstring = (LPSTR)lp;
                              useval = DoScrollAdd(hwnd, pstring, phc);
                              return(useval);

      case IPROP_MLIST_ITEMDATA:  
                              lpd = (LPDATASTRUCT)lp;
                              cidx = (SHORT)lpd->index[0].data;
                              lines = GetTotalLines(hwnd);
                              if(cidx< -1 || cidx>=(short)lines) return(381);
                              if(VBSendControlMsg(hctl, LB_SETITEMDATA, 
                                       cidx, lpd->data)==LB_ERR) return(390);
                              break;

      case IPROP_MLIST_ITEMBACKCOLOR:
      case IPROP_MLIST_ITEMFORECOLOR:
      case IPROP_MLIST_ITEMSTRIKETHRU:
                              lpd = (LPDATASTRUCT)lp;
                              cidx = (SHORT)lpd->index[0].data;
                              lpattr = LockItem(hctl, cidx);
                              if(!lpattr) return(380);
                              switch(wp) {
                                 case IPROP_MLIST_ITEMBACKCOLOR:
                                       lpattr->cBack = lpd->data;
                                       lpattr->flags |= 1;
                                       break;
                                 case IPROP_MLIST_ITEMFORECOLOR:
                                       lpattr->cFore = lpd->data;
                                       lpattr->flags |= 2;
                                       break;
                                 case IPROP_MLIST_ITEMSTRIKETHRU:
                                       if(lpd->data & 0x0ffff)
                                          lpattr->flags |= 4;
                                       else lpattr->flags &= ~4;
                                       break;
                                 }
                              UnlockItem(hctl, cidx);
                              InvalidateRect(hwnd, NULL, TRUE);
                              break;

      case IPROP_MLIST_ITEMIMAGE:
                              lpd = (LPDATASTRUCT)lp;
                              cidx = (SHORT)lpd->index[0].data;
                              lpattr = LockItem(hctl, cidx);
                              if(!lpattr) return(380);
                              if(lpattr->hPic) VBFreePic(lpattr->hPic);
                              lpattr->hPic = (HPIC)lpd->data;
                              VBRefPic(lpattr->hPic);
                              UnlockItem(hctl, cidx);
                              InvalidateRect(hwnd, NULL, TRUE);
                              phc = VBDerefControl(hctl);
                              if(phc->multicolumn>0) {
                                 UpdateAllHeights(hctl);
                                 }
                              else {
                                 VBSendControlMsg(hctl, LB_SETITEMHEIGHT,
                                    cidx, MAKELONG(GetHeight(hctl, cidx),0));
                                 }
                              break;


      default: dodefault = TRUE;
                              break;
      }
   if (dodefault) return(VBDefControlProc( hctl, hwnd, VBM_SETPROPERTY,
                                             wp, lp ));
   else return(0);
}

/*

 Function Name:   LONG DoGetProperty(HCTL hctl, HWND hwnd, USHORT wp, LONG lp)

 Parameters:      hctl = Control handle
                  hwnd = Window handle of control
                  wp, lp = WORD and LONG parameters depending on property

 Description:     This function handles the VBM_GETPROPERTY messages.
*/

LONG DoGetProperty(HCTL hctl, HWND hwnd, USHORT wp, LONG lp)
{
   BOOL dodefault = FALSE; // Set TRUE to perform default processing
   LPDATASTRUCT lpd;
   char tbuf[128];
   SHORT cidx, slen;
   WORD lines;
   LPSTR lhptr;
   short tlen;
   LOCALHANDLE lhnd = NULL;
   HSZ newhsz;
   PMLIST phc;
   LPITEMATTR lpattr;

   switch(wp) {
      case IPROP_MLIST_TEXT:
                              phc = VBDerefControl(hctl);
                              // LB_GETCURSEL returns <0 on no selection or 
                              // multiple select listbox
                              if((cidx=(SHORT)SendMessage(hwnd, LB_GETCURSEL,
                                                   0, 0L))<0) {
                                 // Return NULL string on error
                                 tbuf[0] = '\0';
                                 newhsz = VBCreateHsz(FP_SEG(hctl),
                                                      (LPSTR)tbuf);
                                 *((HSZ FAR *)lp) = newhsz;
                                 return(0);
                                 }
                              else {// Text is same as List(ListIndex)
                                 DATASTRUCT ds;
                                 short ires;
                                 ds.data = 0;
                                 ds.index[0].data = cidx;
                                 ds.index[0].datatype = DT_SHORT;
                                 ds.cindex = 1;
                                 ires = VBGetControlProperty(hctl,
                                                      IPROP_MLIST_LIST,
                                                      (LPVOID)&ds);
                                 *((HSZ FAR *)lp) = (HSZ)ds.data;
                                 return(ires);
                                 }
                              break;

      case IPROP_MLIST_LIST: // List array
                              lpd = (LPDATASTRUCT)lp;
                              cidx = (SHORT)lpd->index[0].data;
                              lines = GetTotalLines(hwnd);
                              tbuf[0] = '\0';
                              lhptr = tbuf;
                              if(cidx>= 0 && cidx<(SHORT)lines) {
                                 tlen = (SHORT)SendMessage(hwnd,
                                                   LB_GETTEXTLEN, cidx, 0L);
                                 // In most cases, we can use the small buffer
                                 // tbuf - but in cases of longer strings,
                                 // we allocate a temporary buffer in the heap
                                 if(tlen>= sizeof(tbuf)) {
                                    lhnd = LocalAlloc(LMEM_FIXED, tlen+1);
                                    if(lhnd) lhptr = (LPSTR)LocalLock(lhnd);
                                    }
                                 SendMessage(hwnd, LB_GETTEXT, cidx,
                                             (LONG)lhptr);
                                 }
                              newhsz = VBCreateHsz(FP_SEG(hctl), lhptr);
                              lpd->data = (LONG)newhsz;
                              if(lhnd) LocalUnlock(lhnd);
                              break;

      case IPROP_MLIST_SELSTRING:   // Write only properties
      case IPROP_MLIST_ADDSCROLLED:
                              return(WriteOnlyProp());

      case IPROP_MLIST_LISTINDEX:
                              phc = VBDerefControl(hctl);
                              // Valid only in single select mode
                              if(phc->boxmode>0) return(InvalidProp());
                              lines = (WORD)SendMessage(hwnd, LB_GETCURSEL,
                                                        0,0L);
                              *((SHORT FAR *)lp) = lines;
                              break;

      case IPROP_MLIST_LISTCOUNT:
                              *((LPWORD)lp) = GetTotalLines(hwnd);
                              break;

      case IPROP_MLIST_LISTLEN:
                              lpd = (LPDATASTRUCT)lp;
                              cidx = (SHORT)lpd->index[0].data;
                              // Now verify the index
                              lines = GetTotalLines(hwnd);
                              if(cidx<0 || cidx>=(SHORT)lines) return(380);
                              slen = (SHORT)SendMessage(hwnd, LB_GETTEXTLEN,
                                                        cidx, 0L);
                              if(slen<0) return(380);
                              lpd->data = (LONG)slen;
                              break;

      case IPROP_MLIST_LINESVISIBLE:   // Get number of visible lines
                              *((LPWORD)lp) = GetVisibleLines(hwnd);
                              break;

      case IPROP_MLIST_TOPINDEX: // Get index of top visible line
                              slen = (short)SendMessage(hwnd, LB_GETTOPINDEX,
                                                         0,0L);
                              if(slen<0) return(380);
                              *((LPWORD)lp) = (WORD)slen;
                              break;

      case IPROP_MLIST_TABLOCS:  // Get tab entry
                              lpd = (LPDATASTRUCT)lp;
                              cidx = (SHORT)lpd->index[0].data;
                              if(cidx>32 || cidx<0) return(380);
                              phc = VBDerefControl(hctl);
                              lpd->data = (LONG)phc->tablocs[cidx];
                              break;

      case IPROP_MLIST_SELECTED: // Get selected status of the entry
                                 // specified by the property array index.
                              lpd = (LPDATASTRUCT)lp;
                              cidx = (SHORT)lpd->index[0].data;
                              phc = VBDerefControl(hctl);
                              if(phc->boxmode==0) return(InvalidProp());
                              tlen = (short)SendMessage(hwnd, LB_GETSEL,
                                                        cidx,0L);
                              if(tlen<0) return(380);
                              lpd->data = (tlen>0)? -1:0;
                              break;

      case IPROP_MLIST_SELCOUNT:
                              phc = VBDerefControl(hctl);
                              if(phc->boxmode==0) {   // Single select box
                                 tlen = (SHORT)SendMessage(hwnd,LB_GETCURSEL,
                                                           0, 0L);
                                 tlen = (tlen<0)? 0:1;
                                 }
                              else {   // Multi-Select box
                                 tlen = (short)SendMessage(hwnd,
                                                      LB_GETSELCOUNT, 0,0L);
                                 }
                              if(tlen<0) return(380);
                              *((SHORT FAR *)lp) = tlen;
                              break;


      case IPROP_MLIST_ITEMDATA: // List array
                              lpd = (LPDATASTRUCT)lp;
                              cidx = (SHORT)lpd->index[0].data;
                              lines = GetTotalLines(hwnd);
                              if(cidx<0 || cidx>=(SHORT)lines)
                                 return(381);
                              lpd->data = VBSendControlMsg(hctl,
                                          LB_GETITEMDATA, cidx, 0L);
                              break;

      case IPROP_MLIST_ITEMBACKCOLOR:
      case IPROP_MLIST_ITEMFORECOLOR:
      case IPROP_MLIST_ITEMIMAGE:
      case IPROP_MLIST_ITEMSTRIKETHRU:
                              lpd = (LPDATASTRUCT)lp;
                              cidx = (SHORT)lpd->index[0].data;
                              lines = GetTotalLines(hwnd);
                              if(cidx<0 || cidx>=(SHORT)lines)
                                 return(381);
                              lpattr = LockItem(hctl, cidx);
                              if(!lpattr) return(380);
                              switch(wp) {
                                 case IPROP_MLIST_ITEMBACKCOLOR:
                                       lpd->data = lpattr->cBack;
                                       break;
                                 case IPROP_MLIST_ITEMFORECOLOR:
                                       lpd->data = lpattr->cFore;
                                       break;
                                 case IPROP_MLIST_ITEMIMAGE:
                                       lpd->data = lpattr->hPic;
                                       break;
                                 case IPROP_MLIST_ITEMSTRIKETHRU:
                                       lpd->data = (lpattr->flags & 0x04)?
                                                   -1: 0;
                                       break;
                                 }
                              UnlockItem(hctl, cidx);
                              break;


      default: dodefault = TRUE;
                     break;
      }
   if(lhnd) LocalFree(lhnd);
   if (dodefault) return(VBDefControlProc( hctl, hwnd, VBM_GETPROPERTY,
                                             wp, lp ));
   else return(0);
}

/*

 Function Name:   LONG DoCheckProperty(HCTL hctl, HWND hwnd, USHORT wp, LONG lp)

 Parameters:      hctl = Control handle
                  hwnd = Window handle of control
                  wp, lp = WORD and LONG parameters depending on property

 Description:     This function handles the VBM_CHECKPROPERTY messages.
                  This message is called before VBM_SETPROPERTY when the
                  PF_SETCHECK flag is set for the property
*/

LONG DoCheckProperty(HCTL hctl, HWND hwnd, USHORT wp, LONG lp)
{
   BOOL dodefault = FALSE; // Set TRUE to perform default processing
   LPDATASTRUCT lpd;
   WORD lines;
   PMLIST phc;
   short cidx;
   PIC pic;


   switch(wp) {
      case IPROP_MLIST_LISTINDEX:   // Only valid for single select box.
                                    // Check for valid array index.
                                    phc = VBDerefControl(hctl);
                                    if(phc->boxmode>0) return(InvalidProp());
                                    lines = GetTotalLines(hwnd);
                                    if((short)lp >= (short)lines)
                                       return(381);
                                    break;

      case IPROP_MLIST_SELECTED:    // Only valid for multi-select box
                                    // Check for valid array index
                                    lpd = (LPDATASTRUCT)lp;
                                    cidx = (SHORT)lpd->index[0].data;
                                    phc = VBDerefControl(hctl);
                                    if(phc->boxmode==0)
                                       return(InvalidProp());
                                    lines = GetTotalLines(hwnd);
                                    if( (cidx >= (SHORT)lines) ||
                                        (cidx< -1) )
                                       return(381);
                                    break;

      case IPROP_MLIST_SELSTRING:   // Only valid for single select box.
                                    // Check for valid array index.
                                    phc = VBDerefControl(hctl);
                                    if(phc->boxmode>0) return(InvalidProp());
                                    lpd = (LPDATASTRUCT)lp;
                                    cidx = (SHORT)lpd->index[0].data;
                                    if(cidx<-1 ||
                                       cidx>= (short)(GetTotalLines(hwnd)))
                                       return(381);
                                    break;

      case IPROP_MLIST_MAXSCROLLED: // Must be postive and >= 1
                                    if((SHORT)lp <= 0) return(380);
                                    break;

      case IPROP_MLIST_ADDSCROLLED:
                                    phc= VBDerefControl(hctl);
                                    // Can't do scrolled add to sorted box
                                    if(phc->redrawstate & 0x02)
                                       return(InvalidProp());
                                    break;

      case IPROP_MLIST_ITEMDATA:    // Check for valid array index.
                                    lpd = (LPDATASTRUCT)lp;
                                    cidx = (SHORT)lpd->index[0].data;
                                    lines = GetTotalLines(hwnd);
                                    if((short)cidx >= (short)lines)
                                       return(381);
                                    break;

      case IPROP_MLIST_ITEMFORECOLOR: // Check for valid array index.
      case IPROP_MLIST_ITEMBACKCOLOR: // Check for valid array index.
      case IPROP_MLIST_ITEMSTRIKETHRU:
                                    lpd = (LPDATASTRUCT)lp;
                                    cidx = (SHORT)lpd->index[0].data;
                                    phc = VBDerefControl(hctl);
                                    lines = GetTotalLines(hwnd);
                                    if((short)cidx >= (short)lines)
                                       return(381);
                                    break;

      case IPROP_MLIST_ITEMIMAGE:   // Valid index and image type
                                    lpd = (LPDATASTRUCT)lp;
                                    cidx = (SHORT)lpd->index[0].data;
                                    phc = VBDerefControl(hctl);
                                    lines = GetTotalLines(hwnd);
                                    if((short)lp >= (short)lines)
                                       return(381);

                                    VBGetPic((HPIC)lpd->data, &pic);
                                    switch(pic.picType) {
                                       case PICTYPE_BITMAP:
                                       case PICTYPE_NONE:
                                                return(0);  // ok

                                       default: return(380);
                                       }

                                    break;

      default: dodefault = TRUE;
                     break;
      }
   if (dodefault) return(VBDefControlProc( hctl, hwnd, VBM_CHECKPROPERTY,
                                             wp, lp ));
   else return(0);
}

/*

 Function Name:   LONG DoMethod(HCTL hctl, HWND hwnd, USHORT wp, LONG lp)

 Parameters:      hctl = Control handle
                  hwnd = Window handle of control
                  wp, lp = WORD and LONG parameters depending on property

 Description:     This function handles the VBM_METHOD messages.
                  Note that on sorted listboxes, we always preserve the sort
                  order even when an index is specified.  This may differ
                  from the standard control.
*/

LONG DoMethod(HCTL hctl, HWND hwnd, USHORT wp, LONG lp)
{
   LPDWORD cargs;
   HSZ FAR *ppstring;
   LPDWORD pindex;
   LPSTR pstring;
   BOOL dodefault = FALSE;
   PMLIST phc;
   short useindex;
   DWORD res;

   switch(wp) {
      case METH_ADDITEM:   // AddItem method
                           cargs = (LPDWORD)lp;
                           // Must have at least one parameter
                           if(*cargs==1) { dodefault = TRUE; break; }
                           phc = VBDerefControl(hctl);
                           ppstring = (HSZ FAR *)(cargs+1);
                           // If index is specified, use it, otherwise add
                           // to the end of the list (useindex = -1)
                           if(*cargs>2) {
                              pindex = cargs+2; useindex = (WORD)*pindex;
                              }
                           else {
                              pindex = NULL; useindex = -1;
                              }

                           /* Add the string item */
                           pstring = VBDerefHsz(*ppstring);
                           if(useindex>=0 && !(phc->redrawstate & 0x02)) {
                              // Index exists and it is not a sorted box
                              res = SendMessage(hwnd, LB_INSERTSTRING,
                                                useindex, (LONG)pstring);
                              }
                           else {   // No index or sorted
                              res = SendMessage(hwnd, LB_ADDSTRING,
                                                0, (LONG)pstring);
                              }
                           if(res==LB_ERR) return(5);
                           if(res==LB_ERRSPACE) return(7);
                           else return(0);
                           
      case METH_REMOVEITEM:
                           cargs = (LPDWORD)lp;
                           if(*cargs==1) { dodefault = TRUE; break; }
                           pindex = cargs+1; useindex = (WORD)*pindex;
                           res = SendMessage(hwnd, LB_DELETESTRING,
                                             useindex, 0L);
                           if(res==LB_ERR) return(5);
                           else return(0);

      case METH_CLEAR:     SendMessage(hwnd, LB_RESETCONTENT, 0, 0L);
                           DeleteAllItems(hctl, FALSE);
                           return(0);

      default: dodefault = TRUE; break;   // Standard methods use default
      }
   if (dodefault) return(VBDefControlProc( hctl, hwnd, VBM_METHOD, wp, lp ));
   else return(0);
}
//---------------------------------------------------------------------------
// Event Procedure Parameter Profiles
//---------------------------------------------------------------------------
typedef struct tagPARAMS
{
    LPVOID  Index;          // Reserve space for index parameter to array ctl
} PARAMS;

typedef struct tagSPARAMS
{
   LPWORD   ScrollPos;
   LPWORD   ScrollType;
   LPVOID   Index;
} SPARAMS;


//---------------------------------------------------------------------------
// Mlist Control Procedure
//---------------------------------------------------------------------------

LONG _export FAR PASCAL MlistCtlProc(HCTL hctl, HWND hwnd, USHORT msg, USHORT wp,
                                     LONG lp)
{
   LONG lResult;
   PMLIST phc;
   WORD latestentry;
   LPITEMATTR lpattr;
   DWORD oldbackcolor;

   // Message pre-processing
   switch( msg ) {

      case WM_NCCREATE: {  // This message is sent before the window is
                           // fully created.  It is the only place where
                           // you can set windows style bits and be certain
                           // that they will take effect.
                           // We modify the style bits based on the control
                           // structure properties.
                           LPCREATESTRUCT lpcs;
                           // lpcs points to the windows initialization struct
                           lpcs = (LPCREATESTRUCT)lp;
                           phc = VBDerefControl(hctl);
                           if (phc->boxmode==1)
                              lpcs->style |= LBS_MULTIPLESEL;
                           if (phc->boxmode==2) lpcs->style |=
                                 LBS_MULTIPLESEL | LBS_EXTENDEDSEL;
                           if (phc->redrawstate & 0x02)
                              lpcs->style |= LBS_SORT;
                           if (phc->multicolumn >0) {
                              lpcs->style &= ~LBS_OWNERDRAWVARIABLE;
                              lpcs->style |= (LBS_MULTICOLUMN | 
                                       LBS_OWNERDRAWFIXED |
                                       WS_VSCROLL | WS_HSCROLL);
                              }
                           SetWindowLong(hwnd, GWL_STYLE, lpcs->style);
                           break;
                        }



                              // These messages simply hand off to the
                              // appropriate function.
      case VBM_SETPROPERTY:   return(DoSetProperty(hctl, hwnd, wp, lp));
      case VBM_CHECKPROPERTY: return(DoCheckProperty(hctl, hwnd, wp, lp));
      case VBM_GETPROPERTY:   return(DoGetProperty(hctl, hwnd, wp, lp));
      case VBM_METHOD:        return(DoMethod(hctl, hwnd, wp, lp));

      case VBM_CREATED: // Once the window is fully created and all properties
                        // have been loaded, we set the control attributes
                        // according to the properties
                        InitializeWindowData(hctl, hwnd);
                        Initializing = FALSE;   // Clear flag
                        phc = VBDerefControl(hctl);

                        // We can't allow palette based background color
                        // because we explicitely set the background
                        // during the owner draw.
                        VBGetControlProperty(hctl, IPROP_MLIST_BACKCOLOR, 
                                             &oldbackcolor);
                        if(oldbackcolor & 0xff000000L) {
                           VBSetControlProperty(hctl, IPROP_MLIST_BACKCOLOR, 
                                                0x00ffffffL);
                           }
                        break;

      case VBM_GETPROPERTYHSZ:   // Handle the about box property
                              switch(wp) {
                                 case IPROP_MLIST_ABOUT:
                                          *((HSZ FAR *)lp) =
                                             VBCreateHsz(FP_SEG(hctl),
                                             "Click here for About box -->");
                                          return(0);
                                 default: break;
                                 }
                              break;
                     
      case VBM_INITIALIZE: // Set all default values here
                           // Also set the initializing flag
                           phc = VBDerefControl(hctl);
                           phc->redrawstate |=1;
                           phc->maxscrolled = 100; // Default scroll buffer
                           phc->PrivateHeap = NULL;
                           // DeleteAllItems serves to create the private
                           // heap
                           DeleteAllItems(hctl, FALSE);
                           Initializing = TRUE;
                           break;

      case VBM_INITPROPPOPUP: // Handle the about box
                           switch(wp) {
                              case IPROP_MLIST_ABOUT:
                                    return(HwndInitAnyPopup(IBOX_ABOUT));
                              }
                           break;

      case CM_SETCTLNAME:  // Update 1st line in listbox to match ctlname
                           SetControlName(hctl,hwnd);
                           return(0);

	   case VBN_COMMAND:    // Process custom list events
                  	      switch ( HIWORD(lp) ) {
                     		   case LBN_SELCHANGE:  {  // Click event
                     		      PARAMS params;
                                 params.Index = NULL;
                     		      VBFireEvent( hctl, EVENT_MLIST_CLICK,
                                              &params );
                     		      break;
                                 }
                     		   case LBN_DBLCLK:  {  // DblClick event
                     		      PARAMS params;
                                 params.Index = NULL;
                     		      VBFireEvent( hctl, EVENT_MLIST_DBLCLICK,
                                              &params );
                     		      break;
                                 }
                     		   }
                  	      return(0L);

                           // We defer processing of scroll events.  While
                           // it IS possible to trigger the event right
                           // away, there is a risk of some unknown interaction
                           // between the executing basic code and the 
                           // functioning of the control.  Deferred processing
                           // is much safer.  
      case WM_VSCROLL:     PostMessage(hwnd, CM_VSCROLL, wp, lp);
                           break;
      case WM_HSCROLL:     PostMessage(hwnd, CM_HSCROLL, wp, lp);
                           break;

      case CM_VSCROLL:     // Trigger scroll events
      case CM_HSCROLL:     {
                              SPARAMS params;
                              WORD newpos;
                              params.Index = NULL;
                              params.ScrollType = (LPWORD)&wp;
                              switch(wp) {
                                 case SB_THUMBPOSITION:
                                 case SB_THUMBTRACK:
                                             newpos = LOWORD(lp);
                                             break;
                                 default: newpos = 0;
                                 }
                              params.ScrollPos = (LPWORD)&newpos;
                     		   VBFireEvent( hctl, (msg==CM_VSCROLL)?
                                          EVENT_MLIST_VSCROLL:
                                          EVENT_MLIST_HSCROLL,
                                          &params );
                              return(0L);
                              }
      case LB_GETITEMDATA:
                           lpattr = LockItem(hctl, wp);
                           if(!lpattr) return(LB_ERR);
                           lResult = lpattr->itemdata;
                           UnlockItem(hctl, wp);
                           return(lResult);

      case LB_SETITEMDATA:
                           lpattr = LockItem(hctl, wp);
                           if(!lpattr) return(LB_ERR);
                           lpattr->itemdata = lp;
                           UnlockItem(hctl, wp);
                           return(0L);

      case LB_DELETESTRING:   // Delete the item
                              DeleteItem(hctl, wp);
                              break;


   	case VBN_DRAWITEM:
                     	   // Draw the specified item in the listbox.
                     	   DrawItem(hctl, (LPDRAWITEMSTRUCT)lp);
                     	   return 0;
   
   	case VBN_MEASUREITEM:
                           // Return the height of the specified item in the listbox.
                           MeasureItem(hctl, (LPMEASUREITEMSTRUCT)lp);
                           return 0;

      //case LB_SETTOPINDEX:
      //         SendMessage(hwnd, WM_SETREDRAW, (WORD)0, 0);
      //         break;


      }

   // Default processing:
   lResult = VBDefControlProc( hctl, hwnd, msg, wp, lp );

   // Message post-processing:
   switch( msg ) {
      case WM_DESTROY:
                        DeleteAllItems(hctl, TRUE);
                        break;

      case LB_ADDSTRING:
      case LB_INSERTSTRING:
                        if(lResult<0) break; // Error condition

                        phc = VBDerefControl(hctl);
                        
                        // Save most recently added index
                        phc->newindex = (WORD)lResult;

                        // Clear the item data for the new entry
                        // Windows does not do it for us
                        latestentry = (WORD)VBSuperControlProc(hctl, 
                                          LB_SETITEMDATA, phc->newindex, 0L);
                        break;
      case WM_SETFONT:  { // Record the height of the font in pixels
                           HDC usedc;
                           HFONT oldfont;
                           TEXTMETRIC tc;
                           usedc = GetDC(hwnd);
                           oldfont = SelectObject(usedc, (HFONT)wp);
                           GetTextMetrics(usedc, (LPTEXTMETRIC)&tc);
                           SelectObject(usedc, oldfont);
                           ReleaseDC(hwnd, usedc);
                           phc = VBDerefControl(hctl);
                           phc->fontheight = tc.tmHeight;
                           UpdateAllHeights(hctl);
                           break;
                           }
                        break;




      }

   return lResult;
}



/*

 Function Name:   HWND HwndInitAnyPopup(WORD dlgid)

 Parameters:      dlgid = Resource ID for the dialog template to use

 Description:     VBM_INITPROPPOPUP requires the handle to an invisible
                  window that will receive ShowWindow messages when the
                  dialog (specified by dlgid) is to be displayed.
                  This function creates the invisible window.

 Returns:         The window handle of the invisible window

 Notes:        
*/

HWND HwndInitAnyPopup(WORD dlgid)
{
   static BOOL fRegistered = FALSE;
   HWND hwnd;

   if (!fRegistered) {
      WNDCLASS class;

      class.style	= CS_GLOBALCLASS;
      class.lpfnWndProc = (WNDPROC)AnyPopupWndProc;
      class.cbClsExtra = 0;
      class.cbWndExtra = 2;   // We put the dialog template here
      class.hInstance = hmodDLL; // VBGetHInstance();
      class.hIcon = NULL;
      class.hCursor = NULL;
      class.hbrBackground = NULL;
      class.lpszMenuName  = NULL;
      class.lpszClassName = "MListAnyPopup";
      if (!RegisterClass(&class)) return NULL;
      fRegistered = TRUE;
      }

    // NOTE: May return NULL!
   hwnd = CreateWindow("MListAnyPopup", NULL, WS_POPUP, 0, 0, 0, 0,
               			 NULL, NULL, VBGetHInstance(), NULL );
   if(!hwnd) return(hwnd);
   SetWindowWord(hwnd, 0, dlgid);   /* Save dialog box ID */
   return(hwnd);
}


//---------------------------------------------------------------------------
// Window procedure used for invisible window that supports dialog boxes
//---------------------------------------------------------------------------
LONG _export FAR PASCAL AnyPopupWndProc(HWND hwnd, USHORT msg, USHORT wp,
                                        LONG lp)
{
   extern HANDLE hmodDLL;
   WORD dlgid;

   switch (msg) {
      case WM_SHOWWINDOW: if (wp) {
                        		PostMessage(hwnd, CM_OPENANYDLG, 0, 0L);
                        		return 0L;
                        		}
                           break;

   	case CM_OPENANYDLG:
         dlgid = GetWindowWord(hwnd, 0);
         // Note, we can use different DLG functions based on dlgid if
         // we wish.
   	   VBDialogBoxParam(hmodDLL, (LPSTR)MAKEINTRESOURCE(dlgid),
                           (FARPROC)AboutDlgProc, 0L);
         return 0L;
      }

   DefWindowProc(hwnd, msg, wp, lp);
}

//--------------------------------------------------------------
// Dialog box window procedure
//--------------------------------------------------------------

BOOL _export FAR PASCAL AboutDlgProc(HWND hDlg, USHORT msg, USHORT wp,
                                     LONG lp)
{
   switch (msg) {
      case WM_INITDIALOG:
                           return(TRUE);
      case WM_COMMAND:
               if(wp==IDOK) {
                  EndDialog(hDlg, 0);
                  return(TRUE);
                  }
               break;
      }
    return(FALSE);
}
