/***********************************************************************
   File mlist.h

   Multiple Feature listbox header file

   Copyright (c) Desaware 1992 - All rights reserved

   The following sample code is provided as-is with no warranty
   as to operation, performance or the suitability of this control to be
   used in any application.

   See the readme.txt file provided with this control for more
   information.

************************************************************************/

#define CM_OPENANYDLG      WM_USER+100
#define CM_SETCTLNAME      WM_USER+101
#define CM_HSCROLL         WM_USER+102 // Delayed scroll messages
#define CM_VSCROLL         WM_USER+103


#define IBOX_ABOUT   100

#define OFFSETIN( struc, field )  ( (USHORT)&( ((struc *)0)->field ) )


//---------------------------------------------------------------------------
// Resource ID's
//---------------------------------------------------------------------------
// Toolbox bitmap resource IDs 
//---------------------------------------------------------------------------
#define IDBMP_MLIST             8000
#define IDBMP_MLISTDOWN         8001
#define IDBMP_MLISTMONO         8003
#define IDBMP_MLISTMONODOWN     8004
#define IDBMP_MLISTEGA          8006
#define IDBMP_MLISTEGADOWN      8007


//---------------------------------------------------------------------------
// MLIST control data and structs
//---------------------------------------------------------------------------
// This structure contains data associated with a control.  This data is
// preserved when a window is recreated.  Some of this data is saved
// with the control in the VB application's .EXE file.

typedef struct tagMLIST
{
   BYTE boxmode;        /* 0 - Single select list box
                           1 - Multiple select list box
                           2 - Extended mode multiple select list box
                        */
   short tablocs[34];   /* Up to 32 tabs allowed, 0 terminates list.
                           Tabs are specified in characters. */

   BYTE redrawstate;    /* Bit 0 = 1 if redraw is set
                                   0 if redraw is off
                               1 = 1 if sorted, 0 if not sorted
                        */
   short multicolumn;   /* Size in dialog units of columns */
   short virtualwidth;  /* Size in dialog units of virtual list box
                           0 to cancel horizontal scroll bar */
   short maxscrolled;         // Maximum # of entries during scrolling -
                              //    minimum of 1
   HWND scrollhwnd;           // Scroll window
   GLOBALHANDLE PrivateHeap;  // Private heap
   LPSTR privptr;             // Pointer to Locked Private Heap (when locked)
   WORD newindex;             // Most recently added string location
   WORD fontheight;           // Height in pixels of the current font
}
MLIST;

typedef MLIST FAR * PMLIST;

#define MLISTDEREF( hctl )   ((PMLIST)VBDerefControl(hctl))


//-------------------------------------------------------------------------
// ItemAttr structure - 
//    holds data that is unique to a list box entry
//-------------------------------------------------------------------------

typedef struct tagItemAttr
{
   WORD flags;       // Bit 0 = 1 - Background valid
                     // Bit 1 = 1 - Text color valid
                     // Bit 2 = Do a strikethrough
   COLORREF cBack;   // Background color
   COLORREF cFore;   // Text color
   HPIC hPic;        // Picture
   DWORD itemdata;   // User defined
} ItemAttr;

typedef ItemAttr FAR *LPITEMATTR;

// We'll initialize the private heap at this number of entries
#define INITHEAPSIZE    32 * sizeof(ItemAttr)

//---------------------------------------------------------------------------
// Control Procedure
//---------------------------------------------------------------------------
extern LONG _export FAR PASCAL MlistCtlProc( HCTL, HWND, USHORT, USHORT, LONG );
extern BOOL _export FAR PASCAL AboutDlgProc(HWND, USHORT, USHORT, LONG);
extern LONG _export FAR PASCAL AnyPopupWndProc(HWND, USHORT, USHORT, LONG);
HWND HwndInitAnyPopup(WORD);

extern LPITEMATTR LockItem(HCTL hctl, WORD itemnum);
extern BOOL UnlockItem(HCTL hctl, WORD itemnum);
extern VOID DeleteItem(HCTL hctl, WORD itemnum);
extern VOID DeleteAllItems(HCTL hctl, BOOL stoprecreate);
extern VOID DrawItem(HCTL hctl, LPDRAWITEMSTRUCT lpdi);
extern VOID MeasureItem(HCTL hctl, LPMEASUREITEMSTRUCT lpMI);

extern DWORD DSLocalAlloc(WORD hseg, WORD size);
extern short DSLocalFree(DWORD lhnd);
extern WORD DSLocalSize(DWORD lhnd);
extern LPVOID DSLocalLock(DWORD lhnd);
extern BOOL DSLocalUnlock(DWORD lhnd);
extern WORD GetHeight(HCTL hctl, WORD itemnum);
extern VOID UpdateAllHeights(HCTL);




#ifndef RC_INVOKED
//---------------------------------------------------------------------------
// Property list
//---------------------------------------------------------------------------
// Define the consecutive indicies for the properties
// These are used to make the source file more readable
//---------------------------------------------------------------------------
#define IPROP_MLIST_NAME               0
#define IPROP_MLIST_INDEX              1
#define IPROP_MLIST_PARENT             2
#define IPROP_MLIST_BACKCOLOR          3
#define IPROP_MLIST_FORECOLOR          4
#define IPROP_MLIST_LEFT               5
#define IPROP_MLIST_TOP                6
#define IPROP_MLIST_WIDTH              7
#define IPROP_MLIST_HEIGHT             8
#define IPROP_MLIST_ENABLED            9
#define IPROP_MLIST_VISIBLE            10
#define IPROP_MLIST_MOUSEPOINTER       11
#define IPROP_MLIST_FONTNAME           12
#define IPROP_MLIST_FONTSIZE           13
#define IPROP_MLIST_FONTBOLD           14
#define IPROP_MLIST_FONTITALIC         15
#define IPROP_MLIST_FONTSTRIKE         16
#define IPROP_MLIST_FONTUNDER          17
#define IPROP_MLIST_DRAG               18
#define IPROP_MLIST_DRAGICON           19
#define IPROP_MLIST_TABINDEX           20
#define IPROP_MLIST_TABSTOP            21
#define IPROP_MLIST_TAG                22
#define IPROP_MLIST_TEXT               23
#define IPROP_MLIST_LIST               24
#define IPROP_MLIST_LISTINDEX          25
#define IPROP_MLIST_LISTCOUNT          26
#define IPROP_MLIST_LISTTYPE           27
#define IPROP_MLIST_LISTLEN            28
#define IPROP_MLIST_LINESVISIBLE       29
#define IPROP_MLIST_TOPINDEX           30
#define IPROP_MLIST_TABLOCS            31
#define IPROP_MLIST_REDRAW             32
#define IPROP_MLIST_SELECTED           33
#define IPROP_MLIST_SELSTRING          34
#define IPROP_MLIST_SORTED             35
#define IPROP_MLIST_ABOUT              36
#define IPROP_MLIST_COLUMNS            37
#define IPROP_MLIST_SELCOUNT           38
#define IPROP_MLIST_VIRTUALWIDTH       39
#define IPROP_MLIST_MAXSCROLLED        40
#define IPROP_MLIST_ADDSCROLLED        41
#define IPROP_MLIST_HWND               42
#define IPROP_MLIST_ITEMDATA           43
#define IPROP_MLIST_NEWINDEX           44
#define IPROP_MLIST_ITEMFORECOLOR      45
#define IPROP_MLIST_ITEMBACKCOLOR      46
#define IPROP_MLIST_ITEMIMAGE          47
#define IPROP_MLIST_ITEMSTRIKETHRU     48


#ifdef CTL_DATA

// List - set or retrieve text entry at the position indicated
PROPINFO propinfoList =
{
    "List",
    DT_HSZ | PF_fGetMsg|PF_fSetMsg|PF_fNoShow | PF_fPropArray,
    0, 0
};

// ListIndex - Single select only
// set or retrieve the currently selected entry. -1 if none selected
PROPINFO propinfoListIndex =
{
    "ListIndex",
    DT_SHORT | PF_fGetMsg | PF_fSetMsg | PF_fSetCheck | PF_fNoShow,
    0, 0
};

// ListCount - Read only - Return number of entries in the listbox
PROPINFO propinfoListCount =
{
    "ListCount",
    DT_SHORT | PF_fGetMsg | PF_fNoShow | PF_fNoRuntimeW,
    0, 0
};

// ListLength - Read only 
// Returns length of the string at the index specified by the array
PROPINFO propinfoListLen =
{
    "ListLength",
    DT_SHORT | PF_fGetMsg| PF_fNoShow|PF_fNoRuntimeW | PF_fPropArray,
    0, 0
};

// LinesVisible - Read Only
// Returns number of lines of text visible in the list box. 
PROPINFO propinfoLinesVisible =
{
    "LinesVisible",
    DT_SHORT | PF_fGetMsg| PF_fNoShow|PF_fNoRuntimeW,
    0, 0
};

// Text - Single select only, Read only
// Retrieve value of List(ListIndex)
PROPINFO propinfoText =
{
    "Text",
    DT_HSZ | PF_fGetMsg | PF_fNoShow | PF_fNoRuntimeW,
    0, 0
};

// TopIndex - 
// Set or retreive the index of the topmost line shown in the listbox.
PROPINFO propinfoTopIndex =
{
    "TopIndex",
    DT_SHORT | PF_fGetMsg|PF_fSetMsg| PF_fNoShow,
    0, 0
};

// TabLocs - Set or retreive up to 32 tab stops.  
// Tab stop being accessed is specified by the array index.
// Tab value is measured in average character width of the current font.
PROPINFO propinfoTabLocs =
{
    "TabLocs",
    DT_SHORT | PF_fGetMsg|PF_fSetMsg|PF_fNoShow|PF_fPropArray,
    0, 0
};

// Redraw 0 - Set or retrieve value of the redraw bit.  
// Useful for turning off control updates when performing many 
// operations at once.  Turning redraw on refreshes the listbox.
PROPINFO propinfoRedraw =
{
    "Redraw",
    DT_BOOL | PF_fGetData|PF_fSetData|PF_fSetMsg|PF_fSaveData|PF_fDefVal,
    OFFSETIN(MLIST,redrawstate),0x10, 1,0
};

// Selected - Multi-Select only
// Set or retreive the selected status of any entry in the listbox.
// Entry being accessed is specified by array index.
// TRUE = select, FALSE = deselect.
PROPINFO propinfoSelected =
{
    "Selected",
    DT_BOOL | PF_fGetMsg|PF_fSetMsg|PF_fSetCheck |PF_fNoShow|PF_fPropArray,
    0, 0
};

// SelectString - Write only, Single select listbox only
// Change current selection to the first string with specified text. 
// Search begins at the starting point specified by the array index.
PROPINFO propinfoSelString =
{
    "SelectString",
    DT_HSZ | PF_fSetMsg | PF_fGetMsg|PF_fNoShow | PF_fPropArray,
    0, 0
};


// Sorted - Set or retreive value of the 'sorted' flag.
// Setting this property clears the contents of the listbox.
PROPINFO propinfoSorted =
{
    "Sorted",
    DT_BOOL | PF_fGetData|PF_fSetData|PF_fSetMsg|PF_fSaveData|PF_fDefVal|
              PF_fPreHwnd,
    OFFSETIN(MLIST,redrawstate),0x11, 0,0
};

// ListType - Set & retreive type of list box. (See enumeration list)
// Setting this property clears the contents of the list box.
PROPINFO propinfoListType =
{
    "ListType",
    DT_ENUM | PF_fGetData| PF_fSetMsg|PF_fSaveData|PF_fDefVal|
               PF_fPreHwnd,
    OFFSETIN(MLIST,boxmode),0, 0,
    "Single-Select\0Multi-Select\0Extended Multi-Select\0",2
};

// (About) - Design time only - Brings up the about box
PROPINFO propinfoAbout =
{
    "(About)",
    DT_SHORT | PF_fNoRuntimeR | PF_fNoRuntimeW | PF_fGetHszMsg,
    0, 0
};

// MultiColumn - Set or retreive the width of columns in the list box.
// Column width is specified in dialog units (1/4 average character
// width).  0 = No columns set.  All columns are the same width.
// If changing from no columns to columns or vice versa, the
// contents of the ListBox will be cleared by setting this property.
PROPINFO propinfoMulti =
{
    "MultiColumn",
    DT_SHORT | PF_fGetData| PF_fSetMsg|PF_fSaveData|PF_fDefVal|
              PF_fPreHwnd,
    OFFSETIN(MLIST,multicolumn),0, 0,0
};

// SelCount - Read Only
// Returns the number of entries selected in the listbox
PROPINFO propinfoSelCount =
{
    "SelCount",
    DT_SHORT | PF_fGetMsg| PF_fNoShow|PF_fNoRuntimeW,
    0, 0
};

// VirtualWidth - Sets or retreives the virtual width of the list box.
// If the VirtualWidth set is greater than the visible width of the
// listbox, a horizontal scrollbar will appear to allow scrolling
// of the listbox.  Has no effect in Multi-Column mode.
PROPINFO propinfoVirtualWidth =
{
    "VirtualWidth",
    DT_SHORT | PF_fGetData | PF_fSetData | PF_fSetMsg |
      PF_fDefVal | PF_fSaveData,
    OFFSETIN(MLIST,virtualwidth), 0, 0,0
};

// MaxScrolled - Specifies the number of entries in the listbox 
// to keep during scrolling (using the AddScrolled property)
PROPINFO propinfoMaxScrolled =
{
    "MaxScrolled",
    DT_SHORT | PF_fGetData | PF_fSetData |
      PF_fDefVal | PF_fSaveData,
    OFFSETIN(MLIST,maxscrolled), 0, 100, 0
};

// AddScrolled - Write only - Not valid on sorted listbox
// Adds the text string specified to the end of the listbox,
// scrolling up if the end of the listbox is visible.  If this addition 
// causes the total number of entries in the listbox to exceed 
// MaxScrolled, entries will be removed from the top of the listbox until 
// the number of entries is equal to MaxScrolled.
PROPINFO propinfoAddScrolled =
{
    "AddScrolled",
    DT_HSZ | PF_fSetMsg | PF_fGetMsg | PF_fSetCheck |PF_fNoShow,
    0, 0
};

// ItemData - set or retrieve user defined data at the position indicated
PROPINFO propinfoItemData =
{
    "ItemData",
    DT_LONG | PF_fGetMsg|PF_fSetMsg|PF_fNoShow | PF_fPropArray | 
              PF_fSetCheck,
    0, 0
};

// NewIndex - retrieve index of most recently added entry
PROPINFO propinfoNewIndex =
{
    "NewIndex",
    DT_SHORT | PF_fGetData | PF_fNoShow | PF_fNoRuntimeW,
    OFFSETIN(MLIST,newindex), 0, 0,0
};

PROPINFO propinfoItemForeColor =
{
    "ItemForeColor",
    DT_COLOR | PF_fGetMsg|PF_fSetMsg|PF_fNoShow | PF_fPropArray |
               PF_fSetCheck,
    0, 0
};

PROPINFO propinfoItemBackColor =
{
    "ItemBackColor",
    DT_COLOR | PF_fGetMsg|PF_fSetMsg|PF_fNoShow | PF_fPropArray |
               PF_fSetCheck,
    0, 0
};

PROPINFO propinfoItemImage =
{
    "ItemImage",
    DT_PICTURE | PF_fGetMsg|PF_fSetMsg|PF_fNoShow | PF_fPropArray | 
                 PF_fSetCheck,
    0, 0
};

// ItemStrikeThru - set a strikethrough for the text of this item
PROPINFO propinfoItemStrikeThru =
{
    "ItemStrikeThru",
    DT_BOOL | PF_fGetMsg|PF_fSetMsg|PF_fNoShow | PF_fPropArray | 
              PF_fSetCheck,
    0, 0
};

// This is the property array that is provided to the MODEL structure for
// this control.

PPROPINFO proplistMlist[] =
{
   PPROPINFO_STD_CTLNAME,
   PPROPINFO_STD_INDEX,
   PPROPINFO_STD_PARENT,
   PPROPINFO_STD_BACKCOLOR,
   PPROPINFO_STD_FORECOLOR,
   PPROPINFO_STD_LEFT,
   PPROPINFO_STD_TOP,
   PPROPINFO_STD_WIDTH,
   PPROPINFO_STD_HEIGHT,
   PPROPINFO_STD_ENABLED,
   PPROPINFO_STD_VISIBLE,
   PPROPINFO_STD_MOUSEPOINTER,
   PPROPINFO_STD_FONTNAME,
   PPROPINFO_STD_FONTSIZE,
   PPROPINFO_STD_FONTBOLD,
   PPROPINFO_STD_FONTITALIC,
   PPROPINFO_STD_FONTSTRIKE,
   PPROPINFO_STD_FONTUNDER,
   PPROPINFO_STD_DRAGMODE,
   PPROPINFO_STD_DRAGICON,
   PPROPINFO_STD_TABINDEX,
   PPROPINFO_STD_TABSTOP,
   PPROPINFO_STD_TAG,
   &propinfoText,
   &propinfoList,
   &propinfoListIndex,
   &propinfoListCount,
   &propinfoListType,
   &propinfoListLen,
   &propinfoLinesVisible,
   &propinfoTopIndex,
   &propinfoTabLocs,
   &propinfoRedraw,
   &propinfoSelected,
   &propinfoSelString,
   &propinfoSorted,
   &propinfoAbout,
   &propinfoMulti,
   &propinfoSelCount,
   &propinfoVirtualWidth,
   &propinfoMaxScrolled,
   &propinfoAddScrolled,
   PPROPINFO_STD_HWND,
   &propinfoItemData,
   &propinfoNewIndex,
   &propinfoItemForeColor,
   &propinfoItemBackColor,
   &propinfoItemImage,
   &propinfoItemStrikeThru,
   NULL
};
#endif  // CTL_DATA



//---------------------------------------------------------------------------
// Event procedure parameter prototypes
//---------------------------------------------------------------------------
#ifdef CTL_DATA
WORD Parms_I2[] = { ET_I2 };                   // 1 x I2 parm
WORD Parms_2xI2[] = { ET_I2, ET_I2 };          // 2 x I2 parm
WORD Parms_SD[] = { ET_SD };                   // 1 x SD parm

EVENTINFO Event_Click =
{
    "Click",
    0,
    0,
    NULL,
    NULL
};

EVENTINFO Event_DblClick =
{
    "DblClick",
    0,
    0,
    NULL,
    NULL
};

EVENTINFO Event_HScroll =
{
    "HScroll",
    2,
    4,
    Parms_2xI2,
    "ScrollType As Integer, Position As Integer"
};


EVENTINFO Event_VScroll =
{
    "VScroll",
    2,
    4,
    Parms_2xI2,
    "ScrollType As Integer, Position As Integer"
};
#endif  // CTL_DATA

//---------------------------------------------------------------------------
// Event list
//---------------------------------------------------------------------------
// Define the consecutive indicies for the events
// These are used to make the source file more readable
//---------------------------------------------------------------------------
#define EVENT_MLIST_CLICK        0
#define EVENT_MLIST_DBLCLICK     1
#define EVENT_MLIST_DRAGDROP     2
#define EVENT_MLIST_DRAGOVER     3
#define EVENT_MLIST_GOTFOCUS     4
#define EVENT_MLIST_HSCROLL      5
#define EVENT_MLIST_KEYDOWN      6
#define EVENT_MLIST_KEYPRESS     7
#define EVENT_MLIST_KEYUP        8
#define EVENT_MLIST_LOSTFOCUS    9
#define EVENT_MLIST_MOUSEDOWN    10
#define EVENT_MLIST_MOUSEMOVE    11
#define EVENT_MLIST_MOUSEUP      12
#define EVENT_MLIST_VSCROLL      13
#ifdef CTL_DATA

// Important!  Note how the events are in alphabetical order!

PEVENTINFO eventlistMlist[] =
{
    &Event_Click,
    &Event_DblClick,
    PEVENTINFO_STD_DRAGDROP,
    PEVENTINFO_STD_DRAGOVER,
    PEVENTINFO_STD_GOTFOCUS,
    &Event_HScroll,
    PEVENTINFO_STD_KEYDOWN,
    PEVENTINFO_STD_KEYPRESS,
    PEVENTINFO_STD_KEYUP,
    PEVENTINFO_STD_LOSTFOCUS,
    PEVENTINFO_STD_MOUSEDOWN,
    PEVENTINFO_STD_MOUSEMOVE,
    PEVENTINFO_STD_MOUSEUP,
    &Event_VScroll,
    NULL
};
#endif  // CTL_DATA


//---------------------------------------------------------------------------
// Model struct
//---------------------------------------------------------------------------
// Define the control model.
//---------------------------------------------------------------------------
#ifdef CTL_DATA
MODEL modelMlist =
{
   VB_VERSION,          // VB version being used
   MODEL_fFocusOk |     // This control can receive the focus
   MODEL_fInitMsg |     // control receives VBM_INITIALIZE messages
   MODEL_fLoadMsg |     // receives VBM_CREATED and VBM_LOADED messages
   MODEL_fArrows,       // This control processes arrows - i.e., arrows
                        // are not used to move between controls.
   (PCTLPROC)MlistCtlProc, // Specify control procedure
   CS_DBLCLKS |         // This control understands double-clicks
   CS_HREDRAW |         // Redraw this control when the horizontal
   CS_VREDRAW,          // or vertical size changes.
   WS_CHILD |           // This is a child window
   LBS_NOTIFY |         // This window notifies parent on click
   LBS_USETABSTOPS |    // This listbox supports tabs
   WS_BORDER |          // It has a border
   WS_HSCROLL |         // Show horizontal scroll bars as needed.
   LBS_OWNERDRAWVARIABLE |  // Owner draw variable
   LBS_HASSTRINGS,
   sizeof(MLIST),       // Allocate extra space for MLIST structure.
   IDBMP_MLIST,         // Toolbar Palette bitmap ID
   "Mlist",             // Default control name
   "dwMList2Box",       // Visual Basic class name for this control.
   "LISTBOX",           // Parent class name
   proplistMlist,       // Array of Properties for this control.
   eventlistMlist,      // Array of Events for this control.
   0,
   0
};
#endif  // CTL_DATA

#endif  // RC_INVOKED
