/***********************************************************************
   File mlist2.c

   Multiple Feature listbox main file

   Copyright (c) Desaware 1992 - All rights reserved

   The following sample code is provided as-is with no warranty
   as to operation, performance or the suitability of this control to be
   used in any application.

   See the readme.txt file provided with this control for more
   information.

************************************************************************/

#define NOCOMM
#include <windows.h>
#include "vbapi.h"
#include "mlist.h"
#include <dos.h>
#include <string.h>
/*

 Function Name:   LPITEMATTR LockItem(HCTL hctl, WORD itemnum)

 Parameters:      hctl = Control handle
                  itemnum = Item number to access

 Description:     Locks the item attribute and allows it to be edited.
                  Creates a new entry in necessary (and possible)

 Returns:         Pointer to the ItemAttr structure

 Notes:        
*/

LPITEMATTR LockItem(HCTL hctl, WORD itemnum)
{
   PMLIST phc;
   LOCALHANDLE lhnd;
   LPITEMATTR res;


   phc = VBDerefControl(hctl);
   if(!phc->PrivateHeap) {
      // Create the private heap on first call
      DeleteAllItems(hctl, FALSE);
      phc = VBDerefControl(hctl);
      if(!phc->PrivateHeap) return(NULL); // Should never happen
      }

   phc->privptr = GlobalLock(phc->PrivateHeap);

   // Now find out if there is a data item attached
   lhnd = (LOCALHANDLE)VBSuperControlProc(hctl, LB_GETITEMDATA, itemnum, 0L);
   if(!lhnd) { // None exists - create it
      lhnd = (LOCALHANDLE)DSLocalAlloc(FP_SEG(phc->privptr), sizeof(ItemAttr));
      VBSuperControlProc(hctl, LB_SETITEMDATA, itemnum, (LONG)lhnd);
      }
   if(lhnd) {  // Lock and get the pointer
      res = (LPITEMATTR)DSLocalLock(MAKELONG(lhnd, FP_SEG(phc->privptr)));
      return(res);   // Leave everything locked
      }
   // No handle, so unlock the global heap
   GlobalUnlock(phc->PrivateHeap);
   return(NULL);
}

/*

 Function Name:   BOOL UnlockItem(HCTL hctl, WORD itemnum)

 Parameters:      hctl = Control handle
                  itemnum = Item number to access

 Description:     UnLocks the item attribute.  Each LockItem must be
                  balanced with an UnlockItem

 Returns:         FALSE on success

 Notes:        
*/

BOOL UnlockItem(HCTL hctl, WORD itemnum)
{
   PMLIST phc;
   LOCALHANDLE lhnd;
   BOOL res;


   phc = VBDerefControl(hctl);
   if(!phc->PrivateHeap || !phc->privptr) return(NULL); // Should never happen

   // Now find out if there is a data item attached
   lhnd = (LOCALHANDLE)VBSuperControlProc(hctl, LB_GETITEMDATA, itemnum, 0L);
   if(!lhnd) { // None exists - this is an error
      return(TRUE);
      }
   res = DSLocalUnlock(MAKELONG(lhnd, FP_SEG(phc->privptr)));
   // Now unlock the global heap
   GlobalUnlock(phc->PrivateHeap);
   return(NULL);
}


/*

 Function Name:   VOID DeleteItem(HCTL hctl, WORD itemnum)

 Parameters:      hctl = Control handle
                  itemnum = Item number to access

 Description:     Deletes the item attribute.

 Notes:        
*/

VOID DeleteItem(HCTL hctl, WORD itemnum)
{
   PMLIST phc;
   LOCALHANDLE lhnd;
   BOOL res;
   LPITEMATTR lpattr;


   phc = VBDerefControl(hctl);
   if(!phc->PrivateHeap) return; // Should never happen

   // Now find out if there is a data item attached
   lhnd = (LOCALHANDLE)VBSuperControlProc(hctl, LB_GETITEMDATA, itemnum, 0L);
   if(!lhnd) { // None exists - no need to delete
      return;
      }

   phc->privptr = GlobalLock(phc->PrivateHeap);
   lpattr = (LPITEMATTR)DSLocalLock(MAKELONG(lhnd, FP_SEG(phc->privptr)));
   
   // Delete any objects in the structure here
   if(lpattr->hPic) VBFreePic(lpattr->hPic);

   // And unlock the handle
   DSLocalUnlock(MAKELONG(lhnd, FP_SEG(phc->privptr)));

   res = DSLocalFree(MAKELONG(lhnd, FP_SEG(phc->privptr)));
   // Now unlock the global heap
   GlobalUnlock(phc->PrivateHeap);
   return;
}

/*

 Function Name:   VOID DeleteAllItems(HCTL hctl, BOOL stoprecreate)

 Parameters:      hctl = Control handle
                  stoprecreate = TRUE to prevent initializeing a new heap

 Description:     Deletes all data items in the list by deleting then
                  recreateing the private heap

 Notes:        
*/

VOID DeleteAllItems(HCTL hctl, BOOL stoprecreate)
{
   PMLIST phc;
   WORD itemcount, x;
   phc = VBDerefControl(hctl);

   if(phc->PrivateHeap) {
      itemcount = (WORD)VBSendControlMsg(hctl, LB_GETCOUNT, 0, 0L);
      if(itemcount != (WORD)LB_ERR) {
         for(x=0; x<itemcount; x++) DeleteItem(hctl, x);
         }
      GlobalFree(phc->PrivateHeap);
      phc->PrivateHeap = NULL;
      }
   if(!stoprecreate)
      phc->PrivateHeap = GlobalAlloc(GHND, (DWORD)INITHEAPSIZE);
   if(phc->PrivateHeap) {
      LPSTR lptr;
      WORD seg;
      lptr = (LPSTR)GlobalLock(phc->PrivateHeap);
      seg = HIWORD(lptr);
      LocalInit(seg, 0, INITHEAPSIZE-1);
      GlobalUnlock(phc->PrivateHeap);
      }
}



VOID DrawItem(HCTL hctl, LPDRAWITEMSTRUCT lpdi)
{
   CHAR     buffer[255];
   LPSTR    lpBuf;
   HBRUSH   hBrush = NULL;
   USHORT   usLen;
   PIC      pic;
   BITMAP   bmp;
   HDC      hdcMem;
   LPITEMATTR lpattr;
   SHORT    offset;
   short dchold;
   DWORD usecolor;
   PMLIST phc;


   if((short)lpdi->itemID<0) {   // It's an empty list box
      if (lpdi->itemState & ODS_FOCUS) {
         // Just draw the focus rectangle if necessary
         DrawFocusRect(lpdi->hDC, &lpdi->rcItem);
         }
      return;
      }

   dchold = SaveDC(lpdi->hDC);
   lpattr = LockItem(hctl, lpdi->itemID);

   // If we've defined a background, clear the
   // background and set the background color
   if(lpattr->flags & 0x01) {
      usecolor = lpattr->cBack;
      }
   else {
      VBGetControlProperty(hctl, IPROP_MLIST_BACKCOLOR, &usecolor);
      }
   hBrush = CreateSolidBrush(usecolor);
   FillRect(lpdi->hDC, (LPRECT)&lpdi->rcItem, hBrush);
   DeleteObject(hBrush);
   SetBkColor(lpdi->hDC, usecolor);

   offset = 0;
   if (lpattr->hPic) {  // Place the bitmap
      VBGetPic(lpattr->hPic, &pic);
      GetObject(pic.picData.bmp.hbitmap, sizeof(BITMAP), (LPSTR)&bmp);
      hdcMem = CreateCompatibleDC(lpdi->hDC);
      SelectObject(hdcMem, pic.picData.bmp.hbitmap);

      BitBlt(lpdi->hDC, lpdi->rcItem.left, lpdi->rcItem.top,
               lpdi->rcItem.right - lpdi->rcItem.left + 1,
               lpdi->rcItem.bottom - lpdi->rcItem.top + 1,
               hdcMem, 0, 0, SRCCOPY);

      DeleteDC(hdcMem);
		offset = (SHORT)(bmp.bmWidth * 1.25);
      }

   lpBuf = (LPSTR)buffer;
   SendMessage(lpdi->hwndItem, LB_GETTEXT, lpdi->itemID, (DWORD)lpBuf);
   usLen = (USHORT)lstrlen(lpBuf);

   if (usLen) {   // Text was found
      RECT userect;
      LPSTR tabptr;
      short extent, tabnum, tabloc;
      TEXTMETRIC tm;
      HFONT horigfnt, hnewfont;
      LOGFONT lf;

      if(lpattr->flags & 0x02) 
         SetTextColor(lpdi->hDC, lpattr->cFore);

      if(lpattr->flags & 0x04) { // Create a strikethru font to use
         // We can use this mechanism to do other font operations - but if
         // we wanted to allow fonts with different heights, we would also
         // need to modify the height calculation routines.
         horigfnt = SelectObject(lpdi->hDC, GetStockObject(SYSTEM_FONT));
         GetObject(horigfnt, sizeof(LOGFONT), &lf);
         lf.lfStrikeOut = TRUE;
         hnewfont = CreateFontIndirect(&lf);
         if(hnewfont) SelectObject(lpdi->hDC, hnewfont);
         }

      userect = lpdi->rcItem;
      userect.left += offset;

      tabptr = _fstrchr(lpBuf, '\t');
      if(tabptr) {
         tabnum = 0;
         GetTextMetrics(lpdi->hDC, &tm);
         phc = VBDerefControl(hctl);
         while(tabptr) {
            *tabptr = '\0';   // Place null at tab location
            usLen = lstrlen(lpBuf);
            if(usLen) {
               extent = LOWORD(GetTextExtent(lpdi->hDC, lpBuf, usLen));
               DrawText(lpdi->hDC, lpBuf, usLen, &userect, 
      		            DT_LEFT | DT_VCENTER | DT_SINGLELINE);
               }
            else extent = 0;
            lpBuf = tabptr+1;
            // Find where the next character should go
            userect.left += extent;

            // Compare with the tab location
            tabloc = lpdi->rcItem.left + 
                     phc->tablocs[tabnum] * tm.tmAveCharWidth;
            // Use tab location if greater
            if (tabloc > userect.left) userect.left = tabloc;
            if(userect.left > userect.right) break;
            tabptr = _fstrchr(lpBuf, '\t');
            tabnum++;
            }
         // We've used up the tabs - just display the rest of the string
         usLen = lstrlen(lpBuf);
         }
      if(userect.left < userect.right && usLen) {
         DrawText(lpdi->hDC, lpBuf, usLen, &userect, 
   		            DT_LEFT | DT_VCENTER | DT_SINGLELINE);
         }

      if(lpattr->flags & 0x04) { // Restore the original font
         SelectObject(lpdi->hDC, horigfnt);
         if(hnewfont) DeleteObject(hnewfont);
         }


      }
   UnlockItem(hctl, lpdi->itemID);


   if (lpdi->itemState & ODS_SELECTED) {
      InvertRect(lpdi->hDC, &lpdi->rcItem);
      }
   if (lpdi->itemState & ODS_FOCUS) {
      DrawFocusRect(lpdi->hDC, &lpdi->rcItem);
      }
   RestoreDC(lpdi->hDC, dchold);
}


//---------------------------------------------------------------------------
// This routine retrieves the height of the items in the listbox.
//---------------------------------------------------------------------------
VOID MeasureItem(HCTL hctl, LPMEASUREITEMSTRUCT lpMI)
{
   PMLIST phc;

   phc = VBDerefControl(hctl);

   // Is it multicolumn?
   if(phc->multicolumn > 0) {
      lpMI->itemHeight = phc->fontheight;
      // Just set an arbitrary height - it will be changed later
      // when the fonts are set.
      if(!lpMI->itemHeight) lpMI->itemHeight = 8;
      return;
      }

   lpMI->itemHeight = GetHeight(hctl, lpMI->itemID);
}


WORD GetHeight(HCTL hctl, WORD itemnum)
{
   PMLIST phc;
   LPITEMATTR lpattr;
   PIC pic;
   WORD useheight;

   phc = VBDerefControl(hctl);
   useheight = phc->fontheight;

   // Is there a bitmap?
   lpattr = LockItem(hctl, itemnum);

   if(lpattr->hPic) {
      BITMAP bmp;
		VBGetPic(lpattr->hPic, &pic);
		GetObject(pic.picData.bmp.hbitmap, sizeof(BITMAP), (LPSTR)&bmp);
      if(bmp.bmHeight > (int)(useheight - 2))
         useheight = bmp.bmHeight+2;
      }
   UnlockItem(hctl, itemnum);
   return(useheight);
}



/*

 Function Name:   VOID UpdateAllHeights(HCTL hctl)

 Parameters:      hctl = Control handle

 Description:     Updates heights of all entries

 Notes:        
*/

VOID UpdateAllHeights(HCTL hctl)
{
   WORD itemcount, x;
   PMLIST phc;

   phc = VBDerefControl(hctl);
   if(phc->multicolumn >0) {   // Use largest height
      WORD largest = 0;
      WORD thisheight;

      itemcount = (WORD)VBSendControlMsg(hctl, LB_GETCOUNT, 0, 0L);
      if(itemcount == (WORD)LB_ERR || itemcount== 0) {
         // Just set the fixed size to the height of the font.
         largest = phc->fontheight;
         itemcount = 0;
         }
      for(x=0; x<itemcount; x++){
         thisheight = GetHeight(hctl, x);
         if(thisheight > largest) largest = thisheight;
         }
      VBSendControlMsg(hctl, LB_SETITEMHEIGHT, 0, MAKELONG(largest,0));
      }
   else {
      itemcount = (WORD)VBSendControlMsg(hctl, LB_GETCOUNT, 0, 0L);
      if(itemcount == (WORD)LB_ERR) return;
      
      // Set each one differently
      for(x=0; x<itemcount; x++){
         VBSendControlMsg(hctl, LB_SETITEMHEIGHT, x, 
                          MAKELONG(GetHeight(hctl, x),0));
         }
      }
}
