        page    ,132
        title   MMSYS.ASM

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; (C) Copyright Microsoft Corp. 1993.  All rights reserved.
;
; You have a royalty-free right to use, modify, reproduce and 
; distribute the Sample Files (and/or any modified version) in 
; any way you find useful, provided that you agree that 
; Microsoft has no warranty obligations or liability for any 
; Sample Application Files which are modified. 
;
; Unless you got this from the MM Sys BBS, it may not be the most
; current version.  We are continually improving our samples and
; their documentation.  Call the BBS at 206 936-4082.
;
; The official version of this sample code is included in the detect
; sample application on the BBS.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;*******************************************************************************
;*									       *
;*  MODULE      : MMSYS.ASM                                                    *
;*									       *
;*  DESCRIPTION : Provide functions to make an application aware of Windows    *
;*                with Multimedia.  Application will work even if Multimedia   *
;*                is not currently installed.                                  *
;*									       *
;*  COPYRIGHT   : Copyright 1991, 1992, Microsoft Corp.  All Rights Reserved.	       *
;*									       *
;*	Authors: Todd Laney and Matt Saettler - Multimedia Systems	       *
;*									       *
;*******************************************************************************

;------------------------------------------------------------------------------
;
; Documentation (such as it is)
;
;------------------------------------------------------------------------------
;
; Call the Windows with Multimedia functions as you normally would.  Include
; MMSYSTEM.H (or .INC) normally.  However, instead of linking to MMSYSTEM.LIB,
; link to MMSYS.OBJ
;
; All functions will return error conditions if the extensions are not found.
; Alternative 'Windows with Multimedia not installed' functions could 
; be provided.  (It would replace the 'fail' proc; see the macros below.)
;
; mmsystemGetVersion() will return 0 if Windows with Multimedia is not 
; installed.
;
; Note: MMP support is not provided by this sample, but could be easily 
;       derived using the macros given.
;
;------------------------------------------------------------------------------
;
;    If defined, the following flags inhibit inclusion of the indicated items:
; 
;        MMNOSOUND        - Sound support
;        MMNOWAVE         - Waveform support
;        MMNOMIDI         - MIDI support
;        MMNOAUX          - Auxiliary output support
;        MMNOTIMER        - Timer support
;        MMNOJOY          - Joystick support
;        MMNOMCI          - MCI support
;        MMNODRV          - Installable driver support
;        MMNOMMIO         - MMIO library support
;        MMNOMMSYSTEM     - No mmsystem general functions
;
;------------------------------------------------------------------------------

;MMNOSOUND       = 1   ; No Sound support
;MMNOWAVE        = 1   ; No Waveform support
;MMNOMIDI        = 1   ; No MIDI support
;MMNOAUX         = 1   ; No Auxiliary output support
;MMNOTIMER       = 1   ; No Timer support
;MMNOJOY         = 1   ; No Joystick support
;MMNOMCI         = 1   ; No MCI support
;MMNODRV         = 1   ; No Installable driver support
;MMNOMMIO        = 1   ; No MMIO library support
;MMNOMMSYSTEM    = 1   ; No No mmsystem general functions

page

        .286
	?PLM=1	    ; PASCAL Calling convention is DEFAULT
        ?WIN=0      ; Windows calling convention

        .xlist
	include cmacros.inc
	.list

;*********************************************************************
;               CONSTANT DECLARATIONS
;*********************************************************************

ifndef MMSYSERR_ERROR
MMSYSERR_ERROR 	equ	1
endif

;*********************************************************************
;               EXTERN DECLARATIONS
;*********************************************************************

	externFP   OutputDebugString
	externFP   _wsprintf
	externFP   GetProcAddress
        externFP   GetModuleHandle
	externFP   mmioAscendStub
	externFP   mmioDescendStub
	externFP   mmioCreateChunkStub

ifdef DEBUG
        externFP    __dprintf               ; in DPRINTF.C
endif

;*********************************************************************
;               STRUCTURE DECLARATIONS
;*********************************************************************

LONG    struc
	lo      dw      ?
	hi      dw      ?
LONG    ends

FARPOINTER      struc
	off     dw      ?
	sel     dw      ?
FARPOINTER      ends

PROCENTRY	struc
	curproc		dd	?	; see parameters to macros, below
        ordinal         dw      ?
	numparms	dw	?
        errret          dd      ?

ifdef DEBUG
	szProc		db	?
endif
PROCENTRY	ends

MODENTRY	struc
	hModule		dw	?
	szModule	db	?
MODENTRY	ends

;*********************************************************************
;               DATA SEGMENT DECLARATIONS
;*********************************************************************

ifndef SEGNAME
        SEGNAME equ <_TEXT>
endif

createSeg %SEGNAME, CodeSeg, word, public, CODE

page

;*********************************************************************
;                  MACRO DECLARATIONS
;*********************************************************************

; 
;------------------------------------------------------------------------------
;
; MACRO DOUT
;
; Parms:
;
;  text		Text to output using OutputDebugString when DEBUG is defined
;		Text is automatically appended with CR/LF
;

DOUT macro text
        local   string_buffer

ifdef DEBUG		; only do output if DEBUG is defined

_DATA segment
string_buffer label byte
        db      "&text&",13,10,0
_DATA ends
        pusha
        push    DataBASE
        push    DataOFFSET string_buffer
        call    OutputDebugString
        popa
endif
        endm

; 
;------------------------------------------------------------------------------
;
; MACRO Begin_Module_Table
;
; Parms:
;
; Module_Name 	Name of Module to Run-Time-Link
;
; defines <Module_Name>_Proc Macro
;
; Use End_Module_Table to close

Begin_Module_Table MACRO Module_Name

sBegin DATA

ifdef DEBUG
	public Module_Name&_Module_Table
endif

Module_Name&_Module_Table label word
	dw	-1		; hModule
	db	"&Module_Name&",0

sEnd Data

sBegin CodeSeg
        assumes cs,CodeSeg
        assumes ds,Data
        assumes es,nothing

ifdef DEBUG			; make public so debugger is aware of it
        public load&Module_Name
endif

;
; entry:
;	DS:BX	--> ProcEntry for API being called
;
load&Module_Name& proc far
	; stack frame is not modified or copied
	; vars are still in place

	mov	ax,DataOFFSET Module_Name&_Module_Table
        jmp     LoadModuleStub

load&Module_Name& endp

sEnd CodeSeg

page

; 
;------------------------------------------------------------------------------
;
; MACRO <Module_Name>_Proc
;
; Parms:
;
; Name of procedure	Name of procedure to emulate
; ordinal of exported proc
; # of stack parms	use 0 for CDECL routines
; error return value	default error value for use by FailAPIStub
; fail proc		defaults to FailAPIStub if not specified
;			use custom 'fail' proc to replace functionlity
;			if specified module/proc not found in system
;
Module_Name&_Proc macro ProcName, Ordinal, sizestack, errret, failproc

sBegin Data

ifdef DEBUG
        public Module_Name&&Ordinal&
endif

Module_Name&&Ordinal& label word
ifb     <failproc>
        dd      load&Module_Name
        dw      &Ordinal
        dw      &sizestack
        dd      &errret
else
        dd      load&Module_Name
        dw      &Ordinal
        dw      -1
        dd      &failproc
endif

ifdef DEBUG
	db	"&ProcName&",0
endif

sEnd    Data

sBegin CodeSeg
        assumes cs,CodeSeg
        assumes ds,Data
        assumes es,nothing

public &ProcName&

&ProcName& proc far
	; stack frame is not modified or copied
	; vars are still in place

        mov     bx,DataOFFSET Module_Name&&Ordinal&
        jmp     [bx].curproc                            ; current proc

&ProcName& endp
	
sEnd    CodeSeg

        endm
        endm

page
;------------------------------------------------------------------------------
;
; MACRO End_Module_Table
;
; Parms
; Module_Name	Must be the same as in Begin_Module_Table
;

End_Module_Table macro Module_Name

        purge   Module_Name&_Proc

        endm


;-----------------------------------------------------------------------------
;
;  Helper routines for MMSYS
;
;-----------------------------------------------------------------------------
sBegin CodeSeg
        assumes cs,CodeSeg
	assumes ds,Data
	assumes es,nothing

;-----------------------------------------------------------------------------
;
; FailApiStub
;
;  Default handler if Module or Proc Address is not found.
;
;  returns default error code
;
;  entry:
;	DS:BX	--> PROCENTRY
;

FailApiStub proc far

	pop	dx		        ; get return addr
	pop	ax

	add	sp,[bx].numparms	; remove params from stack

	push	ax			; restore return addr
	push	dx
	mov	ax,[bx].errret.lo	; return fail code
	mov	dx,[bx].errret.hi
        retf

FailApiStub endp

;-----------------------------------------------------------------------------
;
; LoadModuleStub
;
;  Initial handler for all procs.  Attempts to load module (if not already
;  loaded) and then gets proc address.  If any errors, sets curproc to
;  failproc for 'unavailable' processing.
;
;  If successful, then sets curproc to imported function and calls it.
;
; entry:
;	DS:BX --> PROCENTRY
;	DS:AX --> MODENTRY
;
; NOTE:  Assumes module is already loaded (which is true for MMSystem).
;
;     To be totally general:
;       if can't GetModuleHandle(),
;	needs to do a OpenFile(OF_EXIST,...) + LoadLibrary()
;	needs to FreeLibrary() all DLLs at end/exit
;
LoadModuleStub proc far

ifdef DEBUG
	pusha
	sub	sp,128
	mov	si,sp

	mov	di,ax			; DS:DI --> MODENTRY
	
        push    [bx].ordinal            ; %d

	lea	ax,[bx].szProc		; %ls
	push	ds
	push	ax

	lea	ax,[di].szModule	; %ls
	push	ds
        push    ax

        lea     ax,format_string        ; format string
	push	cs
        push    ax

        push    ss                      ; buffer
	push	si
	call	_wsprintf
        add     sp,9*2                  ; clear 9 words

        cCall   OutputDebugString,<ss,si>

	add	sp,128
	popa
	jmp	@f
format_string:
        db      "Linking %ls!%ls@%d",13,10,0
@@:
endif
	pusha
	
	mov	si,ax			; ds:[si] --> MODENTRY
	mov	di,bx			; ds:[di] --> PROCENTRY

	mov	ax,[si].hModule		
	or	ax,ax
	jz	LoadModuleStubFail	; module does not exist

	cmp	ax,-1
	jne	LoadModuleStubGetProc	

	lea	ax,[si].szModule
	cCall	GetModuleHandle, <ds,ax>
	mov	[si].hModule,ax
	or	ax,ax
	jz	LoadModuleStubLoad

LoadModuleStubGetProc:
        cCall   GetProcAddress,<ax,0,[di].ordinal>
	or	dx,dx
	jz	LoadModuleStubFail

LoadModuleStubDone:
	mov	[di].curproc.lo,ax
	mov	[di].curproc.hi,dx

	popa
	jmp	[bx].curproc

LoadModuleStubLoad:
	
	;; call load library here after verifying with OpenFile()
	;

	; for now, fall through to error

LoadModuleStubFail:
        DOUT    <*** API not found! ***>

        mov     ax,CodeSegOFFSET FailApiStub
        mov     dx,cs

        cmp     [di].numparms,-1            ; do we have a fail proc?
        jne     LoadModuleStubDone          ; no...use FailApiStub

        mov     ax,[di].errret.lo           ; yes..it is stored in errret
        mov     dx,[di].errret.hi
        jmp     short LoadModuleStubDone    ; use it
	
LoadModuleStub endp

sEnd CodeSeg

page
;*********************************************************************
;                    CODE and DATA
;*********************************************************************


;
; Define MMSYSTEM Run-Time-Load Table

Begin_Module_Table MMSYSTEM


;
; Define all the procs in MMSYSTEM

ifndef MMNOSOUND

MMSYSTEM_Proc sndPlaySound,             2,      6, 0

endif

ifndef MMNOMMSYSTEM

MMSYSTEM_Proc mmsystemGetVersion,       5,     00, 0

endif

ifndef MMNOJOY

MMSYSTEM_Proc joyGetNumDevs,            101,   00, 0
MMSYSTEM_Proc joyGetDevCaps,            102,    6, MMSYSERR_ERROR
MMSYSTEM_Proc joyGetPos,                103,    6, MMSYSERR_ERROR
MMSYSTEM_Proc joyGetThreshold,          104,    6, MMSYSERR_ERROR
MMSYSTEM_Proc joyReleaseCapture,        105,    2, MMSYSERR_ERROR
MMSYSTEM_Proc joySetCapture,            106,    6, MMSYSERR_ERROR
MMSYSTEM_Proc joySetThreshold,          107,    4, MMSYSERR_ERROR

endif

ifndef MMNOMIDI

MMSYSTEM_Proc midiOutGetNumDevs,        201,   00, 0
MMSYSTEM_Proc midiOutGetDevCaps,        202,    8, MMSYSERR_ERROR
MMSYSTEM_Proc midiOutGetErrorText,      203,    8, MMSYSERR_ERROR
MMSYSTEM_Proc midiOutOpen,              204,   18, MMSYSERR_ERROR
MMSYSTEM_Proc midiOutClose,             205,    2, MMSYSERR_ERROR
MMSYSTEM_Proc midiOutPrepareHeader,     206,    8, MMSYSERR_ERROR
MMSYSTEM_Proc midiOutUnprepareHeader,   207,    8, MMSYSERR_ERROR
MMSYSTEM_Proc midiOutShortMsg,          208,    6, MMSYSERR_ERROR
MMSYSTEM_Proc midiOutLongMsg,           209,    8, MMSYSERR_ERROR
MMSYSTEM_Proc midiOutReset,             210,    2, MMSYSERR_ERROR
MMSYSTEM_Proc midiOutGetVolume,         211,    6, MMSYSERR_ERROR
MMSYSTEM_Proc midiOutSetVolume,         212,    6, MMSYSERR_ERROR
MMSYSTEM_Proc midiOutCachePatches,      213,   10, MMSYSERR_ERROR
MMSYSTEM_Proc midiOutGetID,             214,    6, MMSYSERR_ERROR

MMSYSTEM_Proc midiInGetNumDevs,         301,   00, 0
MMSYSTEM_Proc midiInGetDevCaps,         302,    8, MMSYSERR_ERROR
MMSYSTEM_Proc midiInGetErrorText,       303,    8, MMSYSERR_ERROR
MMSYSTEM_Proc midiInOpen,               304,   18, MMSYSERR_ERROR
MMSYSTEM_Proc midiInClose,              305,    2, MMSYSERR_ERROR
MMSYSTEM_Proc midiInPrepareHeader,      306,    8, MMSYSERR_ERROR
MMSYSTEM_Proc midiInUnprepareHeader,    307,    8, MMSYSERR_ERROR
MMSYSTEM_Proc midiInAddbuffer,          308,    8, MMSYSERR_ERROR
MMSYSTEM_Proc midiInStart,              309,    2, MMSYSERR_ERROR
MMSYSTEM_Proc midiInStop,               310,    2, MMSYSERR_ERROR
MMSYSTEM_Proc midiInReset,              311,    2, MMSYSERR_ERROR
MMSYSTEM_Proc midiInGetID,              312,    6, MMSYSERR_ERROR

endif

ifndef MMNOAUX

MMSYSTEM_Proc auxGetNumDevs,            350,   00, 0
MMSYSTEM_Proc auxGetDevCaps,            351,    8, MMSYSERR_ERROR
MMSYSTEM_Proc auxGetVolume,             352,    6, MMSYSERR_ERROR
MMSYSTEM_Proc auxSetVolume,             353,    6, MMSYSERR_ERROR

endif

ifndef MMNOWAVE

MMSYSTEM_Proc waveOutGetNumDevs,        401,   00, 0
MMSYSTEM_Proc waveOutGetDevCaps,        402,    8, MMSYSERR_ERROR
MMSYSTEM_Proc waveOutGetErrorText,      403,    8, MMSYSERR_ERROR
MMSYSTEM_Proc waveOutOpen,              404,   22, MMSYSERR_ERROR
MMSYSTEM_Proc waveOutClose,             405,    2, MMSYSERR_ERROR
MMSYSTEM_Proc waveOutPrepareHeader,     406,    8, MMSYSERR_ERROR
MMSYSTEM_Proc waveOutUnprepareHeader,   407,    8, MMSYSERR_ERROR
MMSYSTEM_Proc waveOutWrite,             408,    8, MMSYSERR_ERROR
MMSYSTEM_Proc waveOutPause,             409,    2, MMSYSERR_ERROR
MMSYSTEM_Proc waveOutRestart,           410,    2, MMSYSERR_ERROR
MMSYSTEM_Proc waveOutReset,             411,    2, MMSYSERR_ERROR
MMSYSTEM_Proc waveOutGetPosition,       412,    8, MMSYSERR_ERROR
MMSYSTEM_Proc waveOutGetPitch,          413,    6, MMSYSERR_ERROR
MMSYSTEM_Proc waveOutSetPitch,          414,    6, MMSYSERR_ERROR
MMSYSTEM_Proc waveOutGetVolume,         415,    6, MMSYSERR_ERROR
MMSYSTEM_Proc waveOutSetVolume,         416,    6, MMSYSERR_ERROR
MMSYSTEM_Proc waveOutGetPlaybackRate,   417,    6, MMSYSERR_ERROR
MMSYSTEM_Proc waveOutSetPlaybackRate,   418,    6, MMSYSERR_ERROR
MMSYSTEM_Proc waveOutBreakLoop,         419,    2, MMSYSERR_ERROR
MMSYSTEM_Proc waveOutGetID,             420,    6, MMSYSERR_ERROR

if 0
MMSYSTEM_Proc waveInGetNumDevs,         501,   00, 0
MMSYSTEM_Proc waveInGetDevCaps,         502,    8, MMSYSERR_ERROR
MMSYSTEM_Proc waveInGetErrorText,       503,    8, MMSYSERR_ERROR
MMSYSTEM_Proc waveInOpen,               504,   22, MMSYSERR_ERROR
MMSYSTEM_Proc waveInClose,              505,    2, MMSYSERR_ERROR
MMSYSTEM_Proc waveInPrepareHeader,      506,    8, MMSYSERR_ERROR
MMSYSTEM_Proc waveInUnprepareHeader,    507,    8, MMSYSERR_ERROR
MMSYSTEM_Proc waveInAddBuffer,          508,    8, MMSYSERR_ERROR
MMSYSTEM_Proc waveInStart,              509,    2, MMSYSERR_ERROR
MMSYSTEM_Proc waveInStop,               510,    2, MMSYSERR_ERROR
MMSYSTEM_Proc waveInReset,              511,    0, MMSYSERR_ERROR
MMSYSTEM_Proc waveInGetPosition,        512,    8, MMSYSERR_ERROR
MMSYSTEM_Proc waveInGetID,              513,    6, MMSYSERR_ERROR
endif

endif

ifndef MMNOTIMER

MMSYSTEM_Proc timeGetTime,              607,   00, 0, timeGetTimeStub

if 0;
MMSYSTEM_Proc timeGetSystemTime,        601,    6, MMSYSERR_ERROR
MMSYSTEM_Proc timeSetEvent,             602,   14, 0
MMSYSTEM_Proc timeKillEvent,            603,    2, MMSYSERR_ERROR
MMSYSTEM_Proc timeGetDevCaps,           604,    6, MMSYSERR_ERROR
MMSYSTEM_Proc timeBeginPeriod,          605,    2, MMSYSERR_ERROR
MMSYSTEM_Proc timeEndPeriod,            606,    2, MMSYSERR_ERROR
endif

endif

ifndef MMNOMCI

MMSYSTEM_Proc mciSendCommand,           701,   12, MMSYSERR_ERROR
MMSYSTEM_Proc mciSendString,            702,   12, MMSYSERR_ERROR
MMSYSTEM_Proc mciGetDeviceID,           703,    4, 0
MMSYSTEM_Proc mciGetErrorString,        706,   10, MMSYSERR_ERROR
MMSYSTEM_Proc mciExecute,               712,    4, MMSYSERR_ERROR

endif

ifndef MMNOMMIO

MMSYSTEM_Proc mmioOpen,                 1210,  12, 0              ,mmioOpenStub
MMSYSTEM_Proc mmioClose,                1211,   4, MMSYSERR_ERROR ,mmioCloseStub
MMSYSTEM_Proc mmioRead,                 1212,  10, -1             ,mmioReadStub
MMSYSTEM_Proc mmioSeek,                 1214,   8, -1             ,mmioSeekStub
MMSYSTEM_Proc mmioWrite,                1213,  10, -1             ,mmioWriteStub
MMSYSTEM_Proc mmioGetInfo,              1215,   8, MMSYSERR_ERROR
MMSYSTEM_Proc mmioSetInfo,              1216,   8, MMSYSERR_ERROR
MMSYSTEM_Proc mmioSetBuffer,            1217,  12, 0
MMSYSTEM_Proc mmioFlush,                1218,   4, 0
MMSYSTEM_Proc mmioAdvance,              1219,   8, MMSYSERR_ERROR
MMSYSTEM_Proc mmioStringToFOURCC,       1220,   6, MMSYSERR_ERROR
MMSYSTEM_Proc mmioInstallIOProc,        1221,  12, 0
MMSYSTEM_Proc mmioSendMessage,          1222,  12, MMSYSERR_ERROR
MMSYSTEM_Proc mmioDescend,              1223,  12, 0, mmioDescendStub
MMSYSTEM_Proc mmioAscend,               1224,   8, 0, mmioAscendStub
MMSYSTEM_Proc mmioCreateChunk,          1225,   8, 0, mmioCreateChunkStub

endif

;
; end the MMSYSTEM R-T-L table

End_Module_Table MMSYSTEM

;*********************************************************************
;                    STUB ROUTINES
;*********************************************************************

include windows.inc

externFP    GetCurrentTime
externFP    OpenFile
externFP    _lclose
externFP    _llseek
externA     __AHINCR

sBegin CodeSeg
        assumes cs,CodeSeg
        assumes ds,Data
        assumes es,nothing

;----------------------------------------------------------------------------;
; timeGetTime() - stub routine for timeGetTime()
;
; call windows to get the time.
;----------------------------------------------------------------------------;
        assumes ds,Data
        assumes es,nothing

cProc   timeGetTimeStub, <FAR, PASCAL>, <>
cBegin
        cCall   GetCurrentTime
cEnd

;-----------------------------------------------------------------------;
; mmioOpen()
;-----------------------------------------------------------------------;
        assumes ds,Data
        assumes es,nothing

cProc   mmioOpenStub, <FAR, PASCAL, PUBLIC>, <>
        parmD   szFile
        parmD   pmmioinfo
        parmD   acc

        localV  of,%(size OPENSTRUC)
cBegin
        lea     bx,of
        mov     ax,word ptr acc

        cCall   OpenFile,<szFile, ss,bx, ax>
cEnd

;-----------------------------------------------------------------------;
; mmioClose - Close a DOS file
;-----------------------------------------------------------------------;
        assumes ds,Data
        assumes es,nothing

cProc   mmioCloseStub, <FAR, PASCAL, PUBLIC>, <>
        parmW   fh
        parmW   flags
cBegin
        cCall   _lclose,<fh>
cEnd

;-----------------------------------------------------------------------;
; mmioSeek - seek a DOS file
;-----------------------------------------------------------------------;
        assumes ds,Data
        assumes es,nothing

cProc   mmioSeekStub,<FAR,PASCAL,PUBLIC>,<>
        parmW   fh
        parmD   offSeek
        parmB   Origin
cBegin
        cCall   _llseek,<fh,offSeek,Origin>
cEnd

;-----------------------------------------------------------------------;
; mmioReadStub - read data from a DOS file
;-----------------------------------------------------------------------;
        assumes ds,Data
        assumes es,nothing

cProc   mmioReadStub,<FAR,PASCAL,PUBLIC>,<>
;       parmW   fh
;       parmD   pBuf
;	parmD	Count
cBegin	nogen

	mov	cl,3fh			; read code
        jmp     mmioReadWriteStub

cEnd	nogen

;-----------------------------------------------------------------------;
; mmioWriteStub - write data to a DOS file
;-----------------------------------------------------------------------;
        assumes ds,Data
        assumes es,nothing

cProc   mmioWriteStub,<FAR,PASCAL,PUBLIC>,<>
;       parmW   fh
;       parmD   pBuf
;	parmD	Count
cBegin	nogen

	mov	cl,40h			; write code
        errn$   mmioReadWriteStub

cEnd	nogen

;-----------------------------------------------------------------------;
; mmioReadWriteStub - read/write data to a DOS file
;-----------------------------------------------------------------------;
        assumes ds,Data
        assumes es,nothing

cProc   mmioReadWriteStub,<FAR,PASCAL,PUBLIC>,<si,di>
        parmW   fh
        parmD   pBuf
	parmD	Count

	localB	rwcode
cBegin	
        mov     rwcode,cl

        mov     bx,fh

        mov     di,Count.hi
        mov     cx,Count.lo             ; DI:CX has the count of bytes to read/write
        lds     si,pBuf                 ; DS:SI points to buffer
        assumes ds,nothing

        xor     ax,ax                   ; accumulate amount actualy read/writen
        mov     Count.lo,ax             ; here
        mov     Count.hi,ax

; MS DOS will only read/write 65,535 bytes at a time.

; get in DX:AX the number of bytes left in segment

	xor	ax,ax
	mov	dx,1			; DX:AX = 64k
	sub	ax,si
	sbb	dx,0			

; DX:AX has number of bytes left in the segment and
; DI:CX has the number of bytes to read/write
; DS:SI points to next position in buffer

iterate:

; test to see if the read/write can be satisfied in the current segment

	cmp	dx,di
	ja	all_in_same_segment	; yes
	jb	will_cross_segment
	cmp	ax,cx
	ja	all_in_same_segment
	jb	will_cross_segment

; in this case the number of bytes to read/write is exactly equal to the no left
; in the segment. We can only read 65535 bytes at a time, so in the worst
; case of read/write 64k we will have to read/write twice.

	cmp	di,0			; > 65535 to read/write
	jne	two_reads		; two read/write necessary

; one read/write will suffice

	mov	dx,si			; DS:DX has buffer
	mov	ah,rwcode		; read/write code
        int     21h
        jc      read_error_v
        jmp     read_ret                ; go back

; we have to read/write excatly 64k bytes, read/write 65535 bytes first and then 1 byte
; in this case buffer must be starting at segment:0

two_reads:
	mov	dx,si			; DS:DX points to buffer start
	mov	cx,0ffffh		; read/write 65535
	mov	ah,rwcode
	int	21h			; read/write
read_error_v:
        jc      read_error
        add     Count.lo,ax
        adc     Count.hi,0
	mov	dx,0ffffh		; point to last byte in segment
	mov	cx,1			; read/write the last byte
	mov	ah,rwcode
	int	21h			; finish read/writeing
        jc      read_error
	jmp	read_ret		; go back

all_in_same_segment:

; this is the easiest case when one read/write will suffice, will not cross segs

	mov	dx,si	
	mov	ah,rwcode
	int	21h
        jc      read_error
	jmp	read_ret

will_cross_segment:

; first read/write the rest of the segment

	push	dx
	push	ax
	push	di
	push	cx			; save the counts
	cmp	dx,1			; 64k in this segment ?
	jnz	seg_one			; one read/write

; full 64 k in this segment, so do two read/writes

	mov	dx,si			; DS:DX points to buffer
	mov	cx,0ffffh		; read/write 65535 bytes
	mov	ah,rwcode
	int	21h	
        jc      read_error
        add     Count.lo,ax
        adc     Count.hi,0
	mov	dx,0ffffh		; point to last byte
	mov	cx,1			; read/write 1 byte
	mov	ah,rwcode
	int	21h
        jc      read_error
	jmp	short reduce_count

; one read to the end of the segment

seg_one:
	mov	cx,ax			; bytes left in segment
	mov	dx,si			; DS:SI points to buffer
	mov	ah,rwcode
	int	21h
        jc      read_error
reduce_count:
        add     Count.lo,ax
        adc     Count.hi,0
	mov	dx,ds
        add     dx,__AHINCR             ; go to next segment
	mov	ds,dx
	xor	si,si			; DS:SI points to next segment

; get back counts and reduce no of bytes to read/write

	pop	cx
	pop	di
	pop	ax
	pop	dx

	sub	cx,ax
	sbb	di,dx			
	mov	dx,1
	xor	ax,ax			

; DI:CX has bytes left to read/write
; DX:AX has bytes left in the segment
; DS:SI has the pointer to the next byte to read/write

	jmp	iterate			; iterate till all bytes are read/write

read_error:                             ; return -1:-1 to indicate error
        mov     ax,-1
        mov     dx,ax
        jmp     short read_exit

read_ret:
        mov     cx,ax
        mov     ax,Count.lo             ; return original count, to indicate
        mov     dx,Count.hi             ;   success.

        add     ax,cx                   ; add in result of last read/write
        adc     dx,0

read_exit:
cEnd

sEnd

end
