//***************************************************************************
//
// FILE NAME: MULTIDLG.C
//
//    MULTIDLG.C includes all the hocus-pocus necessary to create and manage
//    the child dialogs.  It includes the following functions:
//
//
// FUNCTIONS:
//
//    WinMain()
//          Just a normal WinMain().
//
//    InitApplication()
//          Called by WinMain.  Registers main window class.
//
//    InitInstance()
//          Called by WinMain.  Creates the main window.
//
//    MainWndProc()
//          The callback function for the main window.  Initiates the
//          multidlg.
//    
//    LoadAndCreateControls()
//          Creates a bunch of controls based on a dialog template.
//
//    ChildDialogVisible ()
//          Hides or shows a given child dialog.
//
//
// AUTHOR: John Dempsey
//
// Written by Microsoft Product Support Services, Windows Developer Support.
// Copyright (c) 1993 Microsoft Corporation.  All rights reserved.
//
//***************************************************************************

#include "windows.h"
#include "multidlg.h"
#include "dlgprocs.h"
#include "resource.h"

// The diDlgs[] array identifies each child dialog's associated listbox
// text, pseudo-callback function, resource ID, control hwnds (through a
// pointer to local memory), and control texts (stored as atoms in an array
// which lives in local memory.  This is necessary to work around a bug in
// IsDialogMessage()), and status (loaded or not loaded).


DLGINFO diDlgs[] = {
    "Identification", IdentityDlgProc, IDD_IDENDLG,   NULL, NULL, FALSE,
    "Preferences",    PrefsDlgProc,    IDD_PREFSDLG,  NULL, NULL, FALSE,
    "Wishes",         WishesDlgProc,   IDD_WISHESDLG, NULL, NULL, FALSE,
    NULL,             NULL,            0,             NULL, NULL, FALSE };

HINSTANCE ghInst;
int       iCurDlg = -1 ;
    

//***************************************************************************
//
// FUNCTION: WinMain()
//
// HISTORY: Date    Author  Reason
//          6/1/93  JRD     Created
//
//***************************************************************************

int PASCAL WinMain(HINSTANCE hInstance,
                   HINSTANCE hPrevInstance,
                   LPSTR lpCmdLine,
                   int nCmdShow)
{
    MSG  msg;

    if (!hPrevInstance)
        if (!InitApplication(hInstance))
            return (FALSE);

    if (!InitInstance(hInstance, nCmdShow))
        return (FALSE);
        
    while (GetMessage(&msg, NULL, NULL, NULL))
        {
        // Our multidlg is modal.  If it were modeless, we'd need a standard
        // call to IsDialogMessage() here.
        
        // if (ghMultiDlg && IsDialogMessage (ghMultiDlg, &msg))
        //     continue ;

        TranslateMessage(&msg);    
        DispatchMessage(&msg);     
        }

    return (msg.wParam);
}


//***************************************************************************
//
// FUNCTION: InitApplication()
//
// HISTORY: Date    Author  Reason
//          6/1/93  JRD     Created
//
//***************************************************************************

BOOL InitApplication(HINSTANCE hInstance)
{
    WNDCLASS  wc;

    wc.style = NULL;
    wc.lpfnWndProc = MainWndProc;
    wc.cbClsExtra = 0;     
    wc.cbWndExtra = 0;     
    wc.hInstance = hInstance; 
    wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName =  "MultidlgMenu";  
    wc.lpszClassName = "MultidlgWClass";

    return (RegisterClass(&wc));
}


//***************************************************************************
//
// FUNCTION: InitInstance()
//
// HISTORY: Date    Author  Reason
//          6/1/93  JRD     Created
//
//***************************************************************************

BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
    HWND hWnd ;
    
    ghInst = hInstance;

    hWnd = CreateWindow("MultidlgWClass",
                             "Multiple Child Dialogs Sample",
                             WS_OVERLAPPEDWINDOW | WS_VISIBLE,
                             CW_USEDEFAULT,
                             CW_USEDEFAULT,
                             CW_USEDEFAULT,
                             CW_USEDEFAULT,                 
                             NULL,
                             NULL,
                             hInstance,
                             NULL);
                             
    return (BOOL) hWnd ;
}


//***************************************************************************
//
// FUNCTION: MainWndProc()
//
// PURPOSE: Calls DialogBox() when we tell it to via the menu.
//
// HISTORY: Date    Author  Reason
//          6/1/93  JRD     Created
//
//***************************************************************************

long CALLBACK __export MainWndProc(HWND hWnd,
                                   UINT message,
                                   WPARAM wParam,
                                   LPARAM lParam)
{
    FARPROC lpfnDlg ;
    
    switch (message)
        {
        case WM_COMMAND:
            if (IDM_ABOUT == wParam)
                {
                lpfnDlg = MakeProcInstance (About, ghInst) ;
                DialogBox(ghInst, MAKEINTRESOURCE(IDD_ABOUT), hWnd, lpfnDlg);
                FreeProcInstance (lpfnDlg) ;
                break ;
                }

            if (IDM_DIALOG == wParam)
                {
                lpfnDlg = MakeProcInstance (MainDlgProc, ghInst) ;
                DialogBox (ghInst,
                           MAKEINTRESOURCE(IDD_MAINDLG),
                           hWnd,
                           lpfnDlg) ;
                FreeProcInstance (lpfnDlg) ;                           
                break ;
                }

            return (DefWindowProc(hWnd, message, wParam, lParam));
            
        case WM_DESTROY:
            PostQuitMessage(0);
            break;

        default:
            return (DefWindowProc(hWnd, message, wParam, lParam));
        }
        
    return 0L ;
}

//***************************************************************************
//
// FUNCTION: LoadAndCreateControls()
//
// PARAMETERS:
//      HWND hDlg       The parent on which these controls will be created.
//      int  iDialog    The child dialog in the diDlgs array that we want to
//                      create.  We get the resource ID from
//                      diDlgs[iDialog].iResID.
//
// PURPOSE:
//      Given a dialog resource ID and the target parent, thie function loads
//      the resource and creates the controls found within it onto the given
//      parent.  You can learn more about template parsing in the "Dialog Box
//      Resource" section in online help.
//
// HISTORY: Date    Author  Reason
//          6/1/93  JRD     Created
//
//***************************************************************************

void LoadAndCreateControls (HWND hDlg, int iDialog)
{
    HGLOBAL         hDlgResMem ;
    HRSRC           hDlgRes ;
    BYTE FAR        *lpDlgRes ;
    RECT            rc ;
    HFONT			hDlgFont ;
    LPPOINT         pt ;
    DWORD           style ;
    WORD            wID ;
    BYTE            bNumOfCtrls, bCurCtrl ;
    LPSTR           classname ;
    HWND            *hwnd, hAnchor ;
    int             xOffset, yOffset ;
    
    // We need to get the font of the dialog so we can set the font of
    // the child controls.  If the dialog has no font set, it uses the
    // default system font, and hDlgFont equals zero.
    
    hDlgFont = (HFONT) SendMessage (hDlg, WM_GETFONT, 0, 0L) ;
    
    // Load the resource into memory and get a pointer to it.

    hDlgRes    = FindResource (ghInst,
                               MAKEINTRESOURCE(diDlgs[iDialog].iResID),
                               RT_DIALOG) ;
    hDlgResMem = LoadResource (ghInst, hDlgRes) ;
    lpDlgRes   = (BYTE FAR *) LockResource (hDlgResMem) ;

    // Advance past the dialog header (DialogBoxHeader structure in online
    // help.
    
    style =     * (LPDWORD) lpDlgRes ;                     // Grab the style
    lpDlgRes    += sizeof (DWORD) ;                        // Pass the style
    
    bNumOfCtrls  = * (LPBYTE) lpDlgRes ;      // Grab the number of controls
    lpDlgRes    += sizeof (BYTE) ;            // Pass the number of controls 
    
    lpDlgRes    += 4 * sizeof (WORD) ;    // Pass x, y, cx, and cy of dialog
    
    if (0xFF == *lpDlgRes)
        lpDlgRes += 3 ;     // Menu by ordinal.  Skip 0xff and the ordinal.
    else
        {                   // Menu by name, or no menu at all.
        while (*lpDlgRes)
            ++lpDlgRes ;
        ++lpDlgRes ;      
        }

    while (*lpDlgRes) ++lpDlgRes ; ++lpDlgRes ;       // Pass the class name
    while (*lpDlgRes) ++lpDlgRes ; ++lpDlgRes ;       // Pass the caption
    
    // Some fields are present only if DS_SETFONT is specified.
    
    if (style & DS_SETFONT)
        {
        lpDlgRes += sizeof (WORD) ;                       // Pass point size
        while (*lpDlgRes) ++lpDlgRes ; ++lpDlgRes ;       // Pass face name
        }
        
        
    // Alloc enough local memory to hold the hwnd and text atom of each ctrl.
    
    diDlgs[iDialog].pCtrlHndls = (HWND NEAR *)
            LocalAlloc (LPTR, sizeof (HWND) + sizeof(HWND) * bNumOfCtrls) ;
            
    {
    BYTE bazoo ;
    
    bazoo = sizeof (ATOM) * bNumOfCtrls ;            

    diDlgs[iDialog].pCtrlTexts = (ATOM NEAR *)
            LocalAlloc (LPTR, bazoo) ;
    }            
            
    {
    ATOM *pTmp ;
    BYTE b = 1 ;
    
    pTmp = diDlgs[iDialog].pCtrlTexts ;
    
    for (b = 0; b < bNumOfCtrls; b++)
        pTmp++ ;
    }
    

    hwnd = (HWND *) diDlgs[iDialog].pCtrlHndls ;

    // The positioning of the controls will be offset according to the
    // top-right corner of the IDC_ANCHOR static control.
    
    hAnchor = GetDlgItem (hDlg, IDC_ANCHOR) ;
    GetWindowRect (hAnchor, &rc) ;
    pt = (LPPOINT) &rc ;
    ScreenToClient (hDlg, pt) ;
    pt++ ;
    ScreenToClient (hDlg, pt) ;
    xOffset = rc.right ;
    yOffset = rc.top ;
    
    // The rest of the dialog template contains ControlData structures.
    // We parse these structures and call CreateWindow() for each.

    for (bCurCtrl = 0; bCurCtrl < bNumOfCtrls; bCurCtrl++)
        {
        // ControlData coordinates are in dialog units.  We need to convert
        // these to pixels before adding the IDC_ANCHOR offset.
 
        CopyRect (&rc, (LPRECT)lpDlgRes) ; // Get the control's coordinates.
        
        MapDialogRect (hDlg, &rc) ;                    // Convert to pixels.
        rc.left += xOffset ;                           // Add the offset.
        rc.top += yOffset ;
        
        lpDlgRes += 4 * sizeof (WORD) ;   // Pass the control's coordinates.
        wID = * (LPWORD) lpDlgRes ;       // Get the control's ID.
        lpDlgRes += sizeof (WORD) ;       // Pass the control's ID.
        style = * (LPDWORD) lpDlgRes ;    // Get the control's styles.
        lpDlgRes += sizeof (DWORD) ;      // Pass the control's style.
        
        // At this point in the ControlData structure (see "Dialog Box
        // Resource" in online help), the class of the control may be
        // described either with text, or as a byte with a pre-defined
        // meaning.
        
        switch (*lpDlgRes)
            {
            case 0x80: classname = "button" ;    break ;
            case 0x81: classname = "edit" ;      break ;
            case 0x82: classname = "static" ;    break ;
            case 0x83: classname = "listbox" ;   break ;
            case 0x84: classname = "scrollbar" ; break ;
            case 0x85: classname = "combobox" ;  break ;
            default:
                classname = lpDlgRes ;
                while (lpDlgRes)  lpDlgRes++ ;
            }
        lpDlgRes++ ;  // Either passes the byte class identifier, or passes
                      // The null terminator of the class name string.

        *hwnd = CreateWindow (classname, lpDlgRes, style,
                                        rc.left, rc.top,
                                        rc.right, rc.bottom,
                                        hDlg, wID, ghInst, NULL) ;
                                        
		// Even though the font is the right size (MapDialogRect() did this),
		// we also need to set the font if it's not the system font.

    	if (hDlgFont)
    	    SendMessage (*hwnd, WM_SETFONT, hDlgFont, 0L) ;
                                        
        while (*lpDlgRes)  // Pass the window text.
            lpDlgRes++ ;

        lpDlgRes++ ; // Null terminator for the window text.
        lpDlgRes++ ; // Null terminator for ControlData structure (not well
                     // documented, but here nonetheless.)
        
        hwnd++ ;
        }
    *hwnd = 0 ;

    diDlgs[iDialog].fPresent = TRUE ;       
    
    // Free the resource which we just parsed.
    
    UnlockResource (hDlgResMem) ;
    FreeResource (hDlgResMem) ;
    
    // Send the new child a WM_INITDIALOG so it can do control initialization
    
    (*(diDlgs[iCurDlg].fpDlgProc))(hDlg, WM_INITDIALOG, 0, 0L) ;
}


//***************************************************************************
//
// FUNCTION: ChildDialogVisible()
//
// PARAMETERS:
//      BOOL b          Whether the given child dialog should be made visible
//                      or invisible.
//      HWND hDlg       Handle of the parent dialog.
//      int  iDialog    The child dialog in the diDlgs array that we want to
//                      make visible or invisible.
//
// COMMENTS:
//      This code works around a bug in IsDialogMessage().  The bug causes
//      mnemonics of hidden and even disabled controls to bother the normal
//      functioning of visible controls.  To get around this, we store the
//      text of the controls away while they are hidden, and restore the text
//      when we bring them back.
//
// HISTORY: Date    Author  Reason
//          6/1/93  JRD     Created
//
//***************************************************************************

void ChildDialogVisible (BOOL b, HWND hDlg, int iDialog)
{
    HWND    *hCtl ; 
    ATOM    *hTextAtm ;
    DWORD   dwCtlCode ;
    LONG    lStyle ;
    char    buf[80] ;
    int     i = 0 ;

    
    if ( (iDialog < 0) || (!diDlgs[iDialog].iResID) )
        return ;
        
    // If the child dialog hasn't been created before, do so.
    
    if (!diDlgs[iDialog].fPresent)
        LoadAndCreateControls (hDlg, iDialog) ;
        
    // Set one pointer to the array of hwnds, and another to the array of
    // window text atom handles.

    hCtl = diDlgs[iDialog].pCtrlHndls ;
    hTextAtm = diDlgs[iDialog].pCtrlTexts ;
    
    // If we're making this dialog visible, we call SetWindowText() to
    // restore the text, and ShowWindow() to make the control visible.
    // Otherwise we SW_HIDE the window, steal its text into the array,
    // and set its text temporarily to nothing.  This is necessary to fix
    // a bug in IsDialogMessage() involving mnemonics.
    
    if (b)
        {
        while (*hCtl)
            {
            if (*hTextAtm)
                {
                GetAtomName (*hTextAtm, buf, sizeof (buf)) ;
                SetWindowText (*hCtl, buf) ;
                DeleteAtom (*hTextAtm) ;
                }

            ShowWindow (*hCtl, SW_SHOW) ;
            hCtl++ ;
            hTextAtm++ ;
            }
        }
    else
        {
        while (*hCtl)
            {
            ShowWindow (*hCtl, SW_HIDE) ;
            
            // Ignore text of controls which accept text (like edit controls)
            // and of static controls which have the SS_NOPREFIX style.  (It
            // isn't necessary to ignore the latter, I guess.)

            dwCtlCode = SendMessage (*hCtl, WM_GETDLGCODE, 0, 0L) ;
            if (DLGC_WANTCHARS & dwCtlCode)
                {
                *hTextAtm = 0 ;
                goto ignoretext ;
                }
            
            if (DLGC_STATIC & dwCtlCode)
                {
                lStyle = GetWindowLong (*hCtl, GWL_STYLE) ;
                
                if (SS_NOPREFIX & lStyle)
                    {
                    *hTextAtm = 0 ;
                    goto ignoretext ;
                    }
                }

            GetWindowText (*hCtl, buf, sizeof (buf)) ;
            *hTextAtm = AddAtom (buf) ;
            SetWindowText (*hCtl, "") ;

ignoretext:            
            hCtl++ ;
            hTextAtm++ ;
            }
        }
}
