// Windows Picture Button

#include <windows.h>

const unsigned MAX_NAME_LENGTH = 31;
const unsigned FOCUS_BORDER_WIDTH = 2;

extern "C" {
  long far pascal _export PictureButtonProc(HWND, UINT, WPARAM, LPARAM);
}

/*-----------------------------------------------------------------------------
This function registers the PICTUREBUTTON class. It is called automatically
when the program starts up.

The external variables _hPrev and _hInstance duplicate the arguments
hPrevInstance and hInstance, which are passed to WinMain(). If the startup
code does not supply these external variables, you must pass the arguments to
this function and call it explicitly before invoking any PICTUREBUTTON
control.
-----------------------------------------------------------------------------*/

extern HINSTANCE _hPrev, _hInstance;

static void register_picture_button(void)
{
  if (!_hPrev)
  {
    WNDCLASS w;
    w.cbClsExtra = 0;
    w.cbWndExtra = sizeof(HGLOBAL);
    w.hbrBackground = (HBRUSH) COLOR_WINDOW + 1;
    w.hCursor = LoadCursor(NULL, IDC_ARROW);
    w.hIcon = NULL;
    w.hInstance = _hInstance;
    w.lpfnWndProc = PictureButtonProc;
    w.lpszClassName = "PICTUREBUTTON";
    w.lpszMenuName = NULL;
    w.style = 0;
    RegisterClass(&w);
  }
}

#pragma startup register_picture_button

struct PICTUREBUTTON
{
  BOOL disabled : 1;
  BOOL focus : 1;
  BOOL depressed : 1;
  BOOL captured : 1;
  BOOL tabstop : 1;
  unsigned unused : 11;
  unsigned height;
  unsigned width;
  HBITMAP picture, disabled_picture;
  HWND status_bar;
  char far *status_message;
};

/*-----------------------------------------------------------------------------
This function receives all messages directed to the control.
-----------------------------------------------------------------------------*/

long far pascal _export PictureButtonProc(HWND handle, UINT message,
  WPARAM wParam, LPARAM lParam)
{
  PICTUREBUTTON far *p;
  HGLOBAL hp = GetWindowWord(handle, 0);
  if (hp != NULL)
    p = (PICTUREBUTTON far *) GlobalLock(hp);
  switch (message)
  {
    case WM_CREATE:
    {
      unsigned width = ((CREATESTRUCT far *) lParam)->cx;
      unsigned height = ((CREATESTRUCT far *) lParam)->cy;
      hp = GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT, sizeof(PICTUREBUTTON));
        // + (height * width) / 2);
      if (hp == NULL)
        return -1L;
      p = (PICTUREBUTTON far *) GlobalLock(hp);
      p->height = height;
      p->width = width;
      if (((CREATESTRUCT far *) lParam)->style & WS_DISABLED)
        p->disabled = TRUE;
      if (((CREATESTRUCT far *) lParam)->style & WS_TABSTOP)
        p->tabstop = TRUE;
      char name[MAX_NAME_LENGTH + 2];
      {
        int c;
        for (unsigned i = 0; i < sizeof(name) - 2 &&
          (c = ((CREATESTRUCT far *) lParam)->lpszName[i]) != 0; i++)
        {
          name[i] = c;
        }
        name[i] = 0;
      }
      p->picture =
        LoadBitmap((HINSTANCE) GetWindowWord(handle, GWW_HINSTANCE), name);
      lstrcat(name, "X");
      p->disabled_picture =
        LoadBitmap((HINSTANCE) GetWindowWord(handle, GWW_HINSTANCE), name);
      SetWindowWord(handle, 0, (WORD) hp);
      GlobalUnlock(hp);
      return 0;
    }
    case BM_SETSTYLE:
      if (p->picture != NULL)
        DeleteObject(p->picture);
      if (p->disabled_picture != NULL && p->disabled_picture != p->picture)
        DeleteObject(p->disabled_picture);
      p->picture = (HBITMAP) LOWORD(lParam);
      p->disabled_picture = (HBITMAP) HIWORD(lParam);
      InvalidateRect(handle, NULL, TRUE);
      GlobalUnlock(hp);
      return 0;
    case BM_SETSTATE:
      //MessageBox(handle, "BM_SETSTATE", "", MB_OK);
      p->status_bar = (HWND) wParam;
      p->status_message = (char far *) lParam;
      GlobalUnlock(hp);
      return 0;
    case WM_PAINT:
    {
      PAINTSTRUCT paint;
      HDC dc = BeginPaint(handle, &paint);
      int height = p->height;
      int width = p->width;
      HPEN old_pen = SelectObject(dc, GetStockObject(BLACK_PEN));
      HBRUSH old_brush = SelectObject(dc, GetStockObject(LTGRAY_BRUSH));
      Rectangle(dc, 0, 0, width, height);
      SelectObject(dc, GetStockObject(p->depressed ? BLACK_PEN : WHITE_PEN));
      MoveTo(dc, 1, 1);
      LineTo(dc, width - 3, 1);
      LineTo(dc, width - 4, 2);
      LineTo(dc, 2, 2);
      LineTo(dc, 2, height - 4);
      LineTo(dc, 1, height - 3);
      LineTo(dc, 1, 1);
      SelectObject(dc, GetStockObject(p->depressed ? WHITE_PEN : BLACK_PEN));
      MoveTo(dc, width - 3, height - 3);
      LineTo(dc, width - 3, 3);
      LineTo(dc, width - 2, 2);
      LineTo(dc, width - 2, height - 2);
      LineTo(dc, 2, height - 2);
      LineTo(dc, 3, height - 3);
      LineTo(dc, width - 3, height - 3);
      HBITMAP picture = p->disabled ? p->disabled_picture : p->picture;
      {
        HDC memory = CreateCompatibleDC(dc);
        HBITMAP old_bitmap;
        BITMAP bm;
        int x, y;
        SetBkColor(dc, RGB(192, 192, 192));
        SetTextColor(dc, RGB(0, 0, 0));
        if (picture != NULL)
        {
          GetObject(picture, sizeof(BITMAP), &bm);
          old_bitmap = SelectObject(memory, picture);
          x = (width - bm.bmWidth) / 2;
          if (x < 5) x = 5;
          y = (height - bm.bmHeight) / 2;
          if (y < 5) y = 5;
          if (p->depressed)
          {
            x++;
            y++;
          }
          BitBlt(dc, x, y, bm.bmWidth, bm.bmHeight, memory, 0, 0, SRCCOPY);
          SelectObject(memory, old_bitmap);
          DeleteDC(memory);
        }
        else
        {
          x = y = 5 + FOCUS_BORDER_WIDTH;
          bm.bmWidth = width -  2*x;
          bm.bmHeight = height - 2*y;
        }
        if (p->tabstop && p->focus)
        {
          RECT r;
          r.left = x - FOCUS_BORDER_WIDTH;
          r.right = x + bm.bmWidth + FOCUS_BORDER_WIDTH;
          r.top = y - FOCUS_BORDER_WIDTH;
          r.bottom = y + bm.bmHeight + FOCUS_BORDER_WIDTH;
          DrawFocusRect(dc, &r);
        }
      }
      SelectObject(dc, old_pen);
      SelectObject(dc, old_brush);
      EndPaint(handle, &paint);
      GlobalUnlock(hp);
      return 0;
    }
    case WM_ENABLE:
      if (p->disabled && wParam || !p->disabled && !wParam)
      {
        p->disabled = !p->disabled;
        InvalidateRect(handle, NULL, TRUE);
      }
      GlobalUnlock(hp);
      return 0;
    case WM_LBUTTONDOWN:
      p->captured = TRUE;
      p->depressed = TRUE;
      InvalidateRect(handle, NULL, TRUE);
      SetCapture(handle);
      if (p->tabstop)
        SetFocus(handle);
      if (p->status_bar != NULL && p->status_message != NULL)
        SetWindowText(p->status_bar, p->status_message);
      GlobalUnlock(hp);
      return 0;
    case WM_LBUTTONUP:
    {
      BOOL notify = !p->captured ||
        LOWORD(lParam) < p->width && HIWORD(lParam) < p->height;
      p->depressed = FALSE;
      p->captured = FALSE;
      InvalidateRect(handle, NULL, TRUE);
      ReleaseCapture();
      if (p->status_bar != NULL && p->status_message != NULL)
        SetWindowText(p->status_bar, "");
      if (notify)
      {
        SendMessage(GetParent(handle), WM_COMMAND, GetMenu(handle),
          MAKELONG(handle, BN_CLICKED));
      }
      GlobalUnlock(hp);
      return 0;
    }
    case WM_MOUSEMOVE:
      if (p->captured)
      {
        BOOL old_depressed = p->depressed;
        p->depressed = LOWORD(lParam) < p->width && HIWORD(lParam) < p->height;
        if (p->depressed != old_depressed)
        {
          InvalidateRect(handle, NULL, TRUE);
          GlobalUnlock(hp);
          return 0;
        }
      }
      break;
    case WM_SETFOCUS:
      p->focus = TRUE;
      InvalidateRect(handle, NULL, TRUE);
      GlobalUnlock(hp);
      return 0;
    case WM_KILLFOCUS:
      p->focus = FALSE;
      InvalidateRect(handle, NULL, TRUE);
      GlobalUnlock(hp);
      return 0;
    case WM_GETDLGCODE:
      GlobalUnlock(hp);
      return DLGC_BUTTON;
    case WM_KEYDOWN:
      if (wParam == ' ' && !p->captured && !p->depressed)
      {
        p->depressed = TRUE;
        InvalidateRect(handle, NULL, TRUE);
        GlobalUnlock(hp);
        return 0;
      }
      break;
    case WM_KEYUP:
      if (wParam == ' ' && !p->captured)
      {
        p->depressed = FALSE;
        InvalidateRect(handle, NULL, TRUE);
        SendMessage(GetParent(handle), WM_COMMAND, GetMenu(handle),
          MAKELONG(handle, BN_CLICKED));
        GlobalUnlock(hp);
        return 0;
      }
      break;
    case WM_DESTROY:
      if (p->picture != NULL)
        DeleteObject(p->picture);
      if (p->disabled_picture != NULL && p->disabled_picture != p->picture)
        DeleteObject(p->disabled_picture);
      GlobalUnlock(hp);
      GlobalFree(hp);
      return 0;
  }
  return DefWindowProc(handle, message, wParam, lParam);
}

