/**************************************************************************
	RipTABS - Tab Folder Dialog        Version 4.0 (March 10, 1995)

	(c) 1993 - 1995 Jonathan Carroll - All Rights Reserved
	
	You are licensed to use RipTABS subject to certain restrictions-- see
	your license agreement for details

	All RipTABS source code is formatted with tabs for indenting purposes.
	Tabs were set to two (2) spaces-- viewing these files with tab spacing
	set to some other value may lead to strange formatting.
	
	History Track:
		Version 2.02 	April 15, 1994
		Version 2.1 	May 15, 1994
		Version 3.0		August 12, 1994
		Version 3.0a	September 3, 1994
		Version 3.1		December 10, 1994
		Version 3.2   February 25, 1995
		Version 4.0		March 10, 1995
	
	Changes made subsequent to Version 2.1 may be found in the code
	by doing a search on 'Version 3.0' or Version 3.0a.

**************************************************************************/

/**************************************************************************
	RIP_TABS.H

	This is the header file for the "public portion" of RipTABS.  It
	contains declarations for the two RipTABS classes : CTabTitle &
	CTabDialog.  It also contains declarations for a number of public
	end-user calls such as RipTABRegister & RipTABUnregister.

**************************************************************************/
#define RIPTABBASECLASS CDialog

#ifndef __RIPTABS_H
#define __RIPTABS_H

#ifdef win32
#define _export
#endif

#define TAB_DISABLED 1

// New in Version 3.1
typedef struct tagMOVEINFO 
{
	LPVOID  dlg;
	int			offset;
	HWND    hWndDialog;

} MOVEINFO;

// New in Version 3.1
#define STDFont (CFont*)-100

// Public utility functions.  Documented in the .CPP code files
void RipTABRegister();
void RipTABUnregister();
CString* CStringPtr(const char*);

// Forward declaration of CTabTitle so that CTabDialog can reference it.
class CTabTitle;


/**************************************************************************
	Class : CTabDialog
	
	This is the only class that the end user need work with.  It handles 
	the construction, destruction and management of the tab dialog

	New In Version 3: Last parameter in constructor determines whether tabs 
	are square.

**************************************************************************/
class CTabDialog : public RIPTABBASECLASS
{
public:
	// Modal Construction
	CTabDialog(int theDialog, CWnd* pParent = NULL, int theActiveTab=0, 
							BOOL AllTheSame=FALSE, BOOL Square=FALSE, BOOL bVertical=FALSE);
	// Modeless Construction
	CTabDialog(int theActiveTab=0, BOOL AllTheSame=FALSE, BOOL Square=FALSE, BOOL bVertical=FALSE);
	//Destructor
	~CTabDialog();
	
	// Navigational functions
	void NextTab();
	void PrevTab();

	// Public Information functions.
	CString& 			ActiveTabText() { return rtab_ActiveTabText; }; 
	int						ActiveTab() { return rtab_ActiveTab; };
	int						LastActiveTab() { return rtab_LastActiveTab; };
	int 					TabTextYpos() { return rtab_TabTextPos; };
	// New in V4.0
	void					DisableTab(int nTab);
	void					EnableTab(int nTab);
	void					RedrawTab(int nTab);
	BOOL					IsTabEnabled(int nTab) { return !(rtab_StateArray[nTab] & TAB_DISABLED); };
	void 					SetTabText(int nTab, LPCSTR szText);
	CString				GetTabText(int nTab);
	// End of New in V4.0
	
	void 					SetActiveTab(int theTab);
	CPtrArray*		rtab_OptionsList[2];
	// New in Version 3.0
	CWordArray*		rtab_BitmapList;

	// New in Version 4.0 : Tab Text Color List
	CDWordArray*	rtab_TabTextColorList;

	// New in Version 4.0 : Vertical Tabs?
	BOOL					rtab_bVertical;

	afx_msg void 	OnLButtonDown(UINT flags, CPoint point);
	
	// Internal variables.  They store the index in the child
	// dialog array of the currently-active tab and the last 
	// active tab.
	int						rtab_LastActiveTab, rtab_ActiveTab;
	
	// New in Version 3.0  This next var will track the tab line
	// that the last active tab was on.  If the user selects 
	// another tab on the same line, we don't have to paint as
	// much of the dialog.
	int						rtab_LastActiveTabLine;
	
	// New in Version 3.0.  We use this next tab to block redraws
	// while controls are hiding/unhiding.
	BOOL					rtab_BlockRedraw;
	
	// Internal variables.  m_TabHasFocus' value changes depending
	// on whether the tab title has focus in the dialog.  
	// m_AllTheSame is set to tru if the user has indicated that
	// all tab groups are to be constructed from the same dialog
	// resource. 
	BOOL					m_TabHasFocus, m_AllTheSame;
	
	// New in V4.0 OnOK and OnCancel are public
	// OnOK & OnCancel handle default dialog behaviour.
	virtual void 	OnOK();
	virtual void 	OnCancel();

	friend class  CTabTitle;

protected:

	// InitData is called by a derived class.  Construction of the dialog is 
	// initialized here                            
	// New in Version 3.0: Added parameters for normal and bold dlg font as well
	// as a tab bitmap list. 
	// New in Version 3.1: Added parameter for dialog controls font
	void 					InitData(CPtrArray* theOptionsList1, 
													CPtrArray* theOptionsList2, 
													int TheStaticBox, 
													int theActiveTab=-1, 
													CWordArray* theBitmapList=NULL,
													CDWordArray* theTabTextColorList=NULL,
													CFont* pNormalFont=NULL, 
													CFont* pBoldFont=NULL,
													CFont* pControlFont=NULL, 
													COLORREF cActiveTabTextColor=0, 
													int MaxTabsPerLine=32000);
	
	
	// The resizing of the tab dialog to fit tabs etc is all done here.
	virtual BOOL 	OnInitDialog();
	
	// OnPaint handles paint messages, draws tabs etc.
	afx_msg void 	OnPaint();
	
	// New in Version 3.1
	afx_msg void OnDestroy();
	
	// These two functions notify derived objects that the user 
	// has switched tab groups.  The virtual keyword ensures that
	// the derevided classes member (if defined) gets called on not
	// CTabDialog's.  These functions in the base class do nothing.
	virtual void 	SwitchToNotification(int TabIndex, CString* TabName);
	virtual void 	SwitchFromNotification(int TabIndex, CString* TabName);

	//New in Version 3.0
	//This virtual function gets called prior to a switch initiated by the 
	//user.  Return TRUE to allow the switch to proceed, otherwise FALSE if
	//you want to prevent the switch.
	virtual BOOL  OKToSwitchTo(int TabIndex, CString* TabName);


public:
	// BoxRect holds a rectangle describing the bounding
	// static control.
	CRect					rtab_BoxRect;

	// This next variables indicate (in order) : The width of 
	// a single tab, the space between tab text and tab border,
	// the number of tabs accross a single tab line, the number
	// of rows of tabs, and the height of a tab.
	int						rtab_FTabWidth, rtab_Space, rtab_NumAccross;
	int						rtab_TabLines, rtab_LineHeight;

private:
	// This group of variables store important SectionDialog
	// information relating to the active dialog resource etc.
	void FAR*		 	rtab_lpsectiondir;
	CPtrArray			rtab_RectArray;
  CWordArray		rtab_SectionIndexArray;

	// New in V4.0 a tab state array (enabled, disabled etc)
	CWordArray		rtab_StateArray;


	int						rtab_OrderList[20];
	
	// This variable holds the resource id of the bounding
	// static control passed to us by the derived class.
	int						rtab_IDC_CSTATIC;
	
	// This is the offset between tab pages.  (Tab pages
	// are offset towards the right as you add more tab
	// lines.)
	int						rtab_LineOffset;
	
	// New in Version 3.0
	// these vars hold width & height of tab bitmaps
	int						rtab_BitmapWidth, rtab_BitmapHeight;

	// New in V4.0
	int						rtab_BoxWidth, rtab_BoxHeight;	
	
	// New in Version 3.0: BOOLEAN var determines whether
	// tabs are square.  (default is rounded)
	BOOL 					rtab_SquareTabs;
	
	
	// Pointer to the dialog font.
	CFont*				rtab_DlgFont;
	
	// Version 3.0 change : new members for tab font
	CFont*				rtab_NormalTabFont;
	CFont*				rtab_BoldTabFont; 
	
	// Version 3.1 change : new member for control font
	CFont*				rtab_ControlFont;
	
	// Version 3.1 : flags for whether we need to delete 
	// font objects       
	BOOL					rtab_bNormalTabFontCreated;
	BOOL					rtab_bBoldTabFontCreated;
	BOOL					rtab_bControlFontCreated;

	// Version 4.0 : Tab Text Colors
	COLORREF			rtab_cActiveTabTextColor;

	CWordArray		rtab_RowHeaderList;
	
	// Tab title text.
	CTabTitle*		TabTitle;
	
	// The text contained in the active tab
	CString				rtab_ActiveTabText; 
	
	int						rtab_TabTextPos;
	
	// New in Version 3.0.  BOOL var to track whether
	// a normal and bold font was passed in during
	// construction
	BOOL					bCustomFont;

	// New in Version 3.0.  int var that can be set
	// by calling process to limit number of tabs
	// per line.
	int rtab_MaxNumPerLine;
	

	DECLARE_MESSAGE_MAP()
};

class CTabTitle : public CWnd
{
	public:
	CTabTitle(CTabDialog* pParent, RECT& theRect);
	void PostNcDestroy();
	afx_msg void OnPaint();
	void RePosition(HWND hwndInsertAfter); 
	afx_msg void OnSetFocus(CWnd* OldWnd);  
	afx_msg void OnKillFocus(CWnd* NewWnd);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	
	private:
	CTabDialog* Parent;
	BOOL				HaveFocus;

	DECLARE_MESSAGE_MAP()
};

#endif
