/*----------------------------------------------------------------------------*\
|   timesmp.c - A demonstration of the Multimedia timer services               |
|   Copyright (c) 1991 Microsoft Corporation. All rights reserved.             |
\*----------------------------------------------------------------------------*/

#include <windows.h>
#include <mmsystem.h>
#include "timesmp.h"

/*----------------------------------------------------------------------------*\
|                                                                              |
|   g l o b a l   v a r i a b l e s                                            |
|                                                                              |
\*----------------------------------------------------------------------------*/

char    gszCaption[] = "Timer Demo Application";
HWND    ghWnd;
HANDLE  ghInst;
DWORD   gdwElapsed;
WORD    gwID;
WORD    gwResolution = 20;
WORD    gwPeriod = 2000;
WORD    gwFlags = TIME_ONESHOT;

/*----------------------------------------------------------------------------*\
|                                                                              |
|   f u n c t i o n   d e f i n i t i o n s                                    |
|                                                                              |
\*----------------------------------------------------------------------------*/

void FAR PASCAL TimerCallBack(WORD,WORD,DWORD,DWORD,DWORD);
BOOL AppInit(HANDLE hInst,HANDLE hPrev);
int PASCAL WinMain(HANDLE hInst, HANDLE hPrev, LPSTR szCmdLine, WORD wCmdShow);
LONG FAR PASCAL AppWndProc(HWND hWnd,WORD wMessage,WORD wParam,LONG lParam);
LONG NEAR PASCAL AppCommand(HWND hWnd,WORD wMessage,WORD wParam,LONG lParam);
BOOL FAR PASCAL AboutWndProc(HWND hDlg, WORD wMessage, WORD wParam, LONG lParam);
BOOL FAR PASCAL ConfigWndProc(HWND hDlg, WORD wMessage, WORD wParam, LONG lParam);

/*----------------------------------------------------------------------------*\
|   AppInit( hInst, hPrev)                                                     |
|                                                                              |
|   Description:                                                               |
|  This procedure is called when the application is first loaded into          |
|  memory.                                                                     |
|                                                                              |
|   Arguments:                                                                 |
|  hInst    instance handle of current instance                                |
|  hPrev    instance handle of previous instance                               |
|                                                                              |
|   Returns:                                                                   |
|  TRUE if successful, FALSE if not                                            |
|                                                                              |
\*----------------------------------------------------------------------------*/
BOOL AppInit(HANDLE hInst,HANDLE hPrev)
   {
   WNDCLASS    cls;

   /* Save the instance handle for the DialogBox */
   ghInst = hInst;

   if (!hPrev) 
      {

      /* Register a class for the main application window */

      cls.hCursor        = LoadCursor(NULL,IDC_ARROW);
      cls.hIcon          = LoadIcon(hInst,MAKEINTATOM(ID_APP));
      cls.lpszMenuName   = MAKEINTATOM(IDD_MENU);
      cls.lpszClassName  = MAKEINTATOM(ID_APP);
      cls.hbrBackground  = (HBRUSH)COLOR_WINDOW + 1;
      cls.hInstance      = hInst;
      cls.style          = CS_BYTEALIGNCLIENT | CS_VREDRAW | CS_HREDRAW;
      cls.lpfnWndProc    = AppWndProc;
      cls.cbWndExtra     = 0;
      cls.cbClsExtra     = 0;

      if (!RegisterClass(&cls))
         return FALSE;
      }

   ghWnd = CreateWindow (MAKEINTATOM(ID_APP),    // Class name
                         gszCaption,             // Caption
                         WS_OVERLAPPEDWINDOW,    // Style bits
                         CW_USEDEFAULT,          // Default horizontal position.
                         CW_USEDEFAULT,          // Default vertical position.
                         CW_USEDEFAULT,          // Default width.
                         CW_USEDEFAULT,          // Default height.
                         (HWND)NULL,             // Parent window (no parent)
                         (HMENU)NULL,            // use class menu
                         (HANDLE)hInst,          // handle to window instance
                         (LPSTR)NULL             // no params to pass on
                         );



   return (ghWnd != NULL);
   }

/*----------------------------------------------------------------------------*\
|   WinMain( hInst, hPrev, lpszCmdLine, cmdShow )                              |
|                                                                              |
|   Description:                                                               |
|                                                                              |
|   Arguments:                                                                 |
|  hInst        instance handle of this instance of the application            |
|  hPrev        instance handle of previous instance. NULL if this is the      |
|                   instance of the application                                |
|  szCmdLine    null-terminated command line                                   |
|  cmdShow      specifies how the application's window is initially displayed  |
|                                                                              |
|   Returns:                                                                   |
|       The exit code as specified in the WM_QUIT message.                     |
|                                                                              |
\*----------------------------------------------------------------------------*/
int PASCAL WinMain(HANDLE hInst, HANDLE hPrev, LPSTR szCmdLine, WORD wCmdShow)
   {
   MSG     msg;
   
   /* Call the initialization procedure */
   if (!AppInit(hInst,hPrev))
      return 1;

   ShowWindow(ghWnd,wCmdShow);
   UpdateWindow(ghWnd);

   /* Enter main message loop */
   while (GetMessage (&msg, NULL, 0, 0))
      {
      TranslateMessage (&msg);
      DispatchMessage (&msg);
      }

   return 0;
   }


/*----------------------------------------------------------------------------*\
|   AppWndProc( hWnd, wMessage, wParam, lParam )                               |
|                                                                              |
|   Description:                                                               |
|       The window proc for the application's main window.  This processes all |
|       of the parent window's messages.                                       |
|                                                                              |
|   Arguments:                                                                 |
|       hWnd            window handle for the window                           |
|       wMessage        message number                                         |
|       wParam          message-dependent                                      |
|       lParam          message-dependent                                      |
|                                                                              |
|   Returns:                                                                   |
|       0 if processed, DefWindowProc if ignored                               |
|                                                                              |
\*----------------------------------------------------------------------------*/
LONG FAR PASCAL AppWndProc(HWND hWnd,WORD wMessage,WORD wParam,LONG lParam)
   {

   switch (wMessage) 
      {
      case WM_COMMAND:
         return AppCommand(hWnd,wMessage,wParam,lParam);

      case WM_DESTROY:
         PostQuitMessage(0);
         break;
      }
      
   return DefWindowProc(hWnd,wMessage,wParam,lParam);
   }


/*----------------------------------------------------------------------------*\
|   AppCommand( hWnd, wMessage, wParam, lParam )                               |
|                                                                              |
|   Description:                                                               |
|       This function handles the WM_COMMAND messages passed from the main     |
|       application window.                                                    |
|                                                                              |
|   Arguments:                                                                 |
|       hWnd            window handle for the window                           |
|       wMessage        message number                                         |
|       wParam          message-dependent                                      |
|       lParam          message-dependent                                      |
|                                                                              |
|   Returns:                                                                   |
|       0 if processed, nonzero if ignored                                     |
|                                                                              |
\*----------------------------------------------------------------------------*/
LONG NEAR PASCAL AppCommand(HWND hWnd,WORD wMessage,WORD wParam,LONG lParam)
   {
   FARPROC lpfn;

   switch(wParam)
      {
      case IDM_CONFIGURE:
      
         /* put up the Configure... dialog box */
         
         lpfn = MakeProcInstance(ConfigWndProc, ghInst);

         DialogBox(ghInst,          /* current instance   */
                   MAKEINTRESOURCE(IDD_CONFIG),    /* resource to use    */
                   hWnd,            /* parent handle      */
                   lpfn);           /* instance address   */

         FreeProcInstance(lpfn);
         break;


      case IDM_ABOUT:
      
         /* put up the About... dialog box */
         lpfn = MakeProcInstance(AboutWndProc, ghInst);

         DialogBox(ghInst,          /* current instance   */
                   MAKEINTRESOURCE(IDD_ABOUT),     /* resource to use    */
                   hWnd,            /* parent handle      */
                   lpfn);           /* instance address   */

         FreeProcInstance(lpfn);
         break;


      case IDM_STARTSTOP:
         {
         HMENU hMenu;
         static BOOL fTimerStarted;

         /* fix up our menu item to reflect the present state */
         hMenu = GetMenu(hWnd);
         ModifyMenu(hMenu,IDM_STARTSTOP,MF_STRING,IDM_STARTSTOP,
                    (fTimerStarted ? "&Start Timer":"&Stop Timer"));
         DrawMenuBar(hWnd);

         if (fTimerStarted)
            {
            fTimerStarted = FALSE;

            timeKillEvent(gwID);          /* destroy the timer callback event */
            timeEndPeriod(gwResolution);  /* clear the minimum we set         */

            }
         else
            {
            /* remember our starting time */
            gdwElapsed = timeGetTime();

            fTimerStarted = TRUE;

            timeBeginPeriod(gwResolution);      /* set the minimum resolution */

            /*  Set up the callback event.  The callback function 
             *  MUST be in a FIXED CODE DLL!!!
             */
            gwID = timeSetEvent(gwPeriod,       /* how often                 */
                                gwResolution,   /* timer resolution          */
                                TimerCallBack,  /* callback function         */
                                (LONG)ghWnd,    /* info to pass to callback  */
                                gwFlags);       /* oneshot or periodic?      */
                                
            if (!gwID)
               {
               MessageBeep(MB_ICONHAND);
               MessageBox(ghWnd,"Uh-Oh","timeSetEvent failed.",MB_ICONHAND|MB_OK);
               fTimerStarted = FALSE;
               }
            }
            
         }
         break;


      case IDM_TIMER:
         {
         /* message posted back to us by our callback function 
            when a timer goes off.  The DLL has passed the current time
            in the lParam paramater of this message. */
         
         HDC hDC;
         char szBuf[128];
         
         hDC = GetDC(hWnd);
         wsprintf(szBuf,
              "Timer message received. Elapsed time: %ld milliseconds.         ",
              lParam - gdwElapsed);
         TextOut(hDC,10,10,szBuf,lstrlen(szBuf));
         ReleaseDC(hWnd,hDC);
         gdwElapsed = lParam;

         /* for a one-shot, we need to kill the event */
         if (gwFlags == TIME_ONESHOT)
            PostMessage(hWnd,WM_COMMAND,IDM_STARTSTOP,0L);
         }
         break;


      default:
         return 1L;
      }

   return 0L;
   }


/*----------------------------------------------------------------------------*\
|   ConfigWndWndProc( hWnd, wMessage, wParam, lParam )                         |
|                                                                              |
|   Description:                                                               |
|       The window proc for the configure dialog box.                          |
|                                                                              |
|   Arguments:                                                                 |
|       hWnd            window handle for the window                           |
|       wMessage        message number                                         |
|       wParam          message-dependent                                      |
|       lParam          message-dependent                                      |
|                                                                              |
|   Returns:                                                                   |
|       TRUE if processed, FALSE if ignored                                    |
|                                                                              |
\*----------------------------------------------------------------------------*/
BOOL FAR PASCAL ConfigWndProc(HWND hDlg, WORD wMessage, WORD wParam, LONG lParam)
   {
   switch (wMessage)
      {
      case WM_INITDIALOG:
         SetDlgItemInt(hDlg,IDD_RESOLUTION,gwResolution,FALSE);
         SetDlgItemInt(hDlg,IDD_PERIOD,gwPeriod,FALSE);
         if (gwFlags == TIME_ONESHOT)
            CheckRadioButton(hDlg,IDD_ONESHOT,IDD_PERIODIC,IDD_ONESHOT);
         else
            CheckRadioButton(hDlg,IDD_ONESHOT,IDD_PERIODIC,IDD_PERIODIC);
         return TRUE;

      case WM_COMMAND:
         switch (wParam)
            {
            BOOL fTranslated;
            
            case IDOK:
               gwPeriod     = GetDlgItemInt(hDlg,IDD_PERIOD,&fTranslated,FALSE);
               if (!fTranslated)
                  {
                  /* reset to our default value */
                  gwPeriod = 2000;
                  EndDialog(hDlg, FALSE);
                  return TRUE;
                  }
               
               gwResolution = GetDlgItemInt(hDlg,IDD_RESOLUTION,&fTranslated,FALSE);
               if (!fTranslated)
                  {
                  /* reset to our default value */
                  gwResolution = 20;
                  EndDialog(hDlg, FALSE);
                  return TRUE;
                  }
               
               if (IsDlgButtonChecked(hDlg,IDD_ONESHOT))
                  gwFlags = TIME_ONESHOT;
               else
                  gwFlags = TIME_PERIODIC;
                  
               /* fall through... */
                  
            case IDCANCEL:
               EndDialog(hDlg, (wParam == IDOK));
               return TRUE;
               
            default:
               break;
            }
         break;
         
      default:
         break;
      }

   return FALSE;
   }
   
   
/*----------------------------------------------------------------------------*\
|   AboutWndProc( hWnd, wMessage, wParam, lParam )                             |
|                                                                              |
|   Description:                                                               |
|       The window proc for the about box.                                     |
|                                                                              |
|   Arguments:                                                                 |
|       hWnd            window handle for the window                           |
|       wMessage        message number                                         |
|       wParam          message-dependent                                      |
|       lParam          message-dependent                                      |
|                                                                              |
|   Returns:                                                                   |
|       TRUE if processed, FALSE if ignored                                    |
|                                                                              |
\*----------------------------------------------------------------------------*/
BOOL FAR PASCAL AboutWndProc(HWND hDlg, WORD wMessage, WORD wParam, LONG lParam)
   {
   switch (wMessage)
      {
      case WM_INITDIALOG:
         return TRUE;

      case WM_COMMAND:
         EndDialog(hDlg, TRUE);
         return TRUE;
      }

   return FALSE;
   }

