#include <types.h>
#include <dos.h>
#include "vxdcalls.h"
#include <vxdfns.h>


BOOL Get_VMM_Version (BYTE *Major, BYTE *Minor, DWORD *Debug_Ver)
{
    VxDParams   p;
    BOOL        ret;

    p.CallNum = zGet_VMM_Version;
    ret = VxDCall (&p);
    *Minor      = (BYTE) (p.OutEAX & 0xFF);
    *Major      = (BYTE) ((p.OutEAX & 0xFF00) >> 8);
    *Debug_Ver  = p.OutECX;
    return ret;
}
/*                                                                            */
/*  Return Non-zero if Handle is valid                                        */
/*                                                                            */
DWORD Validate_VM_Handle (DWORD VM_Handle)
{
    VxDParams    p;

    p.CallNum = zValidate_VM_Handle;
    p.InEBX = VM_Handle;
    return (VxDCall (&p)) ? !(p.OutEFLAGS & CARRYFLAG) : 0;
}
DWORD Get_Cur_VM_Handle (void)
{
    VxDParams    p;

    p.CallNum = zGet_Cur_VM_Handle;
    return (VxDCall (&p)) ? p.OutEBX : 0;
}
DWORD Get_Sys_VM_Handle (void)
{
    VxDParams    p;

    p.CallNum = zGet_Sys_VM_Handle;
    return (VxDCall (&p)) ? p.OutEBX : 0;
}
DWORD Get_Next_VM_Handle (DWORD VM_Handle)
{
    VxDParams    p;

    p.InEBX = VM_Handle;
    p.CallNum = zGet_Next_VM_Handle;
    return (VxDCall (&p)) ? p.OutEBX : 0;
}

DWORD Get_Execution_Focus (void)
{
    VxDParams    p;

    p.CallNum = zGet_Execution_Focus;
    return (VxDCall (&p)) ? p.OutEBX : 0;
}

DWORD SHELL_SYSMODAL_Message (DWORD Flags, char *Caption, char *szFormat, ...)
{
    VxDParams    p;
    char szBuffer[256];
    char *pArgs;

    pArgs = (char *) &szFormat + sizeof szFormat;
    wsprintf (szBuffer, szFormat, pArgs);

    p.CallNum = zSHELL_SYSMODAL_Message;
    p.InEAX = Flags;
    p.InEBX = Get_Cur_VM_Handle();
    p.InECX = MapFlat (szBuffer);
    p.InEDI = MapFlat (Caption);
    return (VxDCall (&p)) ? p.OutEAX : 0;
}

DWORD SHELL_Message (DWORD *retval, DWORD Flags, char *Caption, char *szFormat, ...)
{
    VxDParams    p;
    char szBuffer[256];
    char *pArgs;
    unsigned int ds_reg;

    _asm mov ss:ds_reg, ds

    pArgs = (char *) &szFormat + sizeof szFormat;
    wsprintf (szBuffer, szFormat, pArgs);

    p.CallNum = zSHELL_Message;
    p.InEAX = Flags;
    p.InEBX = Get_Cur_VM_Handle();
    p.InECX = MapFlat (szBuffer);
    p.InEDI = MapFlat (Caption);
    p.InEDX = MapFlat (retval);
    p.InESI = MapFlat (&CB_GetEAX);
    return (VxDCall (&p));
}

BOOL System_Control (WORD Message, WORD Device)
{
    VxDParams    p;

    p.CallNum = zSystem_Control;
    p.InEAX   = Message;
    p.InEBX   = Get_Cur_VM_Handle();
    p.InEDX   = Device;
    p.InESI   = Device;
    p.InEDI   = Device;
    return (VxDCall (&p));
}

BOOL VDD_Set_VMType (DWORD Message)
{
    VxDParams    p;

    p.CallNum = zVDD_Set_VMType;
    p.InEAX   = Message;
    p.InEBX   = Get_Cur_VM_Handle();
    return (VxDCall (&p));
}

BOOL SHELL_Event (DWORD Event, DWORD Parm)
{
    VxDParams    p;

    p.CallNum = zSHELL_Event;
    p.InEAX   = Parm;
    p.InEBX   = Get_Cur_VM_Handle();
    p.InECX   = Event;
    p.InEDX   = 0;
    p.InESI   = 0;
    return (VxDCall (&p));
}

BOOL Set_Execution_Focus ()
{
    VxDParams    p;

    p.CallNum = zSet_Execution_Focus;
    p.InEBX   = Get_Cur_VM_Handle();
    return (VxDCall (&p));
}

BOOL CopyPageTable (DWORD StartPage, WORD NumPages, DWORD *Array)
{
    VxDPushParams p;

    p.CallNum = z_CopyPageTable;
    p.NumP = 4;
    p.P[0] = StartPage;
    p.P[1] = NumPages;
    p.P[2] = MapFlat(Array);
    p.P[3] = 0;
    VxDPushCall(&p);
    return p.OutEAX;
}
DWORD PageAllocate (DWORD nPages, DWORD pType, DWORD VMHandle, DWORD Mask,
                    DWORD MinPhys, DWORD MaxPhys, DWORD *PhysAddrPtr, DWORD flag,
                    DWORD *hPage)
{
    VxDPushParams p;

    p.CallNum = z_PageAllocate;
    p.NumP = 8;
    p.P[0] = nPages;
    p.P[1] = pType;
    p.P[2] = VMHandle;
    p.P[3] = Mask;
    p.P[4] = MinPhys;
    p.P[5] = MaxPhys;
    p.P[6] = MapFlat(PhysAddrPtr);
    p.P[7] = flag;
    VxDPushCall(&p);
    *hPage = p.OutEAX;
    return p.OutEDX;
}
DWORD PageFree (DWORD hMem)
{
    VxDPushParams p;

    p.CallNum = z_PageFree;
    p.NumP = 2;
    p.P[0] = hMem;
    p.P[1] = 0;
    VxDPushCall(&p);
    return p.OutEAX;
}

DWORD PageLock (DWORD hMem, DWORD nPages, DWORD PageOff, DWORD flags)
{
    VxDPushParams p;

    p.CallNum = z_PageLock;
    p.NumP = 4;
    p.P[0] = hMem;
    p.P[1] = nPages;
    p.P[2] = PageOff;
    p.P[3] = flags;
    VxDPushCall(&p);
    return p.OutEAX;
}

DWORD PageUnLock (DWORD hMem, DWORD nPages, DWORD PageOff, DWORD flags)
{
    VxDPushParams p;

    p.CallNum = z_PageUnLock;
    p.NumP = 4;
    p.P[0] = hMem;
    p.P[1] = nPages;
    p.P[2] = PageOff;
    p.P[3] = flags;
    VxDPushCall(&p);
    return p.OutEAX;
}

DWORD Wait_For_Response (DWORD *retval)
{
    for (;;)                            /* Wait for user response             */
        if (*retval != -1) return *retval;
}

DWORD LinMapIntoV86 (DWORD StartPage, DWORD VMHandle, DWORD V86Page,
                     DWORD nPages, DWORD flags)
{
    VxDPushParams p;

    p.CallNum = z_LinMapIntoV86;
    p.NumP = 5;
    p.P[0] = StartPage;
    p.P[1] = VMHandle;
    p.P[2] = V86Page;
    p.P[3] = nPages;
    p.P[4] = flags;
    VxDPushCall(&p);
    return p.OutEAX;
}
BOOL  GetDescriptor (WORD Selector, DWORD VMHandle, DWORD *desc)
{
    VxDPushParams p;

    p.CallNum = z_GetDescriptor;
    p.NumP = 3;
    p.P[0] = Selector;
    p.P[1] = VMHandle;
    p.P[2] = 0;
    VxDPushCall(&p);
    *desc++ = p.OutEAX;
    *desc = p.OutEDX;
    if (p.OutEAX || p.OutEDX) return 1;
    return 0;
}
BOOL Get_Device_Version (WORD DeviceId, WORD *VerMaj, WORD *VerMin)
{
    VxDParams    p;
    BOOL        ret;
    DWORD       Version;
    DWORD       zGet_Ver;

    zGet_Ver = DeviceId;
    zGet_Ver <<= 16;
    zGet_Ver &= 0xFFFF0000;
    p.CallNum = zGet_Ver;

    ret = VxDCall (&p);
    Version = p.OutEAX;
    *VerMin = Version & 0xFF;
    Version >>= 8;
    *VerMaj = Version & 0xFF;
    return ret;
}

BOOL Get_Time_Slice_Info (DWORD *SchedCount, DWORD *Cur_Sched, DWORD *Idle_Sched)
{
    VxDParams    p;
    BOOL        ret;

    p.CallNum = zGet_Time_Slice_Info;

    ret = VxDCall (&p);
    *SchedCount = p.OutEAX;
    *Cur_Sched  = p.OutEBX;
    *Idle_Sched = p.OutECX;
    return ret;
}
BOOL Get_Time_Slice_Priority (DWORD VMHandle, DWORD *Foreground, DWORD *Background,
                              DWORD *Usage, DWORD *Flags)
{
    VxDParams   p;
    BOOL        ret;

    p.CallNum = zGet_Time_Slice_Priority;
    p.InEBX = VMHandle;

    ret = VxDCall (&p);
    *Flags      = p.OutEAX;
    *Foreground = p.OutECX;
    *Background = p.OutEDX;
    *Usage      = p.OutESI;
    return ret;
}
