#define Generic_Dev_ID      0x28c0

#include "winexp.inc"
#include "winexp.dat"

/****************************************************************************
*
*   FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)
*
*   PURPOSE: calls initialization function, processes message loop
*
*   COMMENTS:
*
*       Windows recognizes this function by name as the initial entry point
*       for the program.  This function calls the application initialization
*       routine, if no other instance of the program is running, and always
*       calls the instance initialization routine.  It then executes a message
*       retrieval and dispatch loop that is the top-level control structure
*       for the remainder of execution.  The loop is terminated when a WM_QUIT
*       message is received, at which time this function exits the application
*       instance by returning the value passed by PostQuitMessage().
*
*       If this function must abort before entering the message loop, it
*       returns the conventional value NULL.
*
****************************************************************************/
int PASCAL WinMain (HANDLE hInstance, HANDLE hPrevInstance, LPSTR lpszcmdline,
                    int nCmdShow)
{
MSG         msg;
DWORD       Winlong;                    /* Temp version variable              */

    if (!hPrevInstance)
        InitApplication (hInstance);

    InitInstance (hInstance, nCmdShow);

    if ((VxDCallProc = Get_VxD_API ()) == 0L) {
        MessageBox (hMainWnd,
            "Cannot find the Generic VxD\r\n",
            szAppName,
            MB_ICONEXCLAMATION | MB_OK);
        exit (1);
        }

    Winlong = GetVersion ();
    WinVerMaj = ((WORD) Winlong) & 0xFF;
    Winlong >>= 8;
    WinVerMin = ((WORD) Winlong) & 0xFF;
    Get_VMM_Version (&VMMVerMaj, &VMMVerMin, &VMM_Debug_Ver);

    if (WinVerMaj != 3) {
        MessageBox (hMainWnd,
            "WARNING: This program is compatible with Windows version 3.x only"
            " and may not run under this version of Windows",
            szAppName,
            MB_ICONEXCLAMATION | MB_OK);
        }

    OS_Type = WINDOWS31_OS;
    VMHandle = Get_Sys_VM_Handle();
    if (VMM_Debug_Ver) OS_Type = WINDOWS31_DEBUG_OS;

    SetChoice (C_ACTIVE_VMS, 0, -1);

    while (GetMessage (&msg, NULL, 0, 0)) {
        TranslateMessage (&msg);
        DispatchMessage (&msg);
    }
    return msg.wParam;
}
/******************************************************************************/
/*                                                                            */
/*  This is the first instance of this application.                           */
/*  Define and register the Window Class.                                     */
/*  The inportant aspects are the attaching of the Window Function to this    */
/*    class of window and giving it a name.                                   */
/*                                                                            */
/******************************************************************************/
VOID InitApplication (HANDLE hInstance)
{
WNDCLASS    WndClass;

    WndClass.style          = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS; //Redraw if change to horz or vert size
    WndClass.lpfnWndProc    = WndProc;                 // Name of window function
    WndClass.cbClsExtra     = 0;
    WndClass.cbWndExtra     = 0;
    WndClass.hInstance      = hInstance;
    WndClass.hIcon          = LoadIcon (hInstance, "WinExp");
    WndClass.hCursor        = LoadCursor (NULL, IDC_ARROW); // Arrow Cursor
    WndClass.hbrBackground  = GetStockObject (WHITE_BRUSH);
    WndClass.lpszMenuName   = "VxDMenu";
    WndClass.lpszClassName  = szAppName;

    RegisterClass (&WndClass);
}

/******************************************************************************/
/*                                                                            */
/*  Create and display a window for this instance of the application.         */
/*  The Window Class has already been registered and named.  Use the          */
/*  name attached to the Window Class and display a window.                   */
/*                                                                            */
/******************************************************************************/
VOID InitInstance (HANDLE hInstance, int nCmdShow)
{
    char *locstr = NULL;
    hInst = hInstance;

    hMainWnd = CreateWindow (szAppName, szAppName,
                        WS_OVERLAPPEDWINDOW,
                        100, 100,
                        400, 200,
                        NULL, NULL, hInstance, NULL);

    ShowWindow (hMainWnd, nCmdShow);
    UpdateWindow (hMainWnd);
}

/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
/*                                                                            */
/*  This routine is called by Windows in response to some type of action.     */
/*  Windows will pass the type of action in the message parameter.            */
/*  wParm and lParm are extra information and are dependant on the type       */
/*    of command.                                                             */
/*                                                                            */
/*  Static Variable Usage:                                                    */
/*      cxChar = Average Width of lower case character                        */
/*      cyChar = Maximum heigth of a character                                */
/*      cxClient = Width of the Screen Client area                            */
/*      cyClient = Heigth of the screen client area                           */
/*                                                                            */
/*                                                                            */
/******************************************************************************/
long FAR PASCAL WndProc (HWND hwnd, WORD message, WORD wParm, LONG lParm)
{
static HPEN hPen;

HDC         hdc;
short       i, x, y, nPaintBeg, nPaintEnd;
short       index;
PAINTSTRUCT ps;
TEXTMETRIC  tm;
int         ScrollInc;
//char        buf[240];

    FARPROC lpProcAbout;                /* pointer to the "About" function    */
    HMENU   hMenu;

    switch  (message) {
        case  WM_CREATE :               /* A Window is being created          */
                                        /* Calculate the static variables     */
                                        /* required for this instance         */
            hdc = GetDC (hwnd);         /* Get the Device Context from the Window HAndle */
            SelectObject (hdc, GetStockObject (OEM_FIXED_FONT));
            GetTextMetrics (hdc, &tm);  /* Get text sizes                     */
            cxChar = tm.tmAveCharWidth; /* Average character Width            */
            cxCaps = (tm.tmPitchAndFamily & 1 ? 3 : 2) * cxChar / 2; /* Caps are 150% of cxChar */
            cyChar = tm.tmHeight + tm.tmExternalLeading;
            ReleaseDC (hwnd, hdc);
/*                                                                            */
/*  Create the Timer                                                          */
/*                                                                            */
            if (!SetTimer (hwnd, 1, 250, NULL)) {
                MessageBox (hwnd,
                    "Timer Count Exceeded, Timer not running",
                    szAppName,
                    MB_ICONEXCLAMATION | MB_OK);
                    exit (0);
                }
            VMInfo_Choice = C_ACTIVE_VMS;
            return 0;

        case  WM_VSCROLL :              /* Verticle Scrolling                 */
            switch (wParm) {
                case SB_TOP :
                    ScrollInc = -Scroll_Offset;
                    break;

                case SB_BOTTOM :
/*                                                                            */
/*  Scroll_Depth is the total number of lines that can be displayed for this  */
/*  type of information.                                                      */
/*  WindowHeight is the number of lines that can be displayed at one time     */
/*  NonScrollLines is the number of fixed lines on the screen                 */
/*                                                                            */
                    ScrollInc = (Scroll_Depth-(WindowHeight-NonScrollLines)) -
                                 Scroll_Offset;
                    break;

                case SB_LINEUP :
                    ScrollInc = -1;
                    break;

                case SB_LINEDOWN :
                    ScrollInc = 1;
                    break;

                case SB_PAGEUP :
                    ScrollInc = -(WindowHeight - NonScrollLines);
                    break;

                case SB_PAGEDOWN :
                    ScrollInc = WindowHeight - NonScrollLines;
                    break;

                case SB_THUMBTRACK :
                    ScrollInc = LOWORD (lParm) - Scroll_Offset;
                    break;

                default :
                    ScrollInc = 0;
                }
            SetScroll (Scroll_Depth, Scroll_Offset+ScrollInc);
            UpdateWatchWindow ();
            return 0;

        case WM_KEYDOWN:
            switch (wParm) {
                case VK_HOME : SendMessage (hwnd, WM_VSCROLL, SB_TOP, 0L); return 0;
                case VK_END  : SendMessage (hwnd, WM_VSCROLL, SB_BOTTOM, 0L); return 0;
                case VK_NEXT : SendMessage (hwnd, WM_VSCROLL, SB_PAGEDOWN, 0L); return 0;
                case VK_PRIOR: SendMessage (hwnd, WM_VSCROLL, SB_PAGEUP, 0L); return 0;
                case VK_UP   : SendMessage (hwnd, WM_VSCROLL, SB_LINEUP, 0L); return 0;
                case VK_DOWN : SendMessage (hwnd, WM_VSCROLL, SB_LINEDOWN, 0L); return 0;
                }
            break;

        case WM_COMMAND:
            hMenu = GetMenu (hwnd);
            switch (wParm) {
                case IDM_D_VMINFO:
                    VMInfo_Choice = LatestChoice;
                    VMInfo_Offset = Scroll_Offset;
                    VMInfo_Highlight = LastHighlight;
                    SetChoice (C_ACTIVE_VMS, 0, -1);
                    return 0;

                case IDM_D_TIMESLICE:
                    TS_Choice = LatestChoice;
                    TS_Offset = Scroll_Offset;
                    TS_Highlight = LastHighlight;
                    SetChoice (C_TIMESLICE, 0, -1);
                    return 0;

                case IDM_F_EXIT:
                    SendMessage (hwnd, WM_CLOSE, 0, 0L);
                    return 0;

                case IDM_ABOUT:
                    lpProcAbout = MakeProcInstance(About, hInst);

                    DialogBox(hInst,    /* current instance         */
                        "AboutWinExp",   /* resource to use          */
                        hMainWnd,       /* parent handle            */
                        lpProcAbout);   /* About() instance address */

                    FreeProcInstance(lpProcAbout);
                    return 0;
            }
            return 0;

        case WM_LBUTTONDBLCLK:
            SelectItem ((int) (HIWORD (lParm) / cyChar));
            return 0;

        case WM_LBUTTONDOWN:
            HighLightItem (Scroll_Offset-NonScrollLines+
                           (int) (HIWORD (lParm) / cyChar));
            return 0;

        case WM_RBUTTONDOWN:
            PreviousItem ();
            return 0;


        case WM_TIMER:
            if (++TimerBusy) {
                --TimerBusy;
                return 0;
                }
            HandleTimer ();
            --TimerBusy;
            return 0;

        case  WM_SIZE :                 /* The Window has been resized        */
            cxClient = LOWORD (lParm);  /* Heigth of Window                   */
            cyClient = HIWORD (lParm);  /* Width of Window                    */
            WindowHeight = cyClient/cyChar; /* Number of lines per window     */
            SetScroll (Scroll_Depth, Scroll_Offset);
            return 0;

        case  WM_PAINT :                /* Paint the screen                   */
            hdc = BeginPaint (hwnd, &ps);
            SelectObject (hdc, GetStockObject (OEM_FIXED_FONT));
            nPaintBeg = ps.rcPaint.top / cyChar; /* Starting line in window   */
            nPaintEnd = (ps.rcPaint.bottom+cyChar) / cyChar;
            x = cxChar;
            y = cyChar * nPaintBeg;
            index = nPaintBeg;
            for (i=nPaintBeg; i < nPaintEnd; ++i) {
                TextOut (hdc, x, y, OutputBuffer[index], lstrlen (OutputBuffer[index]));
                y += cyChar;
                ++index;
                }
            HighLightItem (LastHighlight);
            EndPaint (hwnd, &ps);
            return (0);

        case  WM_CLOSE :
            ++TimerBusy;                /* Prevent any updates                */
            break;

        case  WM_DESTROY :
            KillTimer (hwnd, 1);
//            WinHelp (hwnd, "vxdtest.hlp", HELP_QUIT, 0L);
            PostQuitMessage (0);
            return 0;
    }
    return DefWindowProc (hwnd, message, wParm, lParm);

}

/******************************************************************************/
/*                                                                            */
/*  Check the command directory for a new command file.                       */
/*  If found execute only one then return to the message loop                 */
/*                                                                            */
/******************************************************************************/
void HandleTimer ()
{
    UpdateWatchWindow ();
}

/*                                                                            */
/*  OUTPUT_ENTRIES - Maximum number of debug statements saved                 */
/*  OUTPUT_LENGTH  - Maximum length of a debug line                           */
/*  Output_Index   - Index of next entry to add (Oldest entry in table)       */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/*  Set the new selection for display                                         */
/*                                                                            */
/******************************************************************************/


void GetVMInfo (DWORD VMHandle, char *Name, WORD *VMId)
{
    VM_InfoBlock *IBptr;
    VM_ControlBlock *CBptr;
    int vm_offset = 0;                  /* Offset into VM for buffer          */
    int i;

    _asm {
        mov     ax, 7
        call    DWORD PTR VxDCallProc
        jc      VMInfo_Offset_okay
        mov     vm_offset, bx
VMInfo_Offset_okay:
        }

    if (!vm_offset) {
      MessageBox (hMainWnd,
        "Cannot obtain the offset of the VMINFO block.  You may have an old version of VxD.386.  This program requires 2.0 or greater",
        "Windows Explorer",
        MB_ICONEXCLAMATION | MB_OK);
        exit (1);
        }
/*                                                                            */
/*  Gain access to the VM Control block                                       */
/*                                                                            */
    IBptr = Map_Linear (VMHandle+(DWORD) vm_offset, sizeof (VM_InfoBlock));
    for (i=0; i<32; ++i) Name[i] = IBptr->AppName[i];
    Name[i] = '\0';
    Free_Mapped_Linear (IBptr);
    CBptr = Map_Linear (VMHandle, sizeof(VM_ControlBlock));
    *VMId = (WORD) (CBptr->CB_VMID);
    if (*VMId == 1) strcpy (Name,"System VM");
    Free_Mapped_Linear (CBptr);
}

DWORD Get_VxD_API ()
{
    DWORD   CallAddr = 0L;
/*                                                                            */
/*  Get the VxD device API call address                                       */
/*                                                                            */
    _asm {
        push    es
        mov     ax, 1684h
        mov     bx, Generic_Dev_ID
        int     2fh
        mov     ax, es
        or      ax, di
        jz      VxD_Done
        mov     WORD PTR CallAddr, di
        mov     WORD PTR CallAddr+2, es
VxD_Done:
        pop     es
        }
    return CallAddr;
}
/*                                                                            */
/*  item number is zero based                                                 */
/*                                                                            */
void HighLightItem (int itemnum)
{
    HDC hdc;

    ClearHighlight ();

    if (itemnum == -1) return;
    if (Scroll_Depth) if (itemnum >= Scroll_Depth-EndLines) return;

    LastHighlight = itemnum;

    if (OutOfVisibleRange(itemnum)) return;

    itemnum -= Scroll_Offset;
    itemnum += NonScrollLines;
    LastHighlightLine = itemnum;
    hdc = GetDC (hMainWnd);         /* Get the Device Context from the Window HAndle */
    SelectObject (hdc, GetStockObject (OEM_FIXED_FONT));
    SetBkColor (hdc, 0x00A00000);
    SetTextColor (hdc, 0x00EEEEEE);
    TextOut (hdc, cxChar, cyChar*itemnum, OutputBuffer[itemnum], lstrlen (OutputBuffer[itemnum]));
    ReleaseDC (hMainWnd, hdc);
}

void ClearHighlight ()
{

    HDC hdc;
    WORD    linenum;

    if (LastHighlightLine == -1) return;

    linenum = LastHighlightLine;
    LastHighlight = -1;
    LastHighlightLine = -1;


    hdc = GetDC (hMainWnd);         /* Get the Device Context from the Window HAndle */
    SelectObject (hdc, GetStockObject (OEM_FIXED_FONT));
    SetBkColor (hdc, 0x00FFFFFF);
    SetTextColor (hdc, 0x00000000);
    TextOut (hdc, cxChar, cyChar*linenum, OutputBuffer[linenum], lstrlen (OutputBuffer[linenum]));
    ReleaseDC (hMainWnd, hdc);
}
/*                                                                            */
/*  Display the string in the visible scroll range                            */
/*  Itemnum is the virtual item number relative to Scroll_Offset              */
/*                                                                            */
void VStringOut (int itemnum, char *szFormat, ...)
{
    char szBuffer[256];
    char *pArgs;


    if (OutOfVisibleRange (itemnum)) return;

    itemnum -= Scroll_Offset;
    itemnum += NonScrollLines;

    if (!szFormat)
        strcpy (szBuffer, " ");
    else {
        pArgs = (char *) &szFormat + sizeof szFormat;
        vsprintf (szBuffer, szFormat, pArgs);
        }
    xStringOut (itemnum, szBuffer);
}
/*                                                                            */
/*  Place the string into the display buffer at the given linenum             */
/*                                                                            */
void StringOut (int linenum, char *szFormat, ...)
{
    char szBuffer[256];
    char *pArgs;

    if (!szFormat)
        strcpy (szBuffer, " ");
    else {
        pArgs = (char *) &szFormat + sizeof szFormat;
        vsprintf (szBuffer, szFormat, pArgs);
        }
    xStringOut (linenum, szBuffer);

}
void xStringOut (int linenum, char *szBuffer)
{
    RECT    region;

    if (linenum > OUTPUT_ENTRIES) return;
    szBuffer [OUTPUT_LENGTH] = '\0';
/*                                                                            */
/*  If the line is outside the print region, don't worry about it             */
/*                                                                            */
    if (linenum > WindowHeight) {
        strcpy (OutputBuffer[linenum],szBuffer);
        return;
        }
/*                                                                            */
/*  Update the line only if it has changed                                    */
/*                                                                            */
    if (strcmp (szBuffer, OutputBuffer[linenum])) {
        strcpy (OutputBuffer[linenum],szBuffer);
        region.left   = 0;
        region.top    = cyChar * (linenum);
        region.right  = cxClient;
        region.bottom = cyChar * (linenum+1);
        InvalidateRect (hMainWnd, &region, TRUE);
        }
}

void ClearLastLine ()
{
    StringOut (WindowHeight, " ");
}
void ClearScrn ()
{
    int i;

    for (i=0; i<OUTPUT_ENTRIES; ++i) {
        StringOut (i, " ");
        }
    UpdateWindow (hMainWnd);
}
/****************************************************************************
*
*   FUNCTION: About(HWND, unsigned, WORD, LONG)
*
*   PURPOSE:  Processes messages for "About" dialog box
*
*   MESSAGES:
*
*       WM_INITDIALOG - initialize dialog box
*       WM_COMMAND    - Input received
*
*   COMMENTS:
*
*       No initialization is needed for this particular dialog box, but TRUE
*       must be returned to Windows.
*
*       Wait for user to click on "Ok" button, then close the dialog box.
*
****************************************************************************/

BOOL FAR PASCAL About(hDlg, message, wParam, lParam)
HWND hDlg;                                /* window handle of the dialog box */
WORD message;                             /* type of message                 */
WORD wParam;                              /* message-specific information    */
LONG lParam;
{
    switch (message) {
        case WM_INITDIALOG:                /* message: initialize dialog box */
            return (TRUE);

        case WM_COMMAND:                      /* message: received a command */
            switch (wParam) {
                case IDOK:              /* "OK" box selected?                 */
                case IDCANCEL:          /* System menu close command?         */
                    EndDialog(hDlg, TRUE);     /* Exits the dialog box        */
                    return (TRUE);
            }
    }
    return (FALSE);                           /* Didn't process a message    */
}
void fail (unsigned char *str)
{
    MessageBox (hMainWnd,
        str,
        szAppName,
        MB_ICONEXCLAMATION | MB_OK);
        exit (0);
}
