/*
 * WNKMSC.C
 *
 * Windows Kermit dialog box routines
 *
 * Copyright (c) 1990 by
 * William S. Hall
 * 3665 Benton Street, #66
 * Santa Clara, CA 95051
 *
 */

#define NOKANJI
#define NOMINMAX
#define NOATOM
#define NOSOUND
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>	
#include "ascii.h"
#include "wnkerm.h"
#include "wnkdlg.h"

#define MAXPATHLEN	128
static char *defstring = "*.*";		/* default file type */

/* local function declarations */
static	void	NEAR	krmProtocolInit(HWND hDlg);
static	void	NEAR	krmProtocolSet(HWND hDlg);
static	void	NEAR	krmPacketsInit(HWND hDlg);
static	void	NEAR	krmPacketsSet(HWND hDlg);
static	void	NEAR	krmSetCancelFlags(HWND hDlg);
static	void	NEAR	krmSendFileInit(HWND hDlg);
static	int	NEAR	krmCheckSelect(HWND hDlg);
static	void	NEAR	krmGetSendFileList(HWND hDlg, LONG lParam);
static	BOOL	NEAR	krmListSelect(HWND hDlg, LONG lParam);
static	void	NEAR	krmListBoxOK(HWND hDlg, LONG lParam);
static	void	NEAR	krmPathSelect(HWND hDlg, LONG lParam);
static	int	NEAR	GetCheckedRadioButton(HWND hDlg, WORD start, WORD end);
static  int	NEAR	GetComboboxSelection(HWND hDlg, WORD id, char * buffer);
static	int	NEAR	GetComboboxIndex(HWND hDlg, int id);
static  char*   NEAR    getnamestr(char *str, int len);

/*
 * krmOpenDlgBox
 *
 * Open a modal dialog box
 */
int FAR krmOpenDlgBox(HWND hWnd, FARPROC fpProc, WORD boxnum)
{

    FARPROC fp;
    BOOL result;

  /* make a proc instance for the about box window function */
    fp = MakeProcInstance(fpProc, Kermit.hInst);
  /* create a modal dialog box */
    result = DialogBox(Kermit.hInst, MAKEINTRESOURCE(boxnum),hWnd,fp);
    FreeProcInstance(fp);
    return result;

}

/*
 * krmProtocol
 *
 * This dialog box sets general parameters affecting the
 * Kermit protocol.
 */
BOOL FAR PASCAL krmProtocol(HWND hDlg, unsigned message, WORD wParam, LONG lParam)
{
    switch(message) {
	case WM_INITDIALOG:
	krmProtocolInit(hDlg);
	break;

	case WM_COMMAND:
	    switch(wParam) {
		case IDOK:
		    krmProtocolSet(hDlg);
		    EndDialog(hDlg, TRUE);
		    break;
		case IDCANCEL:
		    EndDialog(hDlg, FALSE);
		    break;
		default:
		    return FALSE;
	    }
	    break;
	default:
	    return FALSE;
    }
    return TRUE;
}

/*
 * krmProtocolSet
 *
 * In response to values chosen by the user, this
 * routine sets the various protocol data structures.
 * If requested, values are saved to WIN.INI
 */
static void NEAR krmProtocolSet(HWND hDlg)
{

    char szAppStr[80];
    char szKeyStr[80];
    char szValStr[80];

    KermParams.Bell = IsDlgButtonChecked(hDlg, IDD_KRM_BELL);
    KermParams.Timer = IsDlgButtonChecked(hDlg, IDD_KRM_TIMER);
    KermParams.DiscardPartialFile = IsDlgButtonChecked(hDlg, IDD_KRM_DISCARD);
    KermParams.FileWarning = IsDlgButtonChecked(hDlg, IDD_KRM_FILEWARNING);
    KermParams.verbose = IsDlgButtonChecked(hDlg, IDD_KRM_VERBOSE);

    KermParams.BlockCheckType = GetCheckedRadioButton(hDlg, 
					IDD_KRM_1BYTECHK,
					IDD_KRM_3BYTECHK)
				- IDD_KRM_1BYTECHK + 1;

    GetComboboxSelection(hDlg, IDD_KRM_RETRYLIMIT, szValStr);
    KermParams.RetryLimit = atoi(szValStr);

    GetComboboxSelection(hDlg, IDD_KRM_SENDDELAY, szValStr);
    KermParams.SendDelay = atoi(szValStr);

  /* user wants to save the values, so write them to WIN.INI */
    if (IsDlgButtonChecked(hDlg, IDD_KRM_SAVE)) {
	LoadString(Kermit.hInst,IDS_KRM_KERMIT,(LPSTR)szAppStr,sizeof(szAppStr));

        LoadString(Kermit.hInst, IDS_KRM_TIMER, szKeyStr, sizeof(szKeyStr));
        itoa(KermParams.Timer, szValStr, 10);
        WriteProfileString(szAppStr, szKeyStr, szValStr);

        LoadString(Kermit.hInst, IDS_KRM_VERBOSE, szKeyStr, sizeof(szKeyStr));
        itoa(KermParams.verbose, szValStr, 10);
        WriteProfileString(szAppStr, szKeyStr, szValStr);

        LoadString(Kermit.hInst, IDS_KRM_DISCARD, szKeyStr, sizeof(szKeyStr));
        itoa(KermParams.DiscardPartialFile, szValStr, 10);
        WriteProfileString(szAppStr, szKeyStr, szValStr);

        LoadString(Kermit.hInst, IDS_KRM_BELL, szKeyStr, sizeof(szKeyStr));
        itoa(KermParams.Bell, szValStr, 10);
        WriteProfileString(szAppStr, szKeyStr, szValStr);

        LoadString(Kermit.hInst, IDS_KRM_FILEWARN, szKeyStr, sizeof(szKeyStr));
        itoa(KermParams.FileWarning, szValStr, 10);
        WriteProfileString(szAppStr, szKeyStr, szValStr);

        LoadString(Kermit.hInst, IDS_KRM_BLOCKCHECKTYPE, szKeyStr, sizeof(szKeyStr));
        itoa(KermParams.BlockCheckType, szValStr, 10);
        WriteProfileString(szAppStr, szKeyStr, szValStr);

        LoadString(Kermit.hInst, IDS_KRM_RETRYLIMIT, szKeyStr, sizeof(szKeyStr));
        itoa(KermParams.RetryLimit, szValStr, 10);
        WriteProfileString(szAppStr, szKeyStr, szValStr);

        LoadString(Kermit.hInst, IDS_KRM_SENDDELAY, szKeyStr, sizeof(szKeyStr));
        itoa(KermParams.SendDelay, szValStr, 10);
        WriteProfileString(szAppStr, szKeyStr, szValStr);

	SendMessage((HWND)(-1), WM_WININICHANGE, 0, (LONG)(LPSTR)szAppStr);
    }
}

/*
 * krmProtocolInit
 *
 * Initialize the Kermit protocol settings
 * dialog box.
 */
static void NEAR krmProtocolInit(HWND hDlg)
{
    register int i;
    char buf[80];

    CheckDlgButton(hDlg, IDD_KRM_BELL, KermParams.Bell);
    CheckDlgButton(hDlg, IDD_KRM_TIMER, KermParams.Timer);
    CheckDlgButton(hDlg, IDD_KRM_DISCARD, KermParams.DiscardPartialFile);
    CheckDlgButton(hDlg, IDD_KRM_FILEWARNING, KermParams.FileWarning);
    CheckDlgButton(hDlg, IDD_KRM_VERBOSE, KermParams.verbose);
    CheckRadioButton(hDlg, IDD_KRM_1BYTECHK, IDD_KRM_3BYTECHK,
			   IDD_KRM_1BYTECHK + KermParams.BlockCheckType - 1);

    for (i = 1; i <= 63; i++)
	SendDlgItemMessage(hDlg, IDD_KRM_RETRYLIMIT, CB_INSERTSTRING,
				-1, (LONG)(LPSTR)itoa(i, buf, 10));
    SendDlgItemMessage(hDlg, IDD_KRM_RETRYLIMIT, CB_SELECTSTRING,
			-1, (LONG)(LPSTR)itoa(KermParams.RetryLimit, buf, 10));
				
    for (i = 0; i <= 63; i++)
	SendDlgItemMessage(hDlg, IDD_KRM_SENDDELAY, CB_INSERTSTRING,
				-1, (LONG)(LPSTR)itoa(i, buf, 10));
    SendDlgItemMessage(hDlg, IDD_KRM_SENDDELAY, CB_SELECTSTRING,
			-1, (LONG)(LPSTR)itoa(KermParams.SendDelay, buf, 10));
}

/*
 * krmPackets
 *
 * This dialog box sets the various Kermit packet parameters.
 */
BOOL FAR PASCAL krmPackets(HWND hDlg, unsigned message, WORD wParam, LONG lParam)
{
    switch(message) {
	case WM_INITDIALOG:
	krmPacketsInit(hDlg);
	break;

	case WM_COMMAND:
	    switch(wParam) {
		case IDOK:
		    krmPacketsSet(hDlg);
		    EndDialog(hDlg, TRUE);
		    break;
		case IDCANCEL:
		    EndDialog(hDlg, FALSE);
		    break;
		default:
		    return FALSE;
	    }
	    break;
	default:
	    return FALSE;
    }
    return TRUE;
}

/*
 * krmPacketsSet
 *
 * In response to values chosen by the user, this
 * routine sets the various packet data structures.
 * If requested, values are saved to WIN.INI
 */
static void NEAR krmPacketsSet(HWND hDlg)
{
    char szAppStr[80];
    char szKeyStr[80];
    char szValStr[80];

    sndparams.mark = (BYTE)GetComboboxIndex(hDlg, IDD_KRM_SENDMARK);
    rcvparams.mark = (BYTE)GetComboboxIndex(hDlg, IDD_KRM_RECEIVEMARK);
    sndparams.maxpktsize = (short)GetComboboxIndex(hDlg, IDD_KRM_SENDMAXSIZE)
			   + KRM_MINPACKETSIZE;
    rcvparams.maxpktsize = (short)GetComboboxIndex(hDlg, IDD_KRM_RECEIVEMAXSIZE)
			   + KRM_MINPACKETSIZE;
    sndparams.timeout = (short)GetComboboxIndex(hDlg, IDD_KRM_SENDTIMEOUT)
			   + KRM_MINTIMEOUT;
    rcvparams.timeout = (short)GetComboboxIndex(hDlg, IDD_KRM_RECEIVETIMEOUT)
			   + KRM_MINTIMEOUT;
    sndparams.padcount = (short)GetComboboxIndex(hDlg, IDD_KRM_SENDPADCOUNT);
    rcvparams.padcount = (short)GetComboboxIndex(hDlg, IDD_KRM_RECEIVEPADCOUNT);
    sndparams.padchar = (BYTE)GetComboboxIndex(hDlg, IDD_KRM_SENDPADCHAR);
    if (sndparams.padchar >= ' ')
	sndparams.padchar = DEL;
    rcvparams.padchar = (BYTE)GetComboboxIndex(hDlg, IDD_KRM_RECEIVEPADCHAR);
    if (rcvparams.padchar >= ' ')
	sndparams.padchar = DEL;
    sndparams.eol = (BYTE)GetComboboxIndex(hDlg, IDD_KRM_SENDEOL);
    rcvparams.eol = (BYTE)GetComboboxIndex(hDlg, IDD_KRM_RECEIVEEOL);
    sndparams.quote = (BYTE)(GetComboboxIndex(hDlg, IDD_KRM_SENDQUOTE) + '!');
    if (sndparams.quote >= '?')
	sndparams.quote += 30;
    rcvparams.quote = (BYTE)(GetComboboxIndex(hDlg, IDD_KRM_RECEIVEQUOTE) + '!');
    if (rcvparams.quote >= '?')
	rcvparams.quote += 30;

    if (IsDlgButtonChecked(hDlg, IDD_KRM_SAVE)) {
	LoadString(Kermit.hInst,IDS_KRM_KERMIT,(LPSTR)szAppStr,sizeof(szAppStr));

        LoadString(Kermit.hInst, IDS_KRM_SENDMARK, szKeyStr, sizeof(szKeyStr));
        itoa(sndparams.mark, szValStr, 10);
        WriteProfileString(szAppStr, szKeyStr, szValStr);

        LoadString(Kermit.hInst, IDS_KRM_RCVMARK, szKeyStr, sizeof(szKeyStr));
        itoa(rcvparams.mark, szValStr, 10);
        WriteProfileString(szAppStr, szKeyStr, szValStr);

        LoadString(Kermit.hInst, IDS_KRM_SENDMAXLEN, szKeyStr, sizeof(szKeyStr));
        itoa(sndparams.maxpktsize, szValStr, 10);
        WriteProfileString(szAppStr, szKeyStr, szValStr);

        LoadString(Kermit.hInst, IDS_KRM_RCVMAXLEN, szKeyStr, sizeof(szKeyStr));
        itoa(rcvparams.maxpktsize, szValStr, 10);
        WriteProfileString(szAppStr, szKeyStr, szValStr);

        LoadString(Kermit.hInst, IDS_KRM_SENDTIMEOUT, szKeyStr, sizeof(szKeyStr));
        itoa(sndparams.timeout, szValStr, 10);
        WriteProfileString(szAppStr, szKeyStr, szValStr);

        LoadString(Kermit.hInst, IDS_KRM_RCVTIMEOUT, szKeyStr, sizeof(szKeyStr));
        itoa(rcvparams.timeout, szValStr, 10);
        WriteProfileString(szAppStr, szKeyStr, szValStr);

        LoadString(Kermit.hInst, IDS_KRM_SENDPADCOUNT, szKeyStr, sizeof(szKeyStr));
        itoa(sndparams.padcount, szValStr, 10);
        WriteProfileString(szAppStr, szKeyStr, szValStr);

        LoadString(Kermit.hInst, IDS_KRM_RCVPADCOUNT, szKeyStr, sizeof(szKeyStr));
        itoa(rcvparams.padcount, szValStr, 10);
        WriteProfileString(szAppStr, szKeyStr, szValStr);

        LoadString(Kermit.hInst, IDS_KRM_SENDPADCHAR, szKeyStr, sizeof(szKeyStr));
        itoa(sndparams.padchar, szValStr, 10);
        WriteProfileString(szAppStr, szKeyStr, szValStr);

        LoadString(Kermit.hInst, IDS_KRM_RCVPADCHAR, szKeyStr, sizeof(szKeyStr));
        itoa(rcvparams.padchar, szValStr, 10);
        WriteProfileString(szAppStr, szKeyStr, szValStr);

        LoadString(Kermit.hInst, IDS_KRM_SENDEOL, szKeyStr, sizeof(szKeyStr));
        itoa(sndparams.eol, szValStr, 10);
        WriteProfileString(szAppStr, szKeyStr, szValStr);

        LoadString(Kermit.hInst, IDS_KRM_RCVEOL, szKeyStr, sizeof(szKeyStr));
        itoa(rcvparams.eol, szValStr, 10);
        WriteProfileString(szAppStr, szKeyStr, szValStr);

        LoadString(Kermit.hInst, IDS_KRM_SENDQUOTE, szKeyStr, sizeof(szKeyStr));
        itoa(sndparams.quote, szValStr, 10);
        WriteProfileString(szAppStr, szKeyStr, szValStr);

        LoadString(Kermit.hInst, IDS_KRM_RCVQUOTE, szKeyStr, sizeof(szKeyStr));
        itoa(rcvparams.quote, szValStr, 10);
        WriteProfileString(szAppStr, szKeyStr, szValStr);

	SendMessage((HWND)(-1), WM_WININICHANGE, 0, (LONG)(LPSTR)szAppStr);
    }
}

/*
 * krmPacketsInit
 *
 * Initialize the Kermit packets settings
 * dialog box.
 */
static void NEAR krmPacketsInit(HWND hDlg)
{

    register int i;
    char buf[80];

    for (i = 0; i < ' '; i++) {
	sprintf(buf, "CTRL-%c", '@'+i);
	SendDlgItemMessage(hDlg, IDD_KRM_SENDMARK, CB_INSERTSTRING,
			   -1, (LONG)(LPSTR)buf);
	SendDlgItemMessage(hDlg, IDD_KRM_RECEIVEMARK, CB_INSERTSTRING,
			   -1, (LONG)(LPSTR)buf);
    }
    SendDlgItemMessage(hDlg,IDD_KRM_SENDMARK, CB_SETCURSEL,sndparams.mark, 0L);
    SendDlgItemMessage(hDlg,IDD_KRM_RECEIVEMARK,CB_SETCURSEL,rcvparams.mark, 0L);

    for (i = KRM_MINPACKETSIZE; i <= KRM_MAXPACKETSIZE; i++) {
	itoa(i, buf, 10);
	SendDlgItemMessage(hDlg, IDD_KRM_SENDMAXSIZE, CB_INSERTSTRING,
			   -1, (LONG)(LPSTR)buf);
	SendDlgItemMessage(hDlg, IDD_KRM_RECEIVEMAXSIZE, CB_INSERTSTRING,
			   -1, (LONG)(LPSTR)buf);
    }
    SendDlgItemMessage(hDlg,IDD_KRM_SENDMAXSIZE, CB_SETCURSEL,
		       sndparams.maxpktsize - KRM_MINPACKETSIZE, 0L);
    SendDlgItemMessage(hDlg,IDD_KRM_RECEIVEMAXSIZE, CB_SETCURSEL,
		       rcvparams.maxpktsize - KRM_MINPACKETSIZE, 0L);

    for (i = KRM_MINTIMEOUT; i <= KRM_MAXTIMEOUT; i++) {
	itoa(i, buf, 10);
	SendDlgItemMessage(hDlg, IDD_KRM_SENDTIMEOUT, CB_INSERTSTRING,
			   -1, (LONG)(LPSTR)buf);
	SendDlgItemMessage(hDlg, IDD_KRM_RECEIVETIMEOUT, CB_INSERTSTRING,
			   -1, (LONG)(LPSTR)buf);
    }
    SendDlgItemMessage(hDlg,IDD_KRM_SENDTIMEOUT, CB_SETCURSEL,
		       sndparams.timeout - KRM_MINTIMEOUT, 0L);
    SendDlgItemMessage(hDlg,IDD_KRM_RECEIVETIMEOUT, CB_SETCURSEL,
		       rcvparams.timeout - KRM_MINTIMEOUT, 0L);

    for (i = 0; i <= KRM_MAXPADCOUNT; i++) {
	itoa(i, buf, 10);
	SendDlgItemMessage(hDlg, IDD_KRM_SENDPADCOUNT, CB_INSERTSTRING,
			   -1, (LONG)(LPSTR)buf);
	SendDlgItemMessage(hDlg, IDD_KRM_RECEIVEPADCOUNT, CB_INSERTSTRING,
			   -1, (LONG)(LPSTR)buf);
    }
    SendDlgItemMessage(hDlg,IDD_KRM_SENDPADCOUNT,CB_SETCURSEL,
		       sndparams.padcount, 0L);
    SendDlgItemMessage(hDlg,IDD_KRM_RECEIVEPADCOUNT,CB_SETCURSEL,
		       rcvparams.padcount, 0L);
    for (i = 0; i < ' '; i++) {
	sprintf(buf, "CTRL-%c", '@'+i);
	SendDlgItemMessage(hDlg, IDD_KRM_SENDPADCHAR, CB_INSERTSTRING,
			   -1, (LONG)(LPSTR)buf);
	SendDlgItemMessage(hDlg, IDD_KRM_RECEIVEPADCHAR, CB_INSERTSTRING,
			   -1, (LONG)(LPSTR)buf);
    }
    strcpy(buf, "CTRL ?");
    SendDlgItemMessage(hDlg, IDD_KRM_SENDPADCHAR, CB_INSERTSTRING,
			   -1, (LONG)(LPSTR)buf);
    SendDlgItemMessage(hDlg, IDD_KRM_RECEIVEPADCHAR, CB_INSERTSTRING,
			   -1, (LONG)(LPSTR)buf);
    SendDlgItemMessage(hDlg,IDD_KRM_SENDPADCHAR,CB_SETCURSEL,
		       sndparams.padchar != DEL ? sndparams.padchar : 32, 0L);
    SendDlgItemMessage(hDlg,IDD_KRM_RECEIVEPADCHAR,CB_SETCURSEL,
		       rcvparams.padchar != DEL ? rcvparams.padchar : 32, 0L);

    for (i = 0; i < 32; i++) {
	sprintf(buf, "CTRL-%c", '@'+i);
	SendDlgItemMessage(hDlg, IDD_KRM_SENDEOL, CB_INSERTSTRING,
			   -1, (LONG)(LPSTR)buf);
	SendDlgItemMessage(hDlg, IDD_KRM_RECEIVEEOL, CB_INSERTSTRING,
			   -1, (LONG)(LPSTR)buf);
    }
    SendDlgItemMessage(hDlg,IDD_KRM_SENDEOL, CB_SETCURSEL,sndparams.eol, 0L);
    SendDlgItemMessage(hDlg,IDD_KRM_RECEIVEEOL, CB_SETCURSEL,rcvparams.eol, 0L);

    for (i = 33; i <= 126; i++) {
	if ((i >= 63) && (i <= 95))
	    continue;
	sprintf(buf, "%c", i);
	SendDlgItemMessage(hDlg, IDD_KRM_SENDQUOTE, CB_INSERTSTRING,
			   -1, (LONG)(LPSTR)buf);
	SendDlgItemMessage(hDlg, IDD_KRM_RECEIVEQUOTE, CB_INSERTSTRING,
			   -1, (LONG)(LPSTR)buf);
    }
    SendDlgItemMessage(hDlg,IDD_KRM_SENDQUOTE, CB_SETCURSEL,
			(33 <= sndparams.quote) && (sndparams.quote < 63) ? 
			sndparams.quote - 33 : sndparams.quote - 63, 0L);
    SendDlgItemMessage(hDlg,IDD_KRM_RECEIVEQUOTE, CB_SETCURSEL,
			(33 <= rcvparams.quote) && (rcvparams.quote < 63) ? 
			rcvparams.quote - 33 : rcvparams.quote - 63, 0L);
    
}

/* 
 * krmSendFileProc
 *
 * This dialog box allows the user to choose a list of
 * files to send to a remote Kermit.
 */
BOOL FAR PASCAL krmSendFileProc(HWND hDlg,unsigned message,WORD wParam,LONG lParam)
{

    switch (message) {
	case WM_INITDIALOG:
	    krmSendFileInit(hDlg);
	    break;

	case WM_COMMAND:
	    switch (wParam) {

		/* turn off SEND button if no files selected */
		case IDD_KRM_LISTSELECT:
		    if (HIWORD(lParam) == LBN_SELCHANGE)
			EnableWindow(GetDlgItem(hDlg, IDD_KRM_SEND),
				      krmListSelect(hDlg, lParam));
		    break;

		/* in this dialog box, IDOK sets selections */
		case IDOK:		    
		    krmListBoxOK(hDlg, lParam);
		    EnableWindow(GetDlgItem(hDlg, IDD_KRM_SEND),
				 krmListSelect(hDlg, lParam));
		    break;
		
		case IDCANCEL:
		    EndDialog(hDlg, FALSE);
		    break;

		/* get selected files into a buffn utilizar en sus ficheros .BAT. Tiene un generador aleatorio de efectos y tambin del tiempo de duracin. Tiene pantallas de ayuda
!Smith, Dave
$0
@SOUNDER.ZIP     16296  07-07-93  
*Sounder 3.1
Produces sonidos aleatorios que se pueden utilizar en sus ficheros .BAT. Vd. puede especificar la frecuencia y la duracin
!Steenburgh, Chuck
$5
@SPARKL31.ZIP    75013  10-29-93  
*Sparkle & Menu Magic 3.1
Le permite crear muchos efectos para ficheros .BAR. Soporta 250 tipos diferentes de recuadros, ms de 35 efectos de sonido, ratn y muchas ms opciones
!Smetana, Rob W.
$39
@SPLIT.ZIP        1875  06-22-89  
*Sqwez 2.0
Programa para dividir ficheros en otros ms pequeos para que entren en diskettes y despus puede volver a unirlos. 
!Jim Miller
$25
@STRIKE.ZIP       9619  09-18-93  
*Strike
Pronuncia la frase strike a key when ready en su altavoz
!Cravener, William
$0
@SW_46.ZIP       68352  11-16-93  
*Screen Wizard 4.6
Es un programa de ventanas para utilizarse en ficheros .BAT. Se pueden crear las pantallas como se quiera, tiene pausa, colores de fondos y utilidades para discos
!Small, Warren
$18
@VGACLR.ZIP      70299  06-08-93  
*VGA Clear Screen
Es un conjunto de 28 utilidades para limpiar pantallas VGA. Incluye puertas, estrellas, pelotas y muchas ms. Le hace falta EGA/VGA
!Smith, David
$0
@XSET.ZIP       141897  01-17-94  
*XSet 4.0. 
Le permite manejar las variable del entorno del DOS e introducir datos. Puede hacer clculos en puntos glotantes, puede hacer llamadas de alto nivel de entrada y salida. Es compatibles con DOS, DR. DOS, NDOS y 4D0S. 
!Marc Stern 
$3
&\UTIL_DOS\BATCH1
@2DRIVE.ZIP      11961  02-03-94  
*What_Drv 1.0. 
Coloca en la variable errorlevel el disco en que nos encontramos. Esto le permit cambiar de disco. 
!John Palm
$?
@APPLY_JL.ZIP    13301  04-25-94  
*Apply 1.01. 
Le permite crear comandos y utilizarlos con un grupo de ficheros. Tiene un lenguaje de macros para el proceso de los mismo. 
!Jean S. LaRochelle
$10
$?
@ATBAT101.ZIP    54688  04-04-93  
*AtBat 1.01. 
Le permite crear y ejecutar ficheros BAT de una lista de ficheros, cada uno de los cuales est asociado con un comando del DOS o con un texto. Los ficheros son seleccionados por tamao, fecha, atributos o contenido. 
!Softwest 
$15
$?
@BATCHES.ZIP     55457  01-18-94  
*Batches 1.0. 
Son 4 utilidades para los ficheros .BAT. T-PAUSE permite para el proceso durante un tiempo sin tener que pulsar una tecla, KEY# devuelve el valor ASCII de la tecla pulsada en ERRORLEVEL, SAVER sirve para guardar pantalla y SHOWERR comprueba los errores
$?
@BATMASTR.ZIP   261381  03-09-94  
*BatMaster
Es un editor que en principio fue creado para el mantenimiento de ficheros .BAT, puede acceder al AUTOEXEC.BAT y al CONFIG.SYS. Tiene un ayuda en lnea que le permite acceder a la informacin de los CONFIG.SYS y los comandos batch del DOS. soporta ratn
!Newlin, John J.
$25
@BATMNU.ZIP      46052  03-04-94  
*BatMnu 2.3
Es un men para utilizarse con ficheros .BAT. Los mens pueden tener hasta 26 opciones diferentes
!Warren Small
$18
@BMATH1.ZIP      34023  05-25-94  
*BatMath 1.0
Puede ejecutar operaciones matemticas dentro de ficheros .BAT
!Renato Francisco
$0
@BOXES.ZIP        1285  08-10-94  
*Boxes 1.0
Es un fuente en Clipper para sacar cajas de mensajes.
!Kevin S. Gallagher
$0
@BTMK26.ZIP     382348  08-19-93  
*BatchMaker Plus 2.6. 
Es un shell para crear y manejar ficheros .BAT. 
!Northland Multisoft
$28
@BTX.ZIP        198605  03-05-94  
*BTX
Es una utilidad para crear y ejecutar ficheros .BAT. Soporta ratn, mens, mensajes en ventanas, claves y muchas ms opciones
!KASE
$15
@CHKSPACE.ZIP    13247  05-16-94  
*ChkSpace 1.0
Comprueba el espacio libre en el disco y devuelve el mismo en ERRORLEVEL. Se puede utilizar en ficheros BAR.
!Wayne Breivogel
$10
@CLEAV157.ZIP    41800  12-07-93  
*Cleave 1.57
Es un programa para reemplazar el CLS del DOS.
!Samuel Kaplin
$6
@COMPDATE.ZIP     7275  04-25-94  
*CompDate 1.04. 
Es un programa, para fichero BAT, que permite comprobar la fecha de 2 ficheros. Se incluye el fuente en PASCAL. 
!Radiodetection Corp
$?
@DAYWEEK.ZIP      1882  04-25-94  
*Day Of Week
Muestra el da de la semana de la fecha del computador. Devuelve el valor en ERRORLEVEL el 0 es el domingo. Se incluye el fuente y el ejecutable.
!Dave, Jr. Navarro
$0
@DIRTEST.ZIP     30494  03-18-94  
*DIRTest 1.0.
Le permite comprobar la existencia de un directorio desde un BAT.
!Chris Sanders
$?
@DO.ZIP          26839  12-05-90  
*Do 4.1
Manejador de ficheros binarios y de texto desde la lnea de mandatos del DOS.
!Larrie V. Hutton
$29
@DOSVAR.ZIP      59068  06-09-94  
*DosVar 1.01a
Le permite modificar las variables de los ficheros .BAT.
!Timothy Campbell
$0
@DOSVER.ZIP       4910  10-11-91  
*DOSVer
Colocar en errorlevel la versin del DOS que est corriendo. Es incluye el cdigo fuent en QB. 
!John de Palma
$?
@DR_BAK.ZIP      27342  09-09-93  
*DR_Bak
Es un programa que le permite guardar el directorio actual y restaurarlo despus desde los ficheros .BAT. Se incluye el fuente en QB
!John De Palma
$?
@DSKCHK3.ZIP      3172  02-11-94  
*DskChk 3.0. 
Puede comprobar desde un fichero .BAT los discos que se encuentran disponibles y si se puede escribir o no en ellos 
!Richard N. Wisan
$?
@ECO.ZIP          3068  07-18-89  
*EcoMaster 2.04
Simulador ecolgico que le dice que animales sobrevivirn a una serie de acontecimientos.
!Steve Estvanik
$20
@EQO10.ZIP       16022  03-11-94  
*EQO 1.0. 
Es un programa para reemplazar el ECHO del DOS
$?
@ERR.ZIP          8701  09-15-93  
*Err. 
Es un programa TSR que le informa de los ERRORLEVELS devueltos por otros programas
$?
@ESCDEM.ZIP      11865  03-18-94  
*EscRun 1.01. 
Para el AUTOEXEC.BAT durante un nmero de segundos. 
!Paradigm Software
$10
@EXTBATCH.ZIP   322908  03-09-94  
*Extended Batch Language-Plus
Es un programa para crear mens y ficheros .BAT. Tiene ms de 100 funciones diferentes. Soporta ratn, sonido y muchas ms opciones
!Seaware Corp. 
$82
@FDATE_SF.ZIP    66492  05-11-94  
*Fdate - SF 8.4A
Le permite hacer operaciones aritmticas con la fecha y la hora en ficheros BAT. Nuevo: se han adicionado funciones de string, convertir los nmeros a hexadecimal y muchas otras. 
!Stephen Ferg
$0
@FDATESF.ZIP     63230  07-22-92  
*Fdate 8.2a
Le permite manejar la fecha y la hora desde sus ficheros .BAT
!Ferg, Stephen
$0
@FEED.ZIP         6161  06-22-94  
*Feed
Programa para el manejo de .BAT
!Jim  Scarletta
$0
@FJTUTOR.ZIP    316740  04-13-94  
*Fr. John's. 
Es un tutor de ficheros BAT que tiene 10 lecciones y 50 ejemplos de como escribir ficheros BAT tanto simples como complejos. Est escrito por orden alfabtico y a colores. Le hace falta 720k de espacio en disco. 
!John Robinson
$33-45
@FREAK125.ZIP    28449  12-07-93  
*Freak 1.25
Es un programa para reemplazar el CLS del DOS.
!Samuel Kaplin
$6
@GET26UPD.ZIP    31272  05-14-94  
*Get 2.61
Adiciona nuevos mandatos a los ficheros .BAT
!Bob Stephan
$16
@GETCOLOR.ZIP     3217  04-28-94  
*GetColor 2.0
Detecta y guarda pantallas al principio de un BAT y las restaura al terminar
!Richard B. Simpson
$0
@GETSET.ZIP      21463  02-10-94  
*Get Set 1.24
Le permite introducir datos en los programas que se ejecutan desde un fichero .BAT
!Robert S. Webber
$0
@IC.ZIP          11939  04-13-94  
*Internal Commands 2.2. 
Es un TSR de 18k que adiciona comandos al DOS, la mayora se pueden utilizar desde ficheros BAR.
!Geoff Friesen
$15
@IMPL155.ZIP     28510  04-28-94  
*Implode 1.55
Es un programa para reemplazar el comando CLS del DOS.
!Samuel Kaplin
$6
@INPUTX.ZIP      41706  02-09-94  
*Input 3.2. 
Le permite crear sus propios Prompt$.  
!Tay-Jee Software
$20
@LOGIT30.ZIP     32913  09-15-93  
*LOGit 3.0. 
Le permite introducir mandatos en un fichero .BAT.
!George Spafford
$?
@LPTRST.ZIP       6052  06-24-94  
*LPTRST 1.2
Arranca el puerto paralelo y coloca el resultado en ERRORLEVEL.
!Jeff Bankston
$0
@MENU_130.ZIP    57249  08-31-93  
*Menu-Man 1.3.
Es un men con ficheros BAR.
!Micrometric
$25
@PATH_114.ZIP    32663  08-31-93  
*Path_Man 1.1. 
Le permite buscar en los directorios definidos en el PATH. 
!Micrometric
$?
@PAUSE7.ZIP       3450  05-19-94  
*Pause7
Es una pausa de 7 segundos, que le permite interrumpir los ficheros BAR, comprobar el errorlevel.
!Jerry Wightman
$0
@PCEXEC.ZIP      56582  03-09-94  
*PCExec
Es un programa para reemplazar a los .BAT del DOS. Tiene muchos ms comando que los BAT. Se incluyen mucho ejemplos, y tiene 80k de documentacin
!LPTS Software Systems Inc.
$45
@POWERBAT.ZIP   271186  06-22-94  
*PowerBatch 2.0
Es un lenguaje para programar y compilar para DOS. No compila los BAT existentes, pero soporta los comandos de los bat. Tiene 40 mandatos nuevos, se pueden crear mens y muchas ms opciones
!Computing Systems Design, Inc.
$30
@PRNSTAT.ZIP      1488  05-19-94  
*PrnStat
Determina si el puerto LPT1 est disponibles. El resultado es devuelto en ERRORLEVEL. Se incluye un BAR y el fuente en asembler.
!Scott Computer Services
$10
@PRTSC.ZIP         762  11-19-93  
*PrtSc
Permite imprimir la pantalla por medio de un fichero .BAT
!Stanley R. Weiss
$0
@PSEDIT.ZIP      67926  02-18-94  
*PSEdit 3.6
Es un editor de ficheros de texto.
!Gary C. Crider
$24
@SCR.ZIP        119719  03-09-94  
*SCR
Es un lenguaje para programa los .BAT. Soporta 80 mandatos diferentes que incluye control de la pantalla, colores, posicin, cajas, sonido y muchas ms opciones. Incluye un editor de BAT
!David Arnold
$35
@SDS10B.ZIP       9008  09-16-93  
*S3 Directory Storage 1.0b 
Guarda la unidad de disco y el nombre del directorio en un fichero para que se pueda restaurar ms tarde
!George Spafford
$5
@SEBFU.ZIP      199211  03-09-94  
*SEBFU
Es un conjunto de utilidades para ficheros. Tiene 100 mandatos y funciones para comprobar el estado del teclado, la entrada del mismo, comprobar el estado del disco, comprobar la disponibilidad de la impresora, la hora y la fecha, la posicin del cursor, los colores de la pantalla y muchas ms opciones
!Scanlon Enterprises
$20
@SOSDFU.ZIP      62800  11-16-93  
*DOS Tool Box Utils
Es una coleccin de 13 mandatos para ficheros .BAT
!Solid Oak Software
$29-39
@SOSTDU.ZIP      37715  11-17-93  
*DOS Tool Box
Es una coleccin de 13 utilidades para ficheros .BAT. Las utilidades devuelven valores en ERRORLEVEL
!Solid Oak Software
$29
@SPAWN.ZIP       15396  05-04-94  
*Spawn
Retorna los ERRORLEVELS del DOS para depurar programas.
!Roger Morris
$0
@SSTUFF30.ZIP   259202  03-18-94  
*Steenburgh's Stuff 3.01. 
Es un conjunto de utilidades para ficheros BAT. 
!Tay J